@@ -1,6 +1,6 @@
-// $Id: Main.cpp,v 1.43 2004/05/22 01:24:16 geuzaine Exp $
+// $Id: Main.cpp,v 1.47 2005/02/16 05:17:54 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -126,7 +126,7 @@ int main(int argc, char *argv[])
     if(TheBgmFileName) {
       MergeProblem(TheBgmFileName);
       if(List_Nbr(CTX.post.list))
-        BGMWithView((Post_View *)
+        BGMWithView(*(Post_View **)
                     List_Pointer(CTX.post.list, List_Nbr(CTX.post.list) - 1));
       else{
         fprintf(stderr, ERROR_STR "Invalid background mesh (no view)\n");
@@ -184,6 +184,7 @@ void Msg(int level, char *fmt, ...)
 
   switch (level) {
 
+  case PROGRESS:
   case STATUS1N:
   case STATUS2N:
   case STATUS3N:
@@ -272,3 +273,19 @@ double Cpu(void)
   GetResources(&s, &us, &mem);
   return (double)s + (double)us / 1.e6;
 }
+
+// interactive value dialog
+
+double GetValue(char *text, double defaultval)
+{
+  if(CTX.nopopup)
+    return defaultval;
+
+  printf("%s (default=%.16g): ", text, defaultval);
+  char str[256];
+  fgets(str, sizeof(str), stdin);
+  if(!strlen(str) || !strcmp(str, "\n"))
+    return defaultval;
+  else
+    return atof(str);
+}
