@@ -1,7 +1,7 @@
 #ifndef _COLORTABLE_H_
 #define _COLORTABLE_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,46 +21,47 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #define COLORTABLE_NBMAX_PARAM 10
-#define COLORTABLE_NBMAX_COLOR 255
+#define COLORTABLE_NBMAX_COLOR 1024
 
 typedef struct{
   unsigned int table[COLORTABLE_NBMAX_COLOR];
   int size; // must be >= 2
   int ipar[COLORTABLE_NBMAX_PARAM];
-  float fpar[COLORTABLE_NBMAX_PARAM];
+  double dpar[COLORTABLE_NBMAX_PARAM];
 }GmshColorTable;
 
-
 // COLORTABLE_MODE
 
 #define COLORTABLE_RGB  1
 #define COLORTABLE_HSV  2
 
-
 // integrer parameters indices
 
 #define COLORTABLE_NUMBER    0  // predefined curve index
-#define COLORTABLE_CHANGED   1  // did the colortable change ?
-#define COLORTABLE_INVERT    2  // invert (rbg<->255-rgb)
-#define COLORTABLE_SWAP      3  // swap (min<->max)
-#define COLORTABLE_ROTATE    4  // rotate
-#define COLORTABLE_MODE      5  // mode (rgb, hsv)
+#define COLORTABLE_INVERT    1  // invert (rbg<->255-rgb)
+#define COLORTABLE_SWAP      2  // swap (min<->max)
+#define COLORTABLE_ROTATION  3  // rotation
+#define COLORTABLE_MODE      4  // mode (rgb, hsv)
 
-// float parameters indices
+// double parameters indices
 
-#define COLORTABLE_CURVE     0  // curvature
+#define COLORTABLE_CURVATURE 0  // curvature
 #define COLORTABLE_BIAS      1  // offset
-#define COLORTABLE_ALPHAPOW  2  // alpha channel power
-#define COLORTABLE_ALPHAVAL  3  // alpha channel value
-#define COLORTABLE_BETA      4  // beta coeff for brighten
+#define COLORTABLE_ALPHA     2  // alpha channel value
+#define COLORTABLE_BETA      3  // beta coeff for brighten
+#define COLORTABLE_ALPHAPOW  4  // alpha channel power value
 
-void ColorTable_InitParam (int number, GmshColorTable * ct);
-void ColorTable_Recompute (GmshColorTable * ct);
+void ColorTable_InitParam(int number, GmshColorTable *ct);
+void ColorTable_Recompute(GmshColorTable *ct);
 void ColorTable_Copy(GmshColorTable *ct);
 void ColorTable_Paste(GmshColorTable *ct);
 void ColorTable_Print(GmshColorTable *ct, FILE *fp) ;
 int  ColorTable_IsAlpha(GmshColorTable *ct) ;
-int  ColorTable_SetAlpha(GmshColorTable * ct, double alpha);
-int  ColorTable_Diff(GmshColorTable * ct1, GmshColorTable * ct2);
+int  ColorTable_Diff(GmshColorTable *ct1, GmshColorTable *ct2);
+
+void RGB_to_HSV(double R, double G, double B,
+		double *H, double *S, double *V);
+void HSV_to_RGB(double H, double S, double V,
+		double *R, double *G, double *B);
 
 #endif
