@@ -1,7 +1,7 @@
 #ifndef _COMMAND_LINE_H_
 #define _COMMAND_LINE_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ extern char gmsh_progname[], gmsh_copyright[], gmsh_version[], gmsh_os[];
 extern char gmsh_date[], gmsh_host[], gmsh_packager[], gmsh_url[];
 extern char gmsh_email[], gmsh_gui[], gmsh_options[], gmsh_license[];
 
-extern char *TheFileNameTab[MAX_OPEN_FILES], *TheBgmFileName, *TheOptString;
+extern char *TheFileNameTab[MAX_OPEN_FILES], *TheBgmFileName;
 
 void Get_Options(int argc, char *argv[], int *nbfiles);
 void Print_Usage(char *name);
