@@ -1,7 +1,7 @@
 #ifndef _CONTEXT_H_
 #define _CONTEXT_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -78,6 +78,7 @@ public :
   int opt_position[2];        // position of the option window on the screen
   int vis_position[2];        // position of the visibility window on the screen
   int clip_position[2];       // position of the clipping planes window on the screen
+  int manip_position[2];      // position of the manipulator window on the screen
   int stat_position[2];       // position of the statistics window on the screen
   int ctx_position[2];        // position of the geo/mesh context windows on the screen
   int solver_position[2];     // position of the solver windows on the screen
@@ -87,15 +88,13 @@ public :
   int batch;                  // 0=full gfx; -1=just parse; 1,2,3=batch 1D, 2D, 3D mesh 
   int initial_context;        // 0=automatic; 1=geom; 2=mesh; 3=solver; 4=post 
   int verbosity;              // 0=silent -> 3=debug 
+  int nopopup;                // never popup dialogs in scripts (use default values instead)
 
   double rot[16];             // current rotation matrix 
-  double mod[16];             // current modelview matrix 
-  double proj[16];            // current projection matrix
-  double r[3];                // position angles (if succ. rot. along x, y and z) 
+  double r[3];                // current Euler angles (in degrees!) 
   double t[3], s[3];          // current translation and scale 
-  int rlock[3], tlock[3], slock[3];
-                              // locks for r, t and s 
-  double quaternion[4];       // the actual quaternion used for "trackball" rotating 
+  double clip_factor;         // clipping plane distance factor
+  double quaternion[4];       // current quaternion used for "trackball" rotation
   int useTrackball;           // do or do not use the trackball for rotations 
   double rotation_center[3];  // point around which to rotate the scene
   int rotation_center_cg;     // rotate around the center of mass instead of rotation_center[]
@@ -103,18 +102,19 @@ public :
   double max[3];              // x, y and z max for the current geometry 
   double cg[3];               // "center of mass" of the current geometry
   double range[3];            // maximum range in the three directions 
-  double lc, lc_middle;       // characteristic lengths for the whole problem, 
-  double lc_order;            // and never used in mesh generation (->only for geo/post) 
+  double lc;                  // characteristic length for the whole problem (never
+                              // used in mesh generation ->only for geo/post) 
 
   int db;                     // double buffer? 
   int ortho;                  // orthogonal projection? 
   int draw_bbox;              // always draw the bounding boxes?
   int fast_redraw;            // only draw the bbox when rotating/panning the model?
-  int axes, small_axes;       // draw axes? 
-  int small_axes_pos[2];      // small axes position 
+  int small_axes, small_axes_size, small_axes_pos[2]; // small axes
+  int axes, axes_auto_position, axes_tics[3]; // large axes
+  double axes_position[6];  
+  char axes_label[3][256], axes_format[3][256];
   int threads, threads_lock;  // threads?, lock (should be a mutex...) 
   int alpha;                  // enable alpha blending 
-  int fake_transparency;      // use fake transparency (a la xpost)
   double zoom_factor;         // mouse2 zoom coefficient
 
   int fontsize;               // font size for fltk UI
@@ -131,6 +131,8 @@ public :
   int render_mode;            // GMSH_RENDER, GMSH_SELECT, GMSH_FEEDBACK 
   int clip[6];                // status of clip planes (bit arrays)
   double clip_plane[6][4];    // clip planes 
+  int polygon_offset, polygon_offset_always; // polygon offset control
+  double polygon_offset_factor, polygon_offset_units; // params for glPolygonOffset
   double pixel_equiv_x, pixel_equiv_y ; 
                               // approximative equivalent model length of a pixel 
   int color_scheme ;          // general color scheme
@@ -141,6 +143,9 @@ public :
   double arrow_rel_stem_radius;  // relative radius of arrow stem
   double arrow_rel_stem_length;  // relative length of arrow stem
 
+  double model[16], proj[16]; // the modelview and projection matrix as they were
+                              // at the time of the last InitPosition() call
+
   // geometry options 
   struct{
     int vis_type;
@@ -205,6 +210,7 @@ public :
   // solver options 
   struct{
     int max_delay, plugins ;
+    char *socket_name ;
   }solver;
 
   // print options 
@@ -213,7 +219,7 @@ public :
     int eps_quality, eps_background, eps_compress, eps_ps3shading;
     int eps_occlusion_culling, eps_best_root;
     double eps_line_width_factor, eps_point_size_factor;
-    int jpeg_quality;
+    int jpeg_quality, jpeg_smoothing;
     int gif_dither, gif_sort, gif_interlace, gif_transparent;
     int gl_fonts;
   } print;
@@ -237,9 +243,11 @@ public :
   } color;
   
   // trackball functions 
-  void buildRotmatrix(void);
-  void setQuaternion (double p1x, double p1y, double p2x, double p2y);
-  void addQuaternion (double p1x, double p1y, double p2x, double p2y);
+  void buildRotationMatrix(void);
+  void setQuaternion(double p1x, double p1y, double p2x, double p2y);
+  void addQuaternion(double p1x, double p1y, double p2x, double p2y);
+  void setQuaternionFromEulerAngles(void);
+  void setEulerAnglesFromRotationMatrix(void);
 };
 
 #endif
