@@ -1,7 +1,7 @@
 #ifndef _DEFAULT_OPTIONS_H_
 #define _DEFAULT_OPTIONS_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +40,19 @@
 // STRINGS
 
 StringXString GeneralOptions_String[] = {
+  { F|O, "AxesFormatX" , opt_general_axes_format0 , "%.3g" , 
+    "Number format for X-axis (in standard C form)" },
+  { F|O, "AxesFormatY" , opt_general_axes_format1 , "%.3g" , 
+    "Number format for Y-axis (in standard C form)" },
+  { F|O, "AxesFormatZ" , opt_general_axes_format2 , "%.3g" , 
+    "Number format for Z-axis (in standard C form)" },
+  { F|O, "AxesLabelX" , opt_general_axes_label0 , "" , 
+    "X-axis label" },
+  { F|O, "AxesLabelY" , opt_general_axes_label1 , "" , 
+    "Y-axis label" },
+  { F|O, "AxesLabelZ" , opt_general_axes_label2 , "" , 
+    "Z-axis label" },
+
   { F|O, "DefaultFileName" , opt_general_default_filename , "untitled.geo" ,
     "Default project file name" },
   { F,   "Display" , opt_general_display , "" ,
@@ -48,6 +61,9 @@ StringXString GeneralOptions_String[] = {
   { F|O, "ErrorFileName" , opt_general_error_filename , ".gmsh-errors" ,
     "File into which the log is saved if a fatal error occurs" },
 
+  { F,   "FileName" , opt_general_filename , "" , 
+    "Current project file name (read-only)" },
+
   { F|O, "GraphicsFont" , opt_general_graphics_font , "Helvetica" ,
     "Font used in the graphic window" }, 
 
@@ -99,11 +115,14 @@ StringXString MeshOptions_String[] = {
 } ;
 
 StringXString SolverOptions_String[] = {
+  { F|O, "SocketName" , opt_solver_socket_name , ".gmshsock" ,
+    "Name of socket (TCP/IP if it contains the `:' character, UNIX otherwise)" },
+
   { F|O, "Name0" , opt_solver_name0 , "GetDP" ,
     "Name of solver 0" },
   { F|O, "Help0" , opt_solver_help0 , 
     "A General environment for the treatment of\nDiscrete Problems\n\n"
-    "Copyright (C) 1997-2004\nPatrick Dular and Christophe Geuzaine\n\n"
+    "Copyright (C) 1997-2005\nPatrick Dular and Christophe Geuzaine\n\n"
     "Visit http://www.geuz.org/getdp/ for more info",
     "Help string for solver 0" },
   { F|O, "Executable0" , opt_solver_executable0 , 
@@ -360,18 +379,34 @@ StringXString PostProcessingOptions_String[] = {
 } ;
 
 StringXString ViewOptions_String[] = {
-  { F|O, "AbscissaName" , opt_view_abscissa_name , "" , 
-    "Abscissa name for 2D graphs" },
-  { F|O, "AbscissaFormat" , opt_view_abscissa_format , "%.3e" , 
-    "Abscissa number format for 2D graphs (in standard C form)" },
+  { F|O, "AxesFormatX" , opt_view_axes_format0 , "%.3g" , 
+    "Number format for X-axis (in standard C form)" },
+  { F|O, "AxesFormatY" , opt_view_axes_format1 , "%.3g" , 
+    "Number format for Y-axis (in standard C form)" },
+  { F|O, "AxesFormatZ" , opt_view_axes_format2 , "%.3g" , 
+    "Number format for Z-axis (in standard C form)" },
+  { F|O, "AxesLabelX" , opt_view_axes_label0 , "" , 
+    "X-axis label" },
+  { F|O, "AxesLabelY" , opt_view_axes_label1 , "" , 
+    "Y-axis label" },
+  { F|O, "AxesLabelZ" , opt_view_axes_label2 , "" , 
+    "Z-axis label" },
 
   { F,   "FileName" , opt_view_filename , "" , 
     "Default post-processing view file name" },
-  { F|O, "Format" , opt_view_format , "%.3e" , 
+  { F|O, "Format" , opt_view_format , "%.3g" , 
     "Number format (in standard C form)" },
 
+  { F|O, "GeneralizedRaiseX" , opt_view_gen_raise0 , "v0" , 
+    "Generalized elevation of the view along X-axis (in model coordinates)" },
+  { F|O, "GeneralizedRaiseY" , opt_view_gen_raise1 , "v1" , 
+    "Generalized elevation of the view along Y-axis (in model coordinates)" },
+  { F|O, "GeneralizedRaiseZ" , opt_view_gen_raise2 , "v2" , 
+    "Generalized elevation of the view along Z-axis (in model coordinates)" },
+
   { F,   "Name" , opt_view_name , "" , 
     "Default post-processing view name" },
+
   { 0, NULL , NULL , NULL , NULL }
 } ;
 
@@ -391,7 +426,27 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "ArrowStemRadius" , opt_general_arrow_stem_radius , 0.02 ,
     "Relative radius of arrow stem" },
   { F|O, "Axes" , opt_general_axes , 0. ,
-    "Display the axes linked to the model" },
+    "Axes (0=none, 1=simple axes, 2=box, 3=full grid, 4=open grid, 5=ruler)" },
+  { F|O, "AxesAutoPosition" , opt_general_axes_auto_position , 1. , 
+    "Position the axes automatically" }, 
+  { F|O, "AxesMaxX" , opt_general_axes_xmax , 1. , 
+    "Maximum X-axis coordinate" }, 
+  { F|O, "AxesMaxY" , opt_general_axes_ymax , 1. , 
+    "Maximum Y-axis coordinate" }, 
+  { F|O, "AxesMaxZ" , opt_general_axes_zmax , 1. , 
+    "Maximum Z-axis coordinate" }, 
+  { F|O, "AxesMinX" , opt_general_axes_xmin , 0. , 
+    "Minimum X-axis coordinate" }, 
+  { F|O, "AxesMinY" , opt_general_axes_ymin , 0. , 
+    "Minimum Y-axis coordinate" }, 
+  { F|O, "AxesMinZ" , opt_general_axes_zmin , 0. , 
+    "Minimum Z-axis coordinate" }, 
+  { F|O, "AxesTicsX" , opt_general_axes_tics0 , 5. ,
+    "Number of tics on the X-axis" },
+  { F|O, "AxesTicsY" , opt_general_axes_tics1 , 5. ,
+    "Number of tics on the Y-axis" },
+  { F|O, "AxesTicsZ" , opt_general_axes_tics2 , 5. ,
+    "Number of tics on the Z-axis" },
   { F,   "Clip0" , opt_general_clip0 , 0. ,
     "Enable clipping plane 0 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
   { F,   "Clip0A" , opt_general_clip0a , 1.0 ,
@@ -452,6 +507,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Third coefficient in equation for clipping plane 5" },
   { F,   "Clip5D" , opt_general_clip5d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 5" },
+  { F,   "ClipFactor" , opt_general_clip_factor , 5.0 , 
+    "Near and far clipping plane distance factor (decrease value for better z-buffer resolution)" },
   { F|S, "ClipPositionX" , opt_general_clip_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the clipping planes window" }, 
   { F|S, "ClipPositionY" , opt_general_clip_position1 , 150. , 
@@ -470,8 +527,6 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "DrawBoundingBoxes" , opt_general_draw_bounding_box, 0. ,
     "Draw bounding boxes" },
 
-  { F|O, "FakeTransparency" , opt_general_fake_transparency , 0. ,
-    "Use fake transparency (cheaper than the real thing, but incorrect)" },
   { F|O, "FastRedraw" , opt_general_fast_redraw, 1. ,
     "Draw simplified model while rotating, panning and zooming" },
   { F|S, "FileChooserPositionX" , opt_general_file_chooser_position0 , 200. , 
@@ -558,6 +613,10 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "LineWidth" , opt_general_line_width , 1.0 , 
     "Display width of lines (in pixels)" },
 
+  { F|S, "ManipulatorPositionX" , opt_general_manip_position0 , 650. , 
+    "Horizontal position (in pixels) of the upper left corner of the manipulator window" }, 
+  { F|S, "ManipulatorPositionY" , opt_general_manip_position1 , 150. , 
+    "Vertical position (in pixels) of the upper left corner of the manipulator window" }, 
   { F|S, "MenuPositionX" , opt_general_menu_position0 , 800. , 
     "Horizontal position (in pixels) of the upper left corner of the menu window" }, 
   { F|S, "MenuPositionY" , opt_general_menu_position1 , 50. ,
@@ -571,6 +630,9 @@ StringXNumber GeneralOptions_Number[] = {
   { F|S, "MessageWidth" , opt_general_message_size0 , 450. , 
     "Width (in pixels) of the message window" }, 
 
+  { F|O, "NoPopup" , opt_general_nopopup , 0. , 
+    "Disable interactive dialog windows in scripts (and use default values instead)" },
+
   { F|S, "OptionsPositionX" , opt_general_option_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the option window" }, 
   { F|S, "OptionsPositionY" , opt_general_option_position1 , 150. , 
@@ -580,6 +642,12 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "PointSize" , opt_general_point_size , 3. , 
     "Display size of points (in pixels)" },
+  { F|O, "PolygonOffsetAlwaysOn" , opt_general_polygon_offset_always , 0. , 
+    "Always apply polygon offset, instead of trying to detect when it is required" },
+  { F|O, "PolygonOffsetFactor" , opt_general_polygon_offset_factor , 1. , 
+    "Polygon offset factor (offset = factor * DZ + r * units)" },
+  { F|O, "PolygonOffsetUnits" , opt_general_polygon_offset_units , 1. , 
+    "Polygon offset units (offset = factor * DZ + r * units)" },
 
   { F|O, "QuadricSubdivisions" , opt_general_quadric_subdivisions, 8. ,
     "Number of subdivisions used to draw points or lines as spheres or cylinders" },
@@ -619,6 +687,8 @@ StringXNumber GeneralOptions_Number[] = {
     "X position of small axes (use negative values for right alignment)" },
   { F|O, "SmallAxesPositionY" , opt_general_small_axes_position1 , -40. ,
     "Y position of small axes (use negative values for bottom alignment)" },
+  { F|O, "SmallAxesSize" , opt_general_small_axes_size , 30. ,
+    "Size (in pixels) of small axes" },
   { F|S, "SolverPositionX" , opt_general_solver_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the solver windows" }, 
   { F|S, "SolverPositionY" , opt_general_solver_position1 , 150. ,
@@ -667,8 +737,8 @@ StringXNumber GeneralOptions_Number[] = {
   { F|S, "VisibilityPositionY" , opt_general_visibility_position1 , 150. , 
     "Vertical position (in pixels) of the upper left corner of the visibility window" }, 
 
-  { F|O, "ZoomFactor" , opt_general_zoom_factor , 1.1 ,
-    "`Speed' of the middle mouse button zoom" },
+  { F|O, "ZoomFactor" , opt_general_zoom_factor , 4.0 ,
+    "Middle mouse button zoom acceleration factor" },
 
   { 0, NULL , NULL , 0. , NULL }
 } ;
@@ -719,7 +789,7 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "ScalingFactor" , opt_geometry_scaling_factor , 1.0 , 
     "Global geometry scaling factor" },
-  { F|O, "StlCreateElementary" , opt_geometry_stl_create_elementary , 1. ,
+  { F|O, "StlCreateElementary" , opt_geometry_stl_create_elementary , 0. ,
     "Treat each STL input face as a new geometrical surface" },
   { F|O, "StlCreatePhysical" , opt_geometry_stl_create_physical , 1. ,
     "Automatically create physical entities when importing STL faces as geometrical surfaces" },
@@ -756,7 +826,7 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "ConstrainedBackgroundMesh" , opt_mesh_constrained_bgmesh, 0. ,
     "Should the background mesh be constrained by the characteristic lengths associated with the geometry?" },
   { F,   "CpuTime" , opt_mesh_cpu_time , 0. , 
-    "CPU time for the generation of the current mesh (in seconds)" },
+    "CPU time (in seconds) for the generation of the current mesh (read-only)" },
   { F,   "CutPlane" , opt_mesh_use_cut_plane , 0 ,
     "Enable mesh cut plane" },
   { F,   "CutPlaneAsSurface" , opt_mesh_cut_plane_as_surface , 0 ,
@@ -781,7 +851,7 @@ StringXNumber MeshOptions_Number[] = {
     "Explode elements (between 0=point and 1=non-transformed)" },
 
   { F|O, "Format" , opt_mesh_format , FORMAT_MSH , 
-    "Mesh output format (1=MSH, 2=UNV, 3=GREF, 19=VRML)" },
+    "Mesh output format (1=msh, 2=unv, 3=gref, 19=vrml)" },
 
   { F|O, "GammaInf" , opt_mesh_gamma_inf , 0.0 , 
     "Only display elements whose Gamma factor is greater than GammaInf" },
@@ -789,7 +859,7 @@ StringXNumber MeshOptions_Number[] = {
     "Only display elements whose Gamma factor is smaller than GammaSup" },
 
   { F|O, "Interactive" , opt_mesh_interactive , 0. ,
-    "Show the construction of the 2D mesh in real time (only with the 2D anisotropic algorithm)" },
+    "Show the construction of 2D anisotropic mesh in real time" },
 
   { F|O, "Light" , opt_mesh_light , 0. , 
     "Enable lighting for the mesh" },
@@ -807,22 +877,22 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "MinimumCirclePoints" , opt_mesh_min_circ_points, 7. ,
     "Minimum number of points used to mesh a circle" },
   { F|O, "MshFileVersion" , opt_mesh_msh_file_version , 1.0 , 
-    "MSH mesh file version to generate" },
+    "Version of the `msh' file format to use" },
 
   { F, "NbHexahedra" , opt_mesh_nb_hexahedra , 0. , 
-    "Number of hexahedra in the current mesh" },
+    "Number of hexahedra in the current mesh (read-only)" },
   { F, "NbNodes" , opt_mesh_nb_nodes , 0. , 
-    "Number of nodes in the current mesh" },
+    "Number of nodes in the current mesh (read-only)" },
   { F, "NbPrisms" , opt_mesh_nb_prisms , 0. , 
-    "Number of prisms in the current mesh" },
+    "Number of prisms in the current mesh (read-only)" },
   { F, "NbPyramids" , opt_mesh_nb_pyramids , 0. , 
-    "Number of pyramids in the current mesh" },
+    "Number of pyramids in the current mesh (read-only)" },
   { F, "NbQuadrangles" , opt_mesh_nb_quadrangles , 0. , 
-    "Number of quadrangles in the current mesh" },
+    "Number of quadrangles in the current mesh (read-only)" },
   { F, "NbTetrahedra" , opt_mesh_nb_tetrahedra , 0. , 
-    "Number of tetrahedra in the current mesh" },
+    "Number of tetrahedra in the current mesh (read-only)" },
   { F, "NbTriangles" , opt_mesh_nb_triangles , 0. , 
-    "Number of triangles in the current mesh" },
+    "Number of triangles in the current mesh (read-only)" },
   { F|O, "Normals" , opt_mesh_normals , 0.0 ,
     "Display size of normal vectors (in pixels)" }, 
 
@@ -834,7 +904,7 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "PointsPerElement" , opt_mesh_points_per_element , 0. , 
     "Display mesh nodes per element (slower, but permits to visualize only a subset of the nodes)" },
   { F|O, "PointInsertion" , opt_mesh_point_insertion, CENTER_CIRCCIRC ,
-    "Point insertion method for isotropic 2D algorithm (1=center of circumscribed circle, 2=cog)" },
+    "Point insertion method for isotropic 2D algorithm (1=center of circumscribed circle, 2=center of gravity)" },
   { F|O, "PointNumbers" , opt_mesh_points_num , 0. , 
     "Display mesh node numbers?" },
   { F|O, "PointSize" , opt_mesh_point_size , 4. , 
@@ -846,9 +916,9 @@ StringXNumber MeshOptions_Number[] = {
   //  "Target quality for tetrahedral elements (not fully functional)" },
 
   { F|O, "RadiusInf" , opt_mesh_radius_inf , 0.0 , 
-    "Only display elements whose Radius is greater than RadiusInf" },
+    "Only display elements whose longest edge is greater than RadiusInf" },
   { F|O, "RadiusSup" , opt_mesh_radius_sup , 0.0 , 
-    "Only display elements whose Radius is smaller than RadiusSup" },
+    "Only display elements whose longest edge is smaller than RadiusSup" },
   { F|O, "RandomFactor" , opt_mesh_rand_factor , 1.e-4 ,
     "Random factor used in 2D and 3D meshing algorithm (test other values when the algorithm fails)" },
 
@@ -886,7 +956,7 @@ StringXNumber MeshOptions_Number[] = {
 
 StringXNumber SolverOptions_Number[] = {
   { F|O, "MaximumDelay" , opt_solver_max_delay , 4.0 ,
-    "Maximum delay allowed for solver response (in seconds)" },
+    "Maximum delay (in seconds) allowed for solver response" },
 
   { F|O, "Plugins" , opt_solver_plugins , 0. ,
     "Enable default solver plugins?" },
@@ -943,17 +1013,17 @@ StringXNumber PostProcessingOptions_Number[] = {
   { F|O, "CombineRemoveOriginal" , opt_post_combine_remove_orig , 1. ,
     "Remove original views after a Combine operation" },
 
-  { F|0, "Format" , opt_post_file_format , 0. ,
-    "Default file format for post-processing views" },
+  { F|O, "Format" , opt_post_file_format , 0. ,
+    "Default file format for post-processing views (0=ASCII view, 1=binary view, 2=parsed view, 3=STL triangulation, 4=text)" },
 
-  { F|O, "HorizontalScales" , opt_post_horizontal_scales , 0. , 
+  { F|O, "HorizontalScales" , opt_post_horizontal_scales , 1. , 
     "Display value scales horizontally" },
 
   { F|O, "Link" , opt_post_link , 0. ,
     "Link post-processing views (0=none, 1/2=changes in visible/all, 3/4=everything in visible/all)" },
 
-  { F,   "NbViews" , opt_post_nb_views , 0. ,//this default val is not used
-    "Current number of views merged" },
+  { F,   "NbViews" , opt_post_nb_views , 0. ,
+    "Current number of views merged (read-only)" },
 
   { F|O, "Plugins" , opt_post_plugins , 1. ,
     "Enable default post-processing plugins?" },
@@ -970,14 +1040,12 @@ StringXNumber PostProcessingOptions_Number[] = {
 } ;
 
 StringXNumber ViewOptions_Number[] = {
-  { F|O, "AlphaChannel" , opt_view_alpha_channel , 1.0 ,
-    "Global alpha channel value (used only if != 1)" },
   { F|O, "AngleSmoothNormals" , opt_view_angle_smooth_normals , 180. ,
     "Threshold angle below which normals are not smoothed" },
   { F|O, "ArrowHeadRadius" , opt_view_arrow_head_radius , 0.12 ,
     "Relative radius of arrow head" },
   { F|O, "ArrowLocation" , opt_view_arrow_location , DRAW_POST_LOCATE_COG , 
-    "Arrow location (1=cog, 2=node)" },
+    "Arrow location (1=center of gravity, 2=node)" },
   { F|O, "ArrowSize" , opt_view_arrow_size , 60. ,
     "Display size of arrows (in pixels)" },
   { F|O, "ArrowSizeProportional" , opt_view_arrow_size_proportional , 1. ,
@@ -987,10 +1055,51 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "ArrowStemRadius" , opt_view_arrow_stem_radius , 0.02 ,
     "Relative radius of arrow stem" },
   { F|O, "AutoPosition" , opt_view_auto_position , 1. , 
-    "Position the scale or the 2D graph automatically to avoid overlaps" }, 
+    "Position the scale or 2D plot automatically" }, 
+  { F|O, "Axes" , opt_view_axes , 0 ,
+    "Axes (0=none, 1=simple axes, 2=box, 3=full grid, 4=open grid, 5=ruler)" },
+  { F|O, "AxesAutoPosition" , opt_view_axes_auto_position , 1. , 
+    "Position the axes automatically" }, 
+  { F|O, "AxesMaxX" , opt_view_axes_xmax , 1. , 
+    "Maximum X-axis coordinate" }, 
+  { F|O, "AxesMaxY" , opt_view_axes_ymax , 1. , 
+    "Maximum Y-axis coordinate" }, 
+  { F|O, "AxesMaxZ" , opt_view_axes_zmax , 1. , 
+    "Maximum Z-axis coordinate" }, 
+  { F|O, "AxesMinX" , opt_view_axes_xmin , 0. , 
+    "Minimum X-axis coordinate" }, 
+  { F|O, "AxesMinY" , opt_view_axes_ymin , 0. , 
+    "Minimum Y-axis coordinate" }, 
+  { F|O, "AxesMinZ" , opt_view_axes_zmin , 0. , 
+    "Minimum Z-axis coordinate" }, 
+  { F|O, "AxesTicsX" , opt_view_axes_tics0 , 5. ,
+    "Number of tics on the X-axis" },
+  { F|O, "AxesTicsY" , opt_view_axes_tics1 , 5. ,
+    "Number of tics on the Y-axis" },
+  { F|O, "AxesTicsZ" , opt_view_axes_tics2 , 5. ,
+    "Number of tics on the Z-axis" },
 
   { F|O, "Boundary" , opt_view_boundary , 0. ,
-    "Draw the `N minus b'-dimensional boundary of the simplex (N=simplex dimension, b=option value)" },
+    "Draw the `N minus b'-dimensional boundary of the element (N=element dimension, b=option value)" },
+
+  { F|O, "ColormapAlpha" , opt_view_colormap_alpha , 1.0 ,
+    "Colormap alpha channel value (used only if != 1)" },
+  { F|O, "ColormapAlphaPower" , opt_view_colormap_alpha_power , 0.0 ,
+    "Colormap alpha channel power" },
+  { F|O, "ColormapBeta" , opt_view_colormap_beta , 0.0 ,
+    "Colormap beta parameter (gamma = 1-beta)" },
+  { F|O, "ColormapBias" , opt_view_colormap_bias , 0.0 ,
+    "Colormap bias" },
+  { F|O, "ColormapCurvature" , opt_view_colormap_curvature , 0.0 ,
+    "Colormap curvature or slope coefficient" },
+  { F|O, "ColormapInvert" , opt_view_colormap_invert , 0. ,
+    "Invert the color values, i.e., replace x with (255-x) in the colormap?" },
+  { F|O, "ColormapNumber" , opt_view_colormap_number , 2. ,
+    "Default colormap number" },
+  { F|O, "ColormapRotation" , opt_view_colormap_rotation , 0. ,
+    "Incremental colormap rotation" },
+  { F|O, "ColormapSwap" , opt_view_colormap_swap , 0. ,
+    "Swap the min/max values in the colormap?" },
 
   { F,   "CustomMax" , opt_view_custom_max , 0. , 
     "User-defined maximum value to be displayed" },
@@ -1029,11 +1138,19 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "ExternalView" , opt_view_external_view , -1. ,
     "Index of the view used to color vector fields (-1=self)" },
 
-  { F|O, "Grid" , opt_view_grid , 2 ,
-    "Grid mode for 2D graphs (0=none, 1=simple, 2=frame, 3=grid)" },
+  { F|O, "FakeTransparency" , opt_view_fake_transparency , 0. ,
+    "Use fake transparency (cheaper than the real thing, but incorrect)" },
+
+  { F|O, "GeneralizedRaiseFactor" , opt_view_gen_raise_factor , 1. ,
+    "Generalized raise amplification factor" },
+  { F|O, "GeneralizedRaiseView" , opt_view_gen_raise_view , -1. ,
+    "Index of the view used for generalized raise (-1=self)" },
+  // for backward compatibility:
+  { F,   "Grid" , opt_view_axes , 0 ,
+    "Grid mode (this option is deprecated: use View.Axes instead)" },
 
   { F|O, "Height" , opt_view_size1 , 200. , 
-    "Height (in pixels) of the scale or 2D graph" }, 
+    "Height (in pixels) of the scale or 2D plot" }, 
 
   { F|O, "IntervalsType" , opt_view_intervals_type , DRAW_POST_CONTINUOUS ,
     "Type of interval display (1=iso, 2=continuous, 3=discrete, 4=numeric)" },
@@ -1047,17 +1164,29 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "LineWidth" , opt_view_line_width , 1.0 , 
     "Display width of lines (in pixels)" },
 
-  { F,   "Max" , opt_view_max , -1.e200 , 
-    "Maximum value in the view (do not change this!)" },
-  { F,   "Min" , opt_view_min , 1.e200 ,
-    "Minimum value in the view (do not change this!)" },
+  { F,   "Max" , opt_view_max , 0. , 
+    "Maximum value in the view (read-only)" },
+  { F,   "MaxX" , opt_view_xmax , 0. , 
+    "Maximum view coordinate along the X-axis (read-only)" }, 
+  { F,   "MaxY" , opt_view_ymax , 0. , 
+    "Maximum view coordinate along the Y-axis (read-only)" }, 
+  { F,   "MaxZ" , opt_view_zmax , 0. , 
+    "Maximum view coordinate along the Z-axis (read-only)" }, 
+  { F,   "Min" , opt_view_min , 0. ,
+    "Minimum value in the view (read-only)" },
+  { F,   "MinX" , opt_view_xmin , 0. , 
+    "Minimum view coordinate along the X-axis (read-only)" }, 
+  { F,   "MinY" , opt_view_ymin , 0. , 
+    "Minimum view coordinate along the Y-axis (read-only)" }, 
+  { F,   "MinZ" , opt_view_zmin , 0. , 
+    "Minimum view coordinate along the Z-axis (read-only)" }, 
 
-  { F|O, "NbAbscissa" , opt_view_nb_abscissa , 5. ,
-    "Number of abscissa intervals for 2D graphs" },
   { F|O, "NbIso" , opt_view_nb_iso , 15. ,
     "Number of intervals" },
   { F,   "NbTimeStep" , opt_view_nb_timestep , 1. ,
     "Number of time steps in the view (do not change this!)" },
+  { F|O, "Normals" , opt_view_normals , 0. ,
+    "Display size of normal vectors (in pixels)" }, 
 
   { F,   "OffsetX" , opt_view_offset0 , 0. , 
     "Translation of the view along X-axis (in model coordinates)" },
@@ -1071,9 +1200,9 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "PointType" , opt_view_point_type , 0. , 
     "Display points as solid color dots (0) or 3D spheres (1)" },
   { F|O, "PositionX" , opt_view_position0 , 100. , 
-    "Horizontal position (in pixels) of the upper left corner of the scale or 2D graph" }, 
+    "Horizontal position (in pixels) of the upper left corner of the scale or 2D plot" }, 
   { F|O, "PositionY" , opt_view_position1 , 50. , 
-    "Vertical position (in pixels) of the upper left corner of the scale or 2D graph" }, 
+    "Vertical position (in pixels) of the upper left corner of the scale or 2D plot" }, 
 
   { F,   "RaiseX" , opt_view_raise0 , 0. , 
     "Elevation of the view along X-axis (in model coordinates)" },
@@ -1096,12 +1225,36 @@ StringXNumber ViewOptions_Number[] = {
     "Show time value in the scale? (1=only if NbTimeStep>1, 2=always)" },
   { F|O, "SmoothNormals" , opt_view_smooth_normals , 0. ,
     "Smooth the normals?" },
+
+  { F|O, "Tangents" , opt_view_tangents , 0. ,
+    "Display size of tangent vectors (in pixels)" }, 
   { F|O, "TensorType" , opt_view_tensor_type , 0. ,
     "Tensor Visualization Type" },
   { F,   "TimeStep" , opt_view_timestep , 0. ,
     "Current time step displayed" },
+  { F,   "Transform11" , opt_view_transform00 , 1. ,
+    "Element (1,1) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform12" , opt_view_transform01 , 0. ,
+    "Element (1,2) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform13" , opt_view_transform02 , 0. ,
+    "Element (1,3) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform21" , opt_view_transform10 , 0. ,
+    "Element (2,1) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform22" , opt_view_transform11 , 1. ,
+    "Element (2,2) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform23" , opt_view_transform12 , 0. ,
+    "Element (2,3) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform31" , opt_view_transform20 , 0. ,
+    "Element (3,1) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform32" , opt_view_transform21 , 0. ,
+    "Element (3,2) of the 3x3 coordinate transformation matrix" },
+  { F,   "Transform33" , opt_view_transform22 , 1. ,
+    "Element (3,3) of the 3x3 coordinate transformation matrix" },
   { F, "Type" , opt_view_type , DRAW_POST_3D ,
-    "Type of graph (1=3D, 2=2D-space, 3=2D-time)" },
+    "Type of plot (1=3D, 2=2D space, 3=2D time)" },
+
+  { F|O, "UseGeneralizedRaise" , opt_view_use_gen_raise , 0 ,
+    "Use generalized raise?" },
 
   { F|O, "VectorType" , opt_view_vector_type , DRAW_POST_ARROW3D ,
     "Vector display type (1=segment, 2=arrow, 3=pyramid, 4=3D arrow, 5=displacement)" },
@@ -1109,7 +1262,7 @@ StringXNumber ViewOptions_Number[] = {
     "Is the view visible?" },
 
   { F|O, "Width" , opt_view_size0 , 300. , 
-    "Width (in pixels) of the scale or 2D graph" }, 
+    "Width (in pixels) of the scale or 2D plot" }, 
 
   { 0, NULL , NULL , 0. , NULL }
 } ;
@@ -1130,7 +1283,7 @@ StringXNumber PrintOptions_Number[] = {
   { F|O, "EpsPS3Shading" , opt_print_eps_ps3shading , 0. ,
     "Enable PostScript Level 3 shading" },
   { F|O, "EpsQuality" , opt_print_eps_quality , 1. ,
-    "PostScript/PDF quality (1=simple sort, 2=BSP tree sort)" },
+    "PostScript/PDF quality (0=bitmap, 1=vector (simple sort), 2=vector (accurate sort)" },
 
   { F|O, "Format" , opt_print_format , FORMAT_AUTO , 
     "File format (10=automatic)" }, 
@@ -1146,6 +1299,8 @@ StringXNumber PrintOptions_Number[] = {
 
   { F|O, "JpegQuality" , opt_print_jpeg_quality , 100. ,
     "JPEG quality (between 1 and 100)" },
+  { F|O, "JpegSmoothing" , opt_print_jpeg_smoothing , 0. ,
+    "JPEG smoothing (between 0 and 100)" },
 
   { 0, NULL , NULL , 0. }
 } ;
@@ -1171,7 +1326,7 @@ StringXColor GeneralOptions_Color[] = {
     PACK_COLOR(0,   0,   0,   255),
     "Text color" },
   { F|O, "Axes" , opt_general_color_axes ,
-    PACK_COLOR(255, 255, 0,   255),
+    PACK_COLOR(255, 255, 255,   255),
     PACK_COLOR(128, 128, 128, 255),
     PACK_COLOR(0,   0,   0,   255),
     "Axes color" },
@@ -1339,10 +1494,35 @@ StringXColor PostProcessingOptions_Color[] = {
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
 
+#define ELECOL  PACK_COLOR(255,255,255,255), PACK_COLOR(0,0,0,255), PACK_COLOR(0,0,0,255)
+
 StringXColor ViewOptions_Color[] = {
+  { F|O, "Points" , opt_view_color_points , ELECOL, "Point color" },
+  { F|O, "Lines" , opt_view_color_lines , ELECOL, "Line color" },
+  { F|O, "Triangles" , opt_view_color_triangles , ELECOL, "Triangle color" },
+  { F|O, "Quadrangles" , opt_view_color_quadrangles , ELECOL, "Quadrangle color" },
+  { F|O, "Tetrahedra" , opt_view_color_tetrahedra , ELECOL, "Tetrahedron color" },
+  { F|O, "Hexahedra" , opt_view_color_hexahedra , ELECOL, "Hexahedron color" },
+  { F|O, "Prisms" , opt_view_color_prisms , ELECOL, "Prism color" },
+  { F|O, "Pyramids" , opt_view_color_pyramids , ELECOL, "Pyramid color" },
+  { F|O, "Tangents" , opt_view_color_tangents ,
+    PACK_COLOR(255, 255, 0, 255),
+    PACK_COLOR(255, 255, 0, 255),
+    PACK_COLOR(0,   0,   0, 255),
+    "Tangent vector color" },
+  { F|O, "Normals" , opt_view_color_normals ,
+    PACK_COLOR(255, 0, 0, 255),
+    PACK_COLOR(255, 0, 0, 255),
+    PACK_COLOR(0,   0, 0, 255),
+    "Normal vector color" },
+  { F|O, "Text2D" , opt_view_color_text2d , ELECOL, "2D text color" },
+  { F|O, "Text3D" , opt_view_color_text3d , ELECOL, "3D text color" },
+  { F|O, "Axes" , opt_view_color_axes , ELECOL, "Axes color" },
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
 
+#undef ELECOL
+
 StringXColor PrintOptions_Color[] = {
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
