@@ -1,7 +1,7 @@
 #ifndef _GMSH_UI_H_
 #define _GMSH_UI_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 #if defined(HAVE_FLTK)
 #  include <FL/Fl.H>
 #  if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 1)
+#    if (FL_PATCH_VERSION >= 6)
+#      define HAVE_FLTK_1_1_6_OR_ABOVE
+#    endif
 #    if (FL_PATCH_VERSION >= 5)
 #      define HAVE_FLTK_1_1_5_OR_ABOVE
 #    endif
@@ -35,6 +38,11 @@
 #  else
 #    include <GL/glu.h>
 #  endif
+# define GMSH_WINDOW_BOX   FL_FLAT_BOX
+# define GMSH_TOGGLE_BOX   FL_DOWN_BOX
+# define GMSH_TOGGLE_COLOR FL_BLACK
+# define GMSH_RADIO_BOX    FL_ROUND_DOWN_BOX
+# define GMSH_RADIO_COLOR  FL_BLACK
 #endif
 
 #endif
