@@ -1,7 +1,7 @@
 #ifndef _MESSAGE_H_
 #define _MESSAGE_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -61,6 +61,8 @@
 #define SOLVER        31  // Solver message
 #define SOLVERR       32  // Solver errors and warnings
 
+#define PROGRESS      40  // Progress indicator
+
 #define WHITE_STR          "        : "
 #define FATAL_STR          "Fatal   : "
 #define ERROR_STR          "Error   : "
@@ -69,9 +71,10 @@
 #define DEBUG_STR          "Debug   : "
 #define STATUS_STR         "Info    : "
 
-void   Signal (int signum);
-void   Msg (int level, char *fmt, ...);
-double Cpu (void);
+void   Signal(int signum);
+void   Msg(int level, char *fmt, ...);
+double Cpu(void);
 void   Exit(int);
+double GetValue(char *text, double defaultval);
 
 #endif
