@@ -1,7 +1,7 @@
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,7 +36,14 @@
 
 // STRINGS
 
+char * opt_general_axes_label0(OPT_ARGS_STR);
+char * opt_general_axes_label1(OPT_ARGS_STR);
+char * opt_general_axes_label2(OPT_ARGS_STR);
+char * opt_general_axes_format0(OPT_ARGS_STR);
+char * opt_general_axes_format1(OPT_ARGS_STR);
+char * opt_general_axes_format2(OPT_ARGS_STR);
 char * opt_general_display(OPT_ARGS_STR);
+char * opt_general_filename(OPT_ARGS_STR);
 char * opt_general_default_filename(OPT_ARGS_STR);
 char * opt_general_tmp_filename(OPT_ARGS_STR);
 char * opt_general_error_filename(OPT_ARGS_STR);
@@ -47,6 +54,7 @@ char * opt_general_web_browser(OPT_ARGS_STR);
 char * opt_general_scheme(OPT_ARGS_STR);
 char * opt_general_graphics_font(OPT_ARGS_STR);
 char * opt_mesh_triangle_options(OPT_ARGS_STR);
+char * opt_solver_socket_name(OPT_ARGS_STR);
 char * opt_solver_name(OPT_ARGS_STR);
 char * opt_solver_name0(OPT_ARGS_STR);
 char * opt_solver_name1(OPT_ARGS_STR);
@@ -194,8 +202,15 @@ char * opt_solver_fifth_button_command4(OPT_ARGS_STR);
 char * opt_view_name(OPT_ARGS_STR);
 char * opt_view_format(OPT_ARGS_STR);
 char * opt_view_filename(OPT_ARGS_STR);
-char * opt_view_abscissa_name(OPT_ARGS_STR);
-char * opt_view_abscissa_format(OPT_ARGS_STR);
+char * opt_view_axes_label0(OPT_ARGS_STR);
+char * opt_view_axes_label1(OPT_ARGS_STR);
+char * opt_view_axes_label2(OPT_ARGS_STR);
+char * opt_view_axes_format0(OPT_ARGS_STR);
+char * opt_view_axes_format1(OPT_ARGS_STR);
+char * opt_view_axes_format2(OPT_ARGS_STR);
+char * opt_view_gen_raise0(OPT_ARGS_STR);
+char * opt_view_gen_raise1(OPT_ARGS_STR);
+char * opt_view_gen_raise2(OPT_ARGS_STR);
 
 // NUMBERS
 
@@ -212,6 +227,9 @@ double opt_general_file_chooser_position0(OPT_ARGS_NUM);
 double opt_general_file_chooser_position1(OPT_ARGS_NUM);
 double opt_general_viewport2(OPT_ARGS_NUM);
 double opt_general_viewport3(OPT_ARGS_NUM);
+double opt_general_polygon_offset_always(OPT_ARGS_NUM);
+double opt_general_polygon_offset_factor(OPT_ARGS_NUM);
+double opt_general_polygon_offset_units(OPT_ARGS_NUM);
 double opt_general_menu_position0(OPT_ARGS_NUM);
 double opt_general_menu_position1(OPT_ARGS_NUM);
 double opt_general_system_menu_bar(OPT_ARGS_NUM);
@@ -227,6 +245,8 @@ double opt_general_visibility_position0(OPT_ARGS_NUM);
 double opt_general_visibility_position1(OPT_ARGS_NUM);
 double opt_general_clip_position0(OPT_ARGS_NUM);
 double opt_general_clip_position1(OPT_ARGS_NUM);
+double opt_general_manip_position0(OPT_ARGS_NUM);
+double opt_general_manip_position1(OPT_ARGS_NUM);
 double opt_general_visibility_mode(OPT_ARGS_NUM);
 double opt_general_session_save(OPT_ARGS_NUM);
 double opt_general_options_save(OPT_ARGS_NUM);
@@ -246,12 +266,14 @@ double opt_general_translation2(OPT_ARGS_NUM);
 double opt_general_scale0(OPT_ARGS_NUM);
 double opt_general_scale1(OPT_ARGS_NUM);
 double opt_general_scale2(OPT_ARGS_NUM);
+double opt_general_clip_factor(OPT_ARGS_NUM);
 double opt_general_point_size(OPT_ARGS_NUM);
 double opt_general_line_width(OPT_ARGS_NUM);
 double opt_general_shine(OPT_ARGS_NUM);
 double opt_general_shine_exponent(OPT_ARGS_NUM);
 double opt_general_color_scheme(OPT_ARGS_NUM);
 double opt_general_verbosity(OPT_ARGS_NUM);
+double opt_general_nopopup(OPT_ARGS_NUM);
 double opt_general_terminal(OPT_ARGS_NUM);
 double opt_general_tooltips(OPT_ARGS_NUM);
 double opt_general_confirm_overwrite(OPT_ARGS_NUM);
@@ -259,13 +281,23 @@ double opt_general_orthographic(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_axes(OPT_ARGS_NUM);
+double opt_general_axes_auto_position(OPT_ARGS_NUM);
+double opt_general_axes_xmin(OPT_ARGS_NUM);
+double opt_general_axes_xmax(OPT_ARGS_NUM);
+double opt_general_axes_ymin(OPT_ARGS_NUM);
+double opt_general_axes_ymax(OPT_ARGS_NUM);
+double opt_general_axes_zmin(OPT_ARGS_NUM);
+double opt_general_axes_zmax(OPT_ARGS_NUM);
+double opt_general_axes_tics0(OPT_ARGS_NUM);
+double opt_general_axes_tics1(OPT_ARGS_NUM);
+double opt_general_axes_tics2(OPT_ARGS_NUM);
 double opt_general_small_axes(OPT_ARGS_NUM);
 double opt_general_small_axes_position0(OPT_ARGS_NUM);
 double opt_general_small_axes_position1(OPT_ARGS_NUM);
+double opt_general_small_axes_size(OPT_ARGS_NUM);
 double opt_general_quadric_subdivisions(OPT_ARGS_NUM);
 double opt_general_double_buffer(OPT_ARGS_NUM);
 double opt_general_alpha_blending(OPT_ARGS_NUM);
-double opt_general_fake_transparency(OPT_ARGS_NUM);
 double opt_general_vector_type(OPT_ARGS_NUM);
 double opt_general_arrow_head_radius(OPT_ARGS_NUM);
 double opt_general_arrow_stem_length(OPT_ARGS_NUM);
@@ -458,35 +490,73 @@ double opt_view_min(OPT_ARGS_NUM);
 double opt_view_max(OPT_ARGS_NUM);
 double opt_view_custom_min(OPT_ARGS_NUM);
 double opt_view_custom_max(OPT_ARGS_NUM);
+double opt_view_xmin(OPT_ARGS_NUM);
+double opt_view_xmax(OPT_ARGS_NUM);
+double opt_view_ymin(OPT_ARGS_NUM);
+double opt_view_ymax(OPT_ARGS_NUM);
+double opt_view_zmin(OPT_ARGS_NUM);
+double opt_view_zmax(OPT_ARGS_NUM);
 double opt_view_offset0(OPT_ARGS_NUM);
 double opt_view_offset1(OPT_ARGS_NUM);
 double opt_view_offset2(OPT_ARGS_NUM);
 double opt_view_raise0(OPT_ARGS_NUM);
 double opt_view_raise1(OPT_ARGS_NUM);
 double opt_view_raise2(OPT_ARGS_NUM);
+double opt_view_transform00(OPT_ARGS_NUM);
+double opt_view_transform01(OPT_ARGS_NUM);
+double opt_view_transform02(OPT_ARGS_NUM);
+double opt_view_transform10(OPT_ARGS_NUM);
+double opt_view_transform11(OPT_ARGS_NUM);
+double opt_view_transform12(OPT_ARGS_NUM);
+double opt_view_transform20(OPT_ARGS_NUM);
+double opt_view_transform21(OPT_ARGS_NUM);
+double opt_view_transform22(OPT_ARGS_NUM);
 double opt_view_arrow_size(OPT_ARGS_NUM);
 double opt_view_arrow_size_proportional(OPT_ARGS_NUM);
 double opt_view_arrow_head_radius(OPT_ARGS_NUM);
 double opt_view_arrow_stem_length(OPT_ARGS_NUM);
 double opt_view_arrow_stem_radius(OPT_ARGS_NUM);
+double opt_view_normals(OPT_ARGS_NUM);
+double opt_view_tangents(OPT_ARGS_NUM);
 double opt_view_displacement_factor(OPT_ARGS_NUM);
+double opt_view_fake_transparency(OPT_ARGS_NUM);
 double opt_view_explode(OPT_ARGS_NUM);
 double opt_view_visible(OPT_ARGS_NUM);
 double opt_view_intervals_type(OPT_ARGS_NUM);
 double opt_view_saturate_values(OPT_ARGS_NUM);
 double opt_view_max_recursion_level(OPT_ARGS_NUM);
 double opt_view_target_error(OPT_ARGS_NUM);
-double opt_view_alpha_channel(OPT_ARGS_NUM);
+double opt_view_colormap_alpha(OPT_ARGS_NUM);
+double opt_view_colormap_alpha_power(OPT_ARGS_NUM);
+double opt_view_colormap_beta(OPT_ARGS_NUM);
+double opt_view_colormap_bias(OPT_ARGS_NUM);
+double opt_view_colormap_curvature(OPT_ARGS_NUM);
+double opt_view_colormap_invert(OPT_ARGS_NUM);
+double opt_view_colormap_number(OPT_ARGS_NUM);
+double opt_view_colormap_rotation(OPT_ARGS_NUM);
+double opt_view_colormap_swap(OPT_ARGS_NUM);
 double opt_view_external_view(OPT_ARGS_NUM);
+double opt_view_gen_raise_view(OPT_ARGS_NUM);
+double opt_view_gen_raise_factor(OPT_ARGS_NUM);
+double opt_view_use_gen_raise(OPT_ARGS_NUM);
 double opt_view_type(OPT_ARGS_NUM);
-double opt_view_grid(OPT_ARGS_NUM);
+double opt_view_axes(OPT_ARGS_NUM);
+double opt_view_axes_auto_position(OPT_ARGS_NUM);
+double opt_view_axes_tics0(OPT_ARGS_NUM);
+double opt_view_axes_tics1(OPT_ARGS_NUM);
+double opt_view_axes_tics2(OPT_ARGS_NUM);
+double opt_view_axes_xmin(OPT_ARGS_NUM);
+double opt_view_axes_ymin(OPT_ARGS_NUM);
+double opt_view_axes_zmin(OPT_ARGS_NUM);
+double opt_view_axes_xmax(OPT_ARGS_NUM);
+double opt_view_axes_ymax(OPT_ARGS_NUM);
+double opt_view_axes_zmax(OPT_ARGS_NUM);
 double opt_view_position0(OPT_ARGS_NUM);
 double opt_view_position1(OPT_ARGS_NUM);
 double opt_view_auto_position(OPT_ARGS_NUM);
 double opt_view_size0(OPT_ARGS_NUM);
 double opt_view_size1(OPT_ARGS_NUM);
 double opt_view_nb_iso(OPT_ARGS_NUM);
-double opt_view_nb_abscissa(OPT_ARGS_NUM);
 double opt_view_boundary(OPT_ARGS_NUM);
 double opt_view_light(OPT_ARGS_NUM);
 double opt_view_light_two_side(OPT_ARGS_NUM);
@@ -526,6 +596,7 @@ double opt_print_eps_background(OPT_ARGS_NUM);
 double opt_print_eps_line_width_factor(OPT_ARGS_NUM);
 double opt_print_eps_point_size_factor(OPT_ARGS_NUM);
 double opt_print_jpeg_quality(OPT_ARGS_NUM);
+double opt_print_jpeg_smoothing(OPT_ARGS_NUM);
 double opt_print_gif_dither(OPT_ARGS_NUM);
 double opt_print_gif_sort(OPT_ARGS_NUM);
 double opt_print_gif_interlace(OPT_ARGS_NUM);
@@ -572,7 +643,19 @@ unsigned int opt_mesh_color_7(OPT_ARGS_COL);
 unsigned int opt_mesh_color_8(OPT_ARGS_COL);
 unsigned int opt_mesh_color_9(OPT_ARGS_COL);
 unsigned int opt_mesh_color_10(OPT_ARGS_COL);
-
+unsigned int opt_view_color_points(OPT_ARGS_COL);
+unsigned int opt_view_color_lines(OPT_ARGS_COL);
+unsigned int opt_view_color_triangles(OPT_ARGS_COL);
+unsigned int opt_view_color_quadrangles(OPT_ARGS_COL);
+unsigned int opt_view_color_tetrahedra(OPT_ARGS_COL);
+unsigned int opt_view_color_hexahedra(OPT_ARGS_COL);
+unsigned int opt_view_color_prisms(OPT_ARGS_COL);
+unsigned int opt_view_color_pyramids(OPT_ARGS_COL);
+unsigned int opt_view_color_tangents(OPT_ARGS_COL);
+unsigned int opt_view_color_normals(OPT_ARGS_COL);
+unsigned int opt_view_color_text2d(OPT_ARGS_COL);
+unsigned int opt_view_color_text3d(OPT_ARGS_COL);
+unsigned int opt_view_color_axes(OPT_ARGS_COL);
 
 // Data structures and global functions
 
@@ -617,7 +700,7 @@ StringXColor * Get_ColorOptionCategory(char * cat);
 
 void Set_DefaultStringOptions(int num, StringXString s[]);
 void Set_DefaultNumberOptions(int num, StringXNumber s[]);
-void Set_DefaultColorOptions(int num, StringXColor s[], int scheme);
+void Set_DefaultColorOptions(int num, StringXColor s[]);
 
 void Set_StringOptions_GUI(int num, StringXString s[]);
 void Set_NumberOptions_GUI(int num, StringXNumber s[]);
