@@ -1,6 +1,6 @@
-// $Id: Colorbar_Window.cpp,v 1.38 2004/05/22 01:24:17 geuzaine Exp $
+// $Id: Colorbar_Window.cpp,v 1.48 2005/01/01 19:35:27 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
    
 #include "Gmsh.h"
 #include "GmshUI.h"
-#include "Numeric.h"
 #include "GUI.h"
 #include "ColorTable.h"
 #include "Colorbar_Window.h"
@@ -34,7 +33,6 @@
 
 extern Context_T CTX;
 
-#define UNDEFINED   0
 #define EPS         1.e-10
 
 // This file defines the Colorbar_Window class (subclass of Fl_Window)
@@ -54,71 +52,12 @@ Colorbar_Window::Colorbar_Window(int x, int y, int w, int h, const char *l)
   minval = maxval = 0.0;
 }
 
-
-// rgb on [0, 1], sv returned on [0, 1] and h on [0, 6]. 
-// Exception: h is returned UNDEFINED if S==0.
-
-#define RETURN_HSV(h,s,v) {*H=h; *S=s; *V=v; return;}
-
-static void RGB_to_HSV(double R, double G, double B,
-                       double *H, double *S, double *V)
-{
-  double v, x, f;
-  int i;
-
-  x = DMIN(DMIN(R, G), B);
-  v = DMAX(DMAX(R, G), B);
-  if(v == x)
-    RETURN_HSV(UNDEFINED, 0, v);
-  f = (R == x) ? G - B : ((G == x) ? B - R : R - G);
-  i = (R == x) ? 3 : ((G == x) ? 5 : 1);
-  RETURN_HSV(i - f / (v - x), (v - x) / v, v);
-}
-
-// h given on [0, 6] or UNDEFINED. s and v given on [0, 1].      
-// rgb each returned on [0, 1].
-
-#define RETURN_RGB(r,g,b) {*R=r; *G=g; *B=b; return;}
-
-static void HSV_to_RGB(double H, double S, double V,
-                       double *R, double *G, double *B)
-{
-  double m, n, f;
-  int i;
-
-  if(H == UNDEFINED)
-    RETURN_RGB(V, V, V);
-  i = (int)floor(H);
-  f = H - i;
-  if(!(i & 1))
-    f = 1 - f;  // if i is even
-  m = V * (1 - S);
-  n = V * (1 - S * f);
-
-  switch (i) {
-  case 6:
-  case 0:
-    RETURN_RGB(V, n, m);
-  case 1:
-    RETURN_RGB(n, V, m);
-  case 2:
-    RETURN_RGB(m, V, n);
-  case 3:
-    RETURN_RGB(m, n, V);
-  case 4:
-    RETURN_RGB(n, m, V);
-  case 5:
-    RETURN_RGB(V, m, n);
-  }
-}
-
-
 // Convert window X coordinate to color table index
 
 int Colorbar_Window::x_to_index(int x)
 {
   int index;
-  index = (int)(x * (float)ct->size / (float)w());
+  index = (int)(x * (double)ct->size / (double)w());
   if(index < 0)
     index = 0;
   else if(index >= ct->size)
@@ -131,7 +70,7 @@ int Colorbar_Window::x_to_index(int x)
 int Colorbar_Window::index_to_x(int index)
 {
   int x;
-  x = (int)(index * (float)w() / (float)(ct->size - 1));
+  x = (int)(index * (double)w() / (double)(ct->size - 1));
   if(x >= w())
     x = w() - 1;
   return x;
@@ -142,7 +81,7 @@ int Colorbar_Window::index_to_x(int index)
 int Colorbar_Window::intensity_to_y(int intensity)
 {
   int y;
-  y = (int)(wedge_y - intensity * (float)wedge_y / 255.);
+  y = (int)(wedge_y - intensity * (double)wedge_y / 255.);
   if(y < 0)
     y = 0;
   else if(y >= wedge_y)
@@ -155,7 +94,7 @@ int Colorbar_Window::intensity_to_y(int intensity)
 int Colorbar_Window::y_to_intensity(int y)
 {
   int intensity;
-  intensity = (int)((wedge_y - y) * 255. / (float)wedge_y);
+  intensity = (int)((wedge_y - y) * 255. / (double)wedge_y);
   if(intensity < 0)
     intensity = 0;
   else if(intensity > 255)
@@ -284,50 +223,53 @@ void Colorbar_Window::redraw_range(int a, int b)
   // print colortable mode and help
   fl_font(FL_HELVETICA, font_height);
   fl_color(fl_contrast(FL_BLACK, color_bg));
-  int xx0 = 10, xx1 = 12 * font_height, yy0 = 10;
+  int xx0 = 10, xx1 = 13 * font_height, yy0 = 10;
   if(help_flag) {
     i = 0;
-    fl_draw("1, 2, ..., 8", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("select predefined colormap", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("1, 2, ..., Ctrl+1, Ctrl+2, ...", xx0, yy0 + (i + 1) * font_height);
+    fl_draw("Select predefined colormap", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("mouse1", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("draw red or hue channel", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Draw red or hue channel", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("mouse2", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("draw green or saturation channel", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Draw green or saturation channel", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("mouse3", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("draw blue or value channel", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Draw blue or value channel", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("Ctrl+mouse1", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("draw alpha channel", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Draw alpha channel", xx1, yy0 + (i + 1) * font_height);
     i++;
-    fl_draw("c, p, r", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("copy, paste or reset colormap", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Ctrl+c, Ctrl+v, r", xx0, yy0 + (i + 1) * font_height);
+    fl_draw("Copy, paste or reset colormap", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("m", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("toggle RGB/HSV mode", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Toggle RGB/HSV mode", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("left, right", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("move colormap", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Translate abscissa", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("Ctrl+left, Ctrl+right", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("rotate colormap", xx1, yy0 + (i + 1) * font_height);
-    i++;
-    fl_draw("up, down", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("modify color curvature", xx1, yy0 + (i + 1) * font_height);
-    i++;
-    fl_draw("Ctrl+up, Ctrl+down", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("incr. or decr. alpha channel", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Rotate abscissa", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("i, Ctrl+i", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("invert x or y range", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Invert abscissa or ordinate", xx1, yy0 + (i + 1) * font_height);
+    i++;
+    fl_draw("up, down", xx0, yy0 + (i + 1) * font_height);
+    fl_draw("Modify curvature of color channels", xx1, yy0 + (i + 1) * font_height);
+    i++;
+    fl_draw("a, Ctrl+a", xx0, yy0 + (i + 1) * font_height);
+    fl_draw("Modify alpha coefficient", xx1, yy0 + (i + 1) * font_height);
+    i++;
+    fl_draw("p, Ctrl+p", xx0, yy0 + (i + 1) * font_height);
+    fl_draw("Modify alpha channel power law", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("b, Ctrl+b", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("increase or decrease gamma", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Modify gamma correction", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("h", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("show this help message", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Show this help message", xx1, yy0 + (i + 1) * font_height);
     i++;
   }
   else if(ct->ipar[COLORTABLE_MODE] == COLORTABLE_RGB)
@@ -343,7 +285,7 @@ void Colorbar_Window::redraw_marker()
 {
   int x, y0, y1;
   char str[50];
-  float val;
+  double val;
 
   make_current();
 
@@ -363,7 +305,7 @@ void Colorbar_Window::redraw_marker()
   // draw marker value
   fl_font(FL_HELVETICA, font_height);
   val =
-    minval + (maxval - minval) * ((float)marker_pos / (float)(ct->size - 1));
+    minval + (maxval - minval) * ((double)marker_pos / (double)(ct->size - 1));
   sprintf(str, "%g", val);
   fl_draw(str, 10, label_y);
 }
@@ -386,7 +328,7 @@ void Colorbar_Window::draw()
 
 // Update
 
-void Colorbar_Window::update(char *name, float min, float max,
+void Colorbar_Window::update(char *name, double min, double max,
                              GmshColorTable * table, int *changed)
 {
   label = name;
@@ -410,13 +352,15 @@ int Colorbar_Window::handle(int event)
 
   switch (event) {
 
+  case FL_FOCUS: // accept focus events when asked
+  case FL_UNFOCUS:
+    return 1;
+
   case FL_ENTER:
-    take_focus();       //force keyboard focus on the ColorbarWindow    
+    take_focus(); // force keyboard focus as soon as the mouse enters
     return 1;
 
   case FL_LEAVE:
-  case FL_FOCUS:
-  case FL_UNFOCUS:
     return 1;
 
   case FL_SHORTCUT:
@@ -461,12 +405,53 @@ int Colorbar_Window::handle(int event)
       ColorTable_InitParam(9, ct);
       compute = 1;
     }
-    else if(Fl::test_shortcut('c')) {
+    else if(Fl::test_shortcut(FL_CTRL + '0')) {
+      ColorTable_InitParam(10, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '1')) {
+      ColorTable_InitParam(11, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '2')) {
+      ColorTable_InitParam(12, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '3')) {
+      ColorTable_InitParam(13, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '4')) {
+      ColorTable_InitParam(14, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '5')) {
+      ColorTable_InitParam(15, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '6')) {
+      ColorTable_InitParam(16, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '7')) {
+      ColorTable_InitParam(17, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '8')) {
+      ColorTable_InitParam(18, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + '9')) {
+      ColorTable_InitParam(19, ct);
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + 'c')) {
       ColorTable_Copy(ct);
     }
-    else if(Fl::test_shortcut('p')) {
+    else if(Fl::test_shortcut(FL_CTRL + 'v')) {
       ColorTable_Paste(ct);
       draw();
+      *viewchanged = 1;
     }
     else if(Fl::test_shortcut('h')) {
       help_flag = !help_flag;
@@ -492,55 +477,63 @@ int Colorbar_Window::handle(int event)
       compute = 1;
     }
     else if(Fl::test_shortcut('b')) {
-      ct->fpar[COLORTABLE_BETA] += 0.05;
-      if(ct->fpar[COLORTABLE_BETA] > 1.0)
-        ct->fpar[COLORTABLE_BETA] = 1.0;
+      ct->dpar[COLORTABLE_BETA] += 0.05;
+      if(ct->dpar[COLORTABLE_BETA] > 1.0)
+        ct->dpar[COLORTABLE_BETA] = 1.0;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_CTRL + 'b')) {
-      ct->fpar[COLORTABLE_BETA] -= 0.05;
-      if(ct->fpar[COLORTABLE_BETA] < -1.0)
-        ct->fpar[COLORTABLE_BETA] = -1.0;
+      ct->dpar[COLORTABLE_BETA] -= 0.05;
+      if(ct->dpar[COLORTABLE_BETA] < -1.0)
+        ct->dpar[COLORTABLE_BETA] = -1.0;
+      compute = 1;
+    }
+    else if(Fl::test_shortcut('a')) {
+      ct->dpar[COLORTABLE_ALPHA] -= 0.05;
+      if(ct->dpar[COLORTABLE_ALPHA] < 0.0)
+        ct->dpar[COLORTABLE_ALPHA] = 0.0;
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + 'a')) {
+      ct->dpar[COLORTABLE_ALPHA] += 0.05;
+      if(ct->dpar[COLORTABLE_ALPHA] > 1.0)
+        ct->dpar[COLORTABLE_ALPHA] = 1.0;
+      compute = 1;
+    }
+    else if(Fl::test_shortcut('p')) {
+      ct->dpar[COLORTABLE_ALPHAPOW] += 0.05;
+      compute = 1;
+    }
+    else if(Fl::test_shortcut(FL_CTRL + 'p')) {
+      ct->dpar[COLORTABLE_ALPHAPOW] -= 0.05;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_Left)) {
-      ct->fpar[COLORTABLE_BIAS] -= 0.05;
+      ct->dpar[COLORTABLE_BIAS] -= 0.05;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_CTRL + FL_Left)) {
-      ct->ipar[COLORTABLE_ROTATE] += 5;
-      if(ct->ipar[COLORTABLE_ROTATE] > ct->size - 1)
-        ct->ipar[COLORTABLE_ROTATE] -= ct->size - 1;
+      ct->ipar[COLORTABLE_ROTATION] += 5;
+      if(ct->ipar[COLORTABLE_ROTATION] > ct->size - 1)
+        ct->ipar[COLORTABLE_ROTATION] -= ct->size - 1;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_Right)) {
-      ct->fpar[COLORTABLE_BIAS] += 0.05;
+      ct->dpar[COLORTABLE_BIAS] += 0.05;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_CTRL + FL_Right)) {
-      ct->ipar[COLORTABLE_ROTATE] -= 5;
-      if(ct->ipar[COLORTABLE_ROTATE] < -(ct->size - 1))
-        ct->ipar[COLORTABLE_ROTATE] += ct->size - 1;
+      ct->ipar[COLORTABLE_ROTATION] -= 5;
+      if(ct->ipar[COLORTABLE_ROTATION] < -(ct->size - 1))
+        ct->ipar[COLORTABLE_ROTATION] += ct->size - 1;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_Up)) {
-      ct->fpar[COLORTABLE_CURVE] -= 0.05;
-      compute = 1;
-    }
-    else if(Fl::test_shortcut(FL_CTRL + FL_Up)) {
-      ct->fpar[COLORTABLE_ALPHAPOW] += 0.05;
-      if(ct->fpar[COLORTABLE_ALPHAPOW] > 1.0)
-        ct->fpar[COLORTABLE_ALPHAPOW] = 1.0;
+      ct->dpar[COLORTABLE_CURVATURE] -= 0.05;
       compute = 1;
     }
     else if(Fl::test_shortcut(FL_Down)) {
-      ct->fpar[COLORTABLE_CURVE] += 0.05;
-      compute = 1;
-    }
-    else if(Fl::test_shortcut(FL_CTRL + FL_Down)) {
-      ct->fpar[COLORTABLE_ALPHAPOW] -= 0.05;
-      if(ct->fpar[COLORTABLE_ALPHAPOW] < 0.0)
-        ct->fpar[COLORTABLE_ALPHAPOW] = 0.0;
+      ct->dpar[COLORTABLE_CURVATURE] += 0.05;
       compute = 1;
     }
     else {
