@@ -1,7 +1,7 @@
 #ifndef _GUI_H_
 #define _GUI_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Menu_Bar.H>
 #include <FL/fl_draw.H>
@@ -54,7 +55,6 @@
 
 #include "Opengl_Window.h"
 #include "Colorbar_Window.h"
-#include "File_Picker.h"
 
 // The dynamic contexts
 
@@ -94,13 +94,15 @@ class GMSH_Plugin;
 
 // A generalized dialogbox for plugins
 
+#define MAX_PLUGIN_OPTIONS 50
+
 struct PluginDialogBox
 {
   int current_view_index;
   Fl_Window *main_window;
   Fl_Return_Button *run_button;
-  Fl_Value_Input *value[20];
-  Fl_Input *input[20] ;
+  Fl_Value_Input *value[MAX_PLUGIN_OPTIONS];
+  Fl_Input *input[MAX_PLUGIN_OPTIONS] ;
 };
 
 // The dialog for solvers
@@ -130,9 +132,6 @@ public:
 
   Fl_Bitmap  *ortho_bmp, *persp_bmp;
 
-  // file chooser
-  File_Picker *fc;
-
   // menu window
   Fl_Window        *m_window ;
 #if defined(__APPLE__) && defined(HAVE_FLTK_1_1_5_OR_ABOVE)
@@ -163,7 +162,7 @@ public:
   Fl_Group         *gen_group ;
   Fl_Check_Button  *gen_butt[20] ;
   Fl_Button        *gen_push_butt[20] ;
-  Fl_Value_Input   *gen_value[20] ;
+  Fl_Value_Input   *gen_value[50] ;
   Fl_Button        *gen_col[50] ;
   Fl_Input         *gen_input[20] ;
   Fl_Choice        *gen_choice[20] ;
@@ -177,6 +176,7 @@ public:
   
   // mesh options
   Fl_Group         *mesh_group ;
+  Fl_Group         *mesh_cut_plane ;
   Fl_Check_Button  *mesh_butt[50] ;
   Fl_Input         *mesh_input[20] ;
   Fl_Value_Input   *mesh_value[20] ;
@@ -187,6 +187,7 @@ public:
   Fl_Group         *solver_group ;
   Fl_Check_Button  *solver_butt[20] ;
   Fl_Value_Input   *solver_value[20] ;
+  Fl_Input         *solver_input[20] ;
 
   // post-processing options
   Fl_Group         *post_group ;
@@ -197,13 +198,14 @@ public:
   // view options
   int view_number ;
   Fl_Group         *view_group ;
-  Fl_Group         *view_2d, *view_range, *view_vector ;
+  Fl_Group         *view_range, *view_vector ;
   Fl_Check_Button  *view_butt[100] ;
   Fl_Value_Input   *view_value[100] ;
   Fl_Input         *view_input[100] ;
   Fl_Repeat_Button *view_butt_rep[100] ;
   Fl_Button        *view_push_butt[100] ;
   Fl_Choice        *view_choice[100] ;
+  Fl_Button        *view_col[50] ;
   Colorbar_Window  *view_colorbar_window ;
   Fl_Return_Button *view_ok ;
 
@@ -220,7 +222,7 @@ public:
   Fl_Choice        *vis_type, *vis_browser_mode, *vis_input_mode ;
   Fl_Browser       *vis_browser ;
   Fl_Check_Button  *vis_butt[20] ;
-  Fl_Input         *vis_input;
+  Fl_Input         *vis_input[10];
 
   // clipping planes window
   Fl_Window        *clip_window ;
@@ -228,6 +230,10 @@ public:
   Fl_Multi_Browser *clip_browser ;
   Fl_Value_Input   *clip_value[4];
 
+  // manipulator window
+  Fl_Window        *manip_window ;
+  Fl_Value_Input   *manip_value[9] ;
+
   // about window
   Fl_Window        *about_window ;
 
@@ -250,8 +256,8 @@ public:
   GUI(int argc, char **argv);
 
   // create the windows
-  void create_menu_window(int argc, char **argv);
-  void create_graphic_window(int argc, char **argv);
+  void create_menu_window();
+  void create_graphic_window();
   void create_option_window();
   void hide_all_option_groups();
   void create_general_options_window();
@@ -263,6 +269,7 @@ public:
   void create_view_options_window(int numview);
   void create_visibility_window();
   void create_clip_window();
+  void create_manip_window();
   void create_statistics_window();
   void create_message_window();
   void create_about_window();
@@ -282,7 +289,6 @@ public:
   int  get_context();
   void set_anim_buttons(int mode);
   void check_anim_buttons();
-  void check_rotation_center_button();
   void set_status(char *msg, int num);
   void add_message(char *msg);
   void save_message(char *filename);
@@ -296,10 +302,17 @@ public:
   void reset_visibility();
   void reset_option_browser();
   void reset_clip_browser();
+  void update_manip_window(int force=0);
   void reset_external_view_list();
   int  selection, try_selection, quit_selection, end_selection, undo_selection;
 
 };
 
+// some utility font functions
+int GetFontIndex(char *fontname);
+int GetFontEnum(int index);
+char *GetFontName(int index);
+int GetFontAlign(char *alignstr);
+
 #endif
 
