@@ -1,6 +1,6 @@
-// $Id: Opengl.cpp,v 1.43 2004/10/28 06:11:22 geuzaine Exp $
+// $Id: Opengl.cpp,v 1.51 2005/03/11 08:56:38 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,9 +37,6 @@ void Process_SelectionBuffer(int x, int y, int *n, GLuint * ii, GLuint * jj);
 void Filter_SelectionBuffer(int n, GLuint * typ, GLuint * ient,
                             Vertex ** thev, Curve ** thec, Surface ** thes,
                             Mesh * m);
-void myZoom(GLdouble X1, GLdouble X2, GLdouble Y1, GLdouble Y2, GLdouble Xc1,
-            GLdouble Xc2, GLdouble Yc1, GLdouble Yc2);
-
 // Draw specialization
 
 void InitOpengl(void)
@@ -66,23 +63,124 @@ void Draw(void)
   WID->redraw_opengl();
 }
 
-void Draw_String(char *s)
+void SanitizeTeXString(char *in, char *out)
 {
+  // if there is a '$' or a '\' in the string, assume the author knows
+  // what he's doing:
+  if(strstr(in, "$") || strstr(in, "\\")){
+    strcpy(out, in);
+    return;
+  }
+  // otherwise, escape the following special characters:
+  char bad[8] = { '%', '^', '#', '%', '&', '_', '{', '}' };
+  while(*in){
+    for(unsigned int i = 0; i < sizeof(bad); i++){
+      if(*in == bad[i]){
+	*out++ = '\\';
+	break;
+      }
+    }
+    *out++ = *in++;
+  }
+  *out = '\0';
+}
+
+void Draw_String(char *s, char *font_name, int font_enum, int font_size, int align)
+{
+  if(align > 0){
+    // change the raster position only if not creating TeX files
+    if(CTX.print.gl_fonts || (CTX.print.format != FORMAT_TEX)){
+      GLboolean valid;
+      glGetBooleanv(GL_CURRENT_RASTER_POSITION_VALID, &valid);
+      if(valid == GL_TRUE){
+	GLdouble pos[4];
+	glGetDoublev(GL_CURRENT_RASTER_POSITION, pos);
+	double x[3], w[3] = {pos[0], pos[1], pos[2]};
+	gl_font(font_enum, font_size);
+	float width = gl_width(s);
+	float height = gl_height();
+	switch(align){
+	case 1: w[0] -= width/2.;                     break; // bottom center
+	case 2: w[0] -= width;                        break; // bottom right
+	case 3:                    w[1] -= height;    break; // top left
+	case 4: w[0] -= width/2.;  w[1] -= height;    break; // top center
+	case 5: w[0] -= width;     w[1] -= height;    break; // top right
+	case 6:                    w[1] -= height/2.; break; // center left
+	case 7: w[0] -= width/2.;  w[1] -= height/2.; break; // center center
+	case 8: w[0] -= width;     w[1] -= height/2.; break; // center right
+	default: break;
+	}
+	Viewport2World(w, x);
+	glRasterPos3d(x[0], x[1], x[2]);
+      }
+    }
+  }
+  
   if(CTX.print.gl_fonts) {
-    gl_font(CTX.gl_font_enum, CTX.gl_fontsize);
+    gl_font(font_enum, font_size);
     gl_draw(s);
   }
-  else {        // ps or *tex output
+  else { // ps, pdf or *tex output
     if(CTX.print.format == FORMAT_JPEGTEX ||
        CTX.print.format == FORMAT_PNGTEX)
       return;
-    gl2psText(s, CTX.gl_font, CTX.gl_fontsize);
+    if(CTX.print.format == FORMAT_TEX){
+      char tmp[1024];
+      SanitizeTeXString(s, tmp);
+      int opt;
+      switch(align){
+      case 1: opt = GL2PS_TEXT_B;   break; // bottom center
+      case 2: opt = GL2PS_TEXT_BR;  break; // bottom right
+      case 3: opt = GL2PS_TEXT_TL;  break; // top left
+      case 4: opt = GL2PS_TEXT_T;   break; // top center
+      case 5: opt = GL2PS_TEXT_TR;  break; // top right
+      case 6: opt = GL2PS_TEXT_CL;  break; // center left
+      case 7: opt = GL2PS_TEXT_C;   break; // center center
+      case 8: opt = GL2PS_TEXT_CR;  break; // center right
+      default: opt = GL2PS_TEXT_BL; break; // bottom left
+      }
+      gl2psTextOpt(tmp, font_name, font_size, opt, 0.);
+    }
+    else
+      gl2psText(s, font_name, font_size);
+  }
+}
+
+void Draw_String(char *s)
+{
+  Draw_String(s, CTX.gl_font, CTX.gl_font_enum, CTX.gl_fontsize, 0);
+}
+
+void Draw_String_Center(char *s)
+{
+  Draw_String(s, CTX.gl_font, CTX.gl_font_enum, CTX.gl_fontsize, 1);
+}
+
+void Draw_String_Right(char *s)
+{
+  Draw_String(s, CTX.gl_font, CTX.gl_font_enum, CTX.gl_fontsize, 2);
+}
+
+void Draw_String(char *s, double style)
+{
+  unsigned int bits = (unsigned int)style;
+
+  if(!bits){ // use defaults
+    Draw_String(s);
+  }
+  else{
+    int size = (bits & 0xff);
+    int font = (bits>>8 & 0xff);
+    int align = (bits>>16 & 0xff);
+    int font_enum = GetFontEnum(font);
+    char *font_name = GetFontName(font);
+    if(!size) size = CTX.gl_fontsize;
+    Draw_String(s, font_name, font_enum, size, align);
   }
 }
 
 void Draw_OnScreenMessages()
 {
-
   glColor4ubv((GLubyte *) & CTX.color.text);
   gl_font(CTX.gl_font_enum, CTX.gl_fontsize);
   double h = gl_height();
@@ -99,31 +197,6 @@ void Draw_OnScreenMessages()
   }
 }
 
-// Euler angles set_XXX
-
-void set_r(int i, double val)
-{
-  if(!CTX.useTrackball) {
-    if(!CTX.rlock[i]) {
-      CTX.r[i] = val;
-    }
-  }
-}
-
-void set_t(int i, double val)
-{
-  if(!CTX.tlock[i]) {
-    CTX.t[i] = val;
-  }
-}
-
-void set_s(int i, double val)
-{
-  if(!CTX.slock[i]) {
-    CTX.s[i] = val;
-  }
-}
-
 // Select entity routines
 
 int check_type(int type, Vertex * v, Curve * c, Surface * s)
