@@ -1,6 +1,6 @@
-// $Id: Opengl_Window.cpp,v 1.40 2004/10/28 03:44:37 geuzaine Exp $
+// $Id: Opengl_Window.cpp,v 1.47 2005/03/11 08:56:38 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,8 +39,6 @@ void Process_SelectionBuffer(int x, int y, int *n, GLuint * ii, GLuint * jj);
 void Filter_SelectionBuffer(int n, GLuint * typ, GLuint * ient,
                             Vertex ** thev, Curve ** thec, Surface ** thes,
                             Mesh * m);
-void myZoom(GLdouble X1, GLdouble X2, GLdouble Y1, GLdouble Y2, GLdouble Xc1,
-            GLdouble Xc2, GLdouble Yc1, GLdouble Yc2);
 int check_type(int type, Vertex * v, Curve * c, Surface * s);
 
 void Opengl_Window::draw()
@@ -70,21 +68,18 @@ void Opengl_Window::draw()
     }
   }
 
-  if(!ZOOM) {
+  if(!ZoomMode) {
     Orthogonalize(0, 0);
     ClearOpengl();
     Draw3d();
     Draw2d();
   }
   else {
-    glPopMatrix();
     glDisable(GL_DEPTH_TEST);
     glMatrixMode(GL_PROJECTION);
-    glPushMatrix();
     glLoadIdentity();
     gluOrtho2D(CTX.vxmin, CTX.vxmax, CTX.vymin, CTX.vymax);
     glMatrixMode(GL_MODELVIEW);
-    glPushMatrix();
     glLoadIdentity();
     glDisable(GL_DEPTH_TEST);
     glColor3f(1., 1., 1.);
@@ -92,36 +87,51 @@ void Opengl_Window::draw()
     glEnable(GL_BLEND);
     glLineWidth(0.2);
     glBegin(GL_LINE_STRIP);
-    glVertex2d(ZOOM_X0, ZOOM_Y0);
-    glVertex2d(ZOOM_X1, ZOOM_Y0);
-    glVertex2d(ZOOM_X1, ZOOM_Y1);
-    glVertex2d(ZOOM_X0, ZOOM_Y1);
-    glVertex2d(ZOOM_X0, ZOOM_Y0);
+    glVertex2d(xzoom0, yzoom0);
+    glVertex2d(xzoom1, yzoom0);
+    glVertex2d(xzoom1, yzoom1);
+    glVertex2d(xzoom0, yzoom1);
+    glVertex2d(xzoom0, yzoom0);
     glEnd();
-    ZOOM_X1 =
-      CTX.vxmin + ((double)Fl::event_x() / (double)w()) * (CTX.vxmax -
-                                                           CTX.vxmin);
-    ZOOM_Y1 =
-      CTX.vymax - ((double)Fl::event_y() / (double)h()) * (CTX.vymax -
-                                                           CTX.vymin);
+    xzoom1 = CTX.vxmin + ((double)Fl::event_x() / (double)w()) * 
+      (CTX.vxmax - CTX.vxmin);
+    yzoom1 = CTX.vymax - ((double)Fl::event_y() / (double)h()) *
+      (CTX.vymax - CTX.vymin);
     glBegin(GL_LINE_STRIP);
-    glVertex2d(ZOOM_X0, ZOOM_Y0);
-    glVertex2d(ZOOM_X1, ZOOM_Y0);
-    glVertex2d(ZOOM_X1, ZOOM_Y1);
-    glVertex2d(ZOOM_X0, ZOOM_Y1);
-    glVertex2d(ZOOM_X0, ZOOM_Y0);
+    glVertex2d(xzoom0, yzoom0);
+    glVertex2d(xzoom1, yzoom0);
+    glVertex2d(xzoom1, yzoom1);
+    glVertex2d(xzoom0, yzoom1);
+    glVertex2d(xzoom0, yzoom0);
     glEnd();
     glDisable(GL_BLEND);
     glEnable(GL_DEPTH_TEST);
-    glPopMatrix();
-    glMatrixMode(GL_PROJECTION);
-    glPopMatrix();
-    glMatrixMode(GL_MODELVIEW);
-    ZOOM = 0;
+    ZoomMode = false;
   }
   locked = 0;
 }
 
+// FIXME: this is notoriously wrong :-)
+
+void myZoom(GLdouble X1, GLdouble X2, GLdouble Y1, GLdouble Y2,
+            GLdouble Xc1, GLdouble Xc2, GLdouble Yc1, GLdouble Yc2)
+{
+  GLdouble xscale1 = CTX.s[0];
+  GLdouble yscale1 = CTX.s[1];
+  CTX.s[0] *= (CTX.vxmax - CTX.vxmin) / (X2 - X1);
+  CTX.s[1] *= (CTX.vymax - CTX.vymin) / (Y1 - Y2);
+  CTX.s[2] = MIN(CTX.s[0], CTX.s[1]); // bof...
+  CTX.t[0] = CTX.t[0] * (xscale1 / CTX.s[0]) - 
+    ((Xc1 + Xc2) / 2.) * (1. - (xscale1 / CTX.s[0]));
+  CTX.t[1] = CTX.t[1] * (yscale1 / CTX.s[1]) - 
+    ((Yc1 + Yc2) / 2.) * (1. - (yscale1 / CTX.s[1]));
+  
+  WID->update_manip_window();
+  InitPosition();
+  Draw();
+}
+
+
 // The event model in FLTK is pretty different from other toolkits:
 // the events are passed to the widget handle of the widget that has
 // the focus. If this handle returns 1, then the event is considered
@@ -134,16 +144,7 @@ int Opengl_Window::handle(int event)
 
   switch (event) {
 
-  case FL_ENTER:
-    take_focus();       //force keyboard focus on the Opengl_Window
-    return 1;
-
-  case FL_LEAVE:
-    return 1;
-
-  case FL_FOCUS:
-    return 1;
-
+  case FL_FOCUS: // accept the focus when I'm asked if I want it
   case FL_UNFOCUS:
     return 1;
 
@@ -156,27 +157,28 @@ int Opengl_Window::handle(int event)
     return Fl_Gl_Window::handle(event);
 
   case FL_PUSH:
+    take_focus(); // force keyboard focus when we click in the window
     FirstClick = 1;
     ibut = Fl::event_button();
     xpos = Fl::event_x();
     ypos = Fl::event_y();
-
+    
     if(ibut == 1 && !Fl::event_state(FL_SHIFT) && !Fl::event_state(FL_ALT)) {
       if(!ZoomClick && Fl::event_state(FL_CTRL)) {
-        ZOOM_X0 = ZOOM_X1 =
-          CTX.vxmin + ((double)xpos / (double)w()) * (CTX.vxmax - CTX.vxmin);
-        ZOOM_Y0 = ZOOM_Y1 =
-          CTX.vymax - ((double)ypos / (double)h()) * (CTX.vymax - CTX.vymin);
-        xc1 = ZOOM_X0 / CTX.s[0] - CTX.t[0];
-        yc1 = ZOOM_Y0 / CTX.s[1] - CTX.t[1];
-        ZoomClick = 1;
+        xzoom0 = xzoom1 = CTX.vxmin + ((double)xpos / (double)w()) *
+	  (CTX.vxmax - CTX.vxmin);
+        yzoom0 = yzoom1 = CTX.vymax - ((double)ypos / (double)h()) * 
+	  (CTX.vymax - CTX.vymin);
+        xc1 = xzoom0 / CTX.s[0] - CTX.t[0];
+        yc1 = yzoom0 / CTX.s[1] - CTX.t[1];
+        ZoomClick = true;
       }
       else if(ZoomClick) {
-        xc2 = ZOOM_X1 / CTX.s[0] - CTX.t[0];
-        yc2 = ZOOM_Y1 / CTX.s[1] - CTX.t[1];
-        ZoomClick = 0;
-        if(ZOOM_X0 != ZOOM_X1 && ZOOM_Y0 != ZOOM_Y1)
-          myZoom(ZOOM_X0, ZOOM_X1, ZOOM_Y0, ZOOM_Y1, xc1, xc2, yc1, yc2);
+        xc2 = xzoom1 / CTX.s[0] - CTX.t[0];
+        yc2 = yzoom1 / CTX.s[1] - CTX.t[1];
+        ZoomClick = false;
+        if(xzoom0 != xzoom1 && yzoom0 != yzoom1)
+          myZoom(xzoom0, xzoom1, yzoom0, yzoom1, xc1, xc2, yc1, yc2);
       }
       else {
         WID->try_selection = 1;
@@ -184,35 +186,29 @@ int Opengl_Window::handle(int event)
     }
     else if(ibut == 2 || (ibut == 1 && Fl::event_state(FL_SHIFT))) {
       if(Fl::event_state(FL_CTRL) && !ZoomClick) {
-        set_s(1, CTX.s[0]);
-        set_s(2, CTX.s[0]);
+        CTX.s[1] = CTX.s[0];
+        CTX.s[2] = CTX.s[0];
         redraw();
       }
       else {
-        ZoomClick = 0;
+        ZoomClick = false;
       }
     }
     else {
       if(Fl::event_state(FL_CTRL) && !ZoomClick) {
         if(CTX.useTrackball)
           CTX.setQuaternion(0., 0., 0., 1.);
-        else {
-          set_r(0, 0.);
-          set_r(1, 0.);
-          set_r(2, 0.);
-        }
-        set_t(0, 0.);
-        set_t(1, 0.);
-        set_t(2, 0.);
-        set_s(0, 1.);
-        set_s(1, 1.);
-        set_s(2, 1.);
+        else
+          CTX.r[0] = CTX.r[1] = CTX.r[2] = 0.;
+	CTX.t[0] = CTX.t[1] = CTX.t[2] = 0.;
+	CTX.s[0] = CTX.s[1] = CTX.s[2] = 1.;
         redraw();
       }
       else {
-        ZoomClick = 0;
+        ZoomClick = false;
       }
     }
+    WID->update_manip_window();
     return 1;
 
   case FL_RELEASE:
@@ -231,7 +227,7 @@ int Opengl_Window::handle(int event)
     ymov = Fl::event_y() - ypos;
 
     if(ZoomClick) {
-      ZOOM = 1;
+      ZoomMode = true;
       redraw();
     }
     else {
@@ -254,26 +250,22 @@ int Opengl_Window::handle(int event)
                             (2.0 * Fl::event_x() - w()) / w(),
                             (h() - 2.0 * Fl::event_y()) / h());
         else {
-          set_r(1, CTX.r[1] + ((abs(xmov) > abs(ymov)) ?
-			       180 * (double)xmov / (double)w() : 0));
-          set_r(0, CTX.r[0] + ((abs(xmov) > abs(ymov)) ? 
-			       0 : 180 * (double)ymov / (double)h()));
+          CTX.r[1] += ((abs(xmov) > abs(ymov)) ? 180 * (double)xmov / (double)w() : 0);
+	  CTX.r[0] += ((abs(xmov) > abs(ymov)) ? 0 : 180 * (double)ymov / (double)h());
         }
       }
       else if(ibut == 2 || (ibut == 1 && Fl::event_state(FL_SHIFT))) {
-        if(!CTX.useTrackball)
-          set_r(2, CTX.r[2] + ((abs(ymov) > abs(xmov)) ? 
-			       0 : -180 * (double)xmov / (double)w()));
-        set_s(0, CTX.s[0] * ((abs(ymov) > abs(xmov)) ? 
-			     ((ymov > 0) ? (double)(CTX.zoom_factor * (abs(ymov) + h())) /
-                              (double)h() : 
-			      (double)(h()) / (double)(CTX.zoom_factor * 
-						       (abs(ymov) + h()))) : 1.));
-        set_s(1, CTX.s[0]);
-        set_s(2, CTX.s[0]);
+	if(!CTX.useTrackball)
+          CTX.r[2] += (abs(ymov) > abs(xmov) ? 0 : -180 * (double)xmov / (double)w());
+	double zoomfact = (ymov > 0) ? 
+	  (double)(CTX.zoom_factor * abs(ymov) + h()) / (double)h() : 
+	  (double)(h()) / (double)(CTX.zoom_factor * abs(ymov) + h());
+	CTX.s[0] *= (abs(ymov) > abs(xmov) ? zoomfact : 1.);
+        CTX.s[1] = CTX.s[0];
+        CTX.s[2] = CTX.s[0];
         if(abs(ymov) > abs(xmov)) {
-          set_t(0, xt1 * (xscale1 / CTX.s[0]) - xc1 * (1. - (xscale1 / CTX.s[0])));
-          set_t(1, yt1 * (yscale1 / CTX.s[1]) - yc1 * (1. - (yscale1 / CTX.s[1])));
+          CTX.t[0] = xt1 * (xscale1 / CTX.s[0]) - xc1 * (1. - (xscale1 / CTX.s[0]));
+          CTX.t[1] = yt1 * (yscale1 / CTX.s[1]) - yc1 * (1. - (yscale1 / CTX.s[1]));
         }
       }
       else {
@@ -281,9 +273,9 @@ int Opengl_Window::handle(int event)
           / CTX.s[0];
         yc = (CTX.vymax - ((double)ypos / (double)h()) * (CTX.vymax - CTX.vymin))
           / CTX.s[1];
-        set_t(0, xc - xc1);
-        set_t(1, yc - yc1);
-        set_t(2, 0.);
+        CTX.t[0] = xc - xc1;
+        CTX.t[1] = yc - yc1;
+        CTX.t[2] = 0.;
       }
 
       if(CTX.fast_redraw) {
@@ -296,15 +288,34 @@ int Opengl_Window::handle(int event)
 
     xpos += xmov;
     ypos += ymov;
+    WID->update_manip_window();
     return 1;
 
-
   case FL_MOVE:
     xmov = Fl::event_x() - xpos;
     ymov = Fl::event_y() - ypos;
+    
+    if(AddPointMode && !Fl::event_state(FL_SHIFT)){
+      WID->g_opengl_window->cursor(FL_CURSOR_CROSS, FL_BLACK, FL_WHITE);
+      // find line in real space corresponding to current cursor position
+      double p[3],d[3];
+      unproject(Fl::event_x(), Fl::event_y(), p, d);
+      // fin closest point to the center of gravity
+      double r[3] = {CTX.cg[0]-p[0], CTX.cg[1]-p[1], CTX.cg[2]-p[2]};
+      double t;
+      prosca(r,d,&t);
+      double sol[3] = {p[0]+t*d[0], p[1]+t*d[1], p[2]+t*d[2]};
+      char str[32];
+      sprintf(str, "%g", sol[0]);        
+      WID->context_geometry_input[2]->value(str);
+      sprintf(str, "%g", sol[1]);
+      WID->context_geometry_input[3]->value(str);
+      sprintf(str, "%g", sol[2]);
+      WID->context_geometry_input[4]->value(str);
+    }
 
     if(ZoomClick) {
-      ZOOM = 1;
+      ZoomMode = true;
       redraw();
     }
     else {
