@@ -1,7 +1,7 @@
 #ifndef _OPENGL_WINDOW_H_
 #define _OPENGL_WINDOW_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,16 +26,15 @@
 
 class Opengl_Window : public Fl_Gl_Window {
  public:
-  int xpos, ypos;
+  bool AddPointMode;
  private:
-  int xmov, ymov, ibut, hits;
-  int ZoomClick, FirstClick;
+  bool ZoomMode, ZoomClick, FirstClick;
+  int xpos, ypos, xmov, ymov, ibut, hits;
+  double xzoom0, yzoom0, xzoom1, yzoom1;
   GLdouble xc, yc, xc1, yc1, xc2, yc2, xt1, yt1, xscale1, yscale1;
   Vertex *v, *ov;
   Curve *c, *oc;
   Surface *s, *os;
-  int ZOOM;
-  double ZOOM_X0, ZOOM_Y0, ZOOM_X1, ZOOM_Y1;
 
   void draw();
   int handle(int);
@@ -44,12 +43,11 @@ class Opengl_Window : public Fl_Gl_Window {
   Opengl_Window(int x,int y,int w,int h,const char *l=0)
     : Fl_Gl_Window(x, y, w, h, l) {
     xpos = ypos = xmov = ymov = ibut = hits = 0;
-    ZoomClick = FirstClick = 0;
+    xzoom0 = yzoom0 = xzoom1 = yzoom1 = 0.;
+    AddPointMode = ZoomMode = ZoomClick = FirstClick = false;
     v = ov = NULL;
     c = oc = NULL;
     s = os = NULL;
-    ZOOM = 0;
-    ZOOM_X0 = ZOOM_Y0 = ZOOM_X1 = ZOOM_Y1 = 0.;
   }
 };
 
