@@ -1,6 +1,6 @@
-// $Id: CAD.cpp,v 1.79 2004/09/29 21:08:25 geuzaine Exp $
+// $Id: CAD.cpp,v 1.85 2005/03/01 00:01:12 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -109,9 +109,13 @@ int compare2Lists(List_T * List1, List_T * List2,
 {
   int i, found;
 
-  if(List_Nbr(List1) != List_Nbr(List2))
-    return List_Nbr(List1) - List_Nbr(List2);
+  if(!List_Nbr(List1) && !List_Nbr(List2))
+    return 0;
 
+  if(!List_Nbr(List1) || !List_Nbr(List2) || 
+     (List_Nbr(List1) != List_Nbr(List2)))
+    return List_Nbr(List1) - List_Nbr(List2);
+  
   List_T *List1Prime = List_Create(List_Nbr(List1), 1, List1->size);
   List_T *List2Prime = List_Create(List_Nbr(List2), 1, List2->size);
   List_Copy(List1, List1Prime);
@@ -236,8 +240,8 @@ void CopyVertex(Vertex * v, Vertex * vv)
 
 Vertex *DuplicateVertex(Vertex * v)
 {
-  Vertex *pv;
-  pv = Create_Vertex(NEWPOINT(), 0, 0, 0, 0, 0);
+  if(!v) return NULL;
+  Vertex *pv = Create_Vertex(NEWPOINT(), 0, 0, 0, 0, 0);
   CopyVertex(v, pv);
   Tree_Insert(THEM->Points, &pv);
   return pv;
@@ -597,20 +601,23 @@ Curve *CreateReversedCurve(Mesh * M, Curve * c)
   Vertex *e1, *e2, *e3, *e4;
   int i;
   newc = Create_Curve(-c->Num, c->Typ, 1, NULL, NULL, -1, -1, 0., 1.);
-  newc->Control_Points =
-    List_Create(List_Nbr(c->Control_Points), 1, sizeof(Vertex *));
-  if(c->Typ == MSH_SEGM_ELLI || c->Typ == MSH_SEGM_ELLI_INV) {
-    List_Read(c->Control_Points, 0, &e1);
-    List_Read(c->Control_Points, 1, &e2);
-    List_Read(c->Control_Points, 2, &e3);
-    List_Read(c->Control_Points, 3, &e4);
-    List_Add(newc->Control_Points, &e4);
-    List_Add(newc->Control_Points, &e2);
-    List_Add(newc->Control_Points, &e3);
-    List_Add(newc->Control_Points, &e1);
+
+  if(List_Nbr(c->Control_Points)){
+    newc->Control_Points =
+      List_Create(List_Nbr(c->Control_Points), 1, sizeof(Vertex *));
+    if(c->Typ == MSH_SEGM_ELLI || c->Typ == MSH_SEGM_ELLI_INV) {
+      List_Read(c->Control_Points, 0, &e1);
+      List_Read(c->Control_Points, 1, &e2);
+      List_Read(c->Control_Points, 2, &e3);
+      List_Read(c->Control_Points, 3, &e4);
+      List_Add(newc->Control_Points, &e4);
+      List_Add(newc->Control_Points, &e2);
+      List_Add(newc->Control_Points, &e3);
+      List_Add(newc->Control_Points, &e1);
+    }
+    else
+      List_Invert(c->Control_Points, newc->Control_Points);
   }
-  else
-    List_Invert(c->Control_Points, newc->Control_Points);
 
   if(c->Typ == MSH_SEGM_NURBS && c->k) {
     newc->k =
@@ -628,7 +635,6 @@ Curve *CreateReversedCurve(Mesh * M, Curve * c)
     newc->Typ = MSH_SEGM_ELLI_INV;
   if(c->Typ == MSH_SEGM_ELLI_INV)
     newc->Typ = MSH_SEGM_ELLI;
-  newc->Vertices = List_Create(10, 1, sizeof(Vertex *));
   newc->Method = c->Method;
   newc->degre = c->degre;
   newc->beg = c->end;
@@ -894,6 +900,11 @@ void ApplyTransformationToCurve(double matrix[4][4], Curve * c)
 {
   Vertex *v;
 
+  if(!c->beg || !c->end){
+    Msg(GERROR, "Cannot transform curve with no begin/end points");
+    return;
+  }
+
   ApplyTransformationToPoint(matrix, c->beg);
   ApplyTransformationToPoint(matrix, c->end);
 
@@ -1291,6 +1302,11 @@ int Extrude_ProtudeCurve(int type, int ic,
     return 0;
   }
 
+  if(!pc->beg || !pc->end){
+    Msg(GERROR, "Cannot extrude curve with no begin/end points");
+    return 0;
+  }
+
   Msg(DEBUG, "Extrude Curve %d", ic);
 
   chapeau = DuplicateCurve(pc);
@@ -1608,9 +1624,13 @@ int compareTwoCurves(const void *a, const void *b)
     return List_Nbr(c1->Control_Points) - List_Nbr(c2->Control_Points);
   
   if(!List_Nbr(c1->Control_Points)){
+    if(!c1->beg || !c2->beg)
+      return 1;
     comp = compareVertex(&c1->beg, &c2->beg);
     if(comp)
       return comp;
+    if(!c1->end || !c2->end)
+      return 1;
     comp = compareVertex(&c1->end, &c2->end);
     if(comp)
       return comp;
