@@ -1,7 +1,7 @@
 #ifndef _EXTRACT_CONTOUR_H_
 #define _EXTRACT_CONTOUR_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "List.h"
 
-int alledgeslinked(int ed, List_T * Liste, List_T * old);
-int allfaceslinked(int iz, List_T * Liste, List_T * old);
+int allEdgesLinked(int ed, List_T * Liste);
+int allFacesLinked(int iz, List_T * Liste);
 
 #endif
