@@ -1,7 +1,7 @@
 #ifndef _GEO_H_
 #define _GEO_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -67,7 +67,6 @@
 #define MSH_SURF_TORUS       1399
 #define MSH_SURF_CONE        1499
 #define MSH_SURF_TRIMMED     1599
-#define MSH_SURF_STL         1699
 #define MSH_SURF_DISCRETE    1799
 
 #define MSH_VOLUME                   14
@@ -95,14 +94,14 @@ typedef struct {
 
 double evaluate_scalarfunction (char *var, double val, char *funct);
 
+void coherence(char *fich);
 void delet(int p1, char *fich, char *what);
 void add_infile(char *text, char *fich);
-void add_trsfsurf(int N, int *l, char *fich);
+void add_trsfline(int N, int *l, char *fich, char *type, char *typearg, char *pts);
+void add_trsfellisurf(int type, int N, int *l, char *fich);
 void add_trsfvol(int N, int *l, char *fich, char *vol);
-void add_ellipticsurf(int N, int *l, char *fich);
 void add_charlength(int N, int *l, char *fich, char *lc);
 void add_recosurf(int N, int *l, char *fich);
-void add_trsfline(int N, int *l, char *fich, char *type, char *typearg, char *pts);
 void add_param(char *par, char *value, char *fich);
 void add_point(char *fich, char *x, char *y, char *z, char *lc);
 void add_attractor(char *fich, int ip, int typ);
