@@ -1,6 +1,6 @@
-// $Id: CreateFile.cpp,v 1.61 2004/05/25 04:10:03 geuzaine Exp $
+// $Id: CreateFile.cpp,v 1.68 2005/03/09 02:18:40 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -78,8 +78,10 @@ void CreateOutputFile(char *name, int format)
     else if(!strcmp(ext, ".opt"))  CreateOutputFile(name, FORMAT_OPT);
     else if(!strcmp(ext, ".msh"))  CreateOutputFile(name, FORMAT_MSH);
     else if(!strcmp(ext, ".unv"))  CreateOutputFile(name, FORMAT_UNV);
+    else if(!strcmp(ext, ".p3d"))  CreateOutputFile(name, FORMAT_P3D);
     else if(!strcmp(ext, ".dmg"))  CreateOutputFile(name, FORMAT_DMG);
-    else if(!strcmp(ext, ".pos"))  CreateOutputFile(name, FORMAT_LC_SUR);
+    else if(!strcmp(ext, ".stl"))  CreateOutputFile(name, FORMAT_STL);
+    else if(!strcmp(ext, ".pos"))  CreateOutputFile(name, FORMAT_LC);
     else if(!strcmp(ext, ".gif"))  CreateOutputFile(name, FORMAT_GIF);
     else if(!strcmp(ext, ".jpg"))  CreateOutputFile(name, FORMAT_JPEG);
     else if(!strcmp(ext, ".jpeg")) CreateOutputFile(name, FORMAT_JPEG);
@@ -105,24 +107,21 @@ void CreateOutputFile(char *name, int format)
     break;
 
   case FORMAT_OPT:
-    Print_Options(0, GMSH_FULLRC, false, name);
+    Print_Options(0, GMSH_FULLRC, true, name);
     break;
 
   case FORMAT_MSH:
   case FORMAT_UNV:
+  case FORMAT_P3D:
   case FORMAT_DMG:
+  case FORMAT_STL:
   case FORMAT_GREF:
   case FORMAT_VRML:
     Print_Mesh(&M, name, format);
     break;
 
-  case FORMAT_LC_SUR:
-    ExportLcFieldOnSurfaces(&M, name);
-    Msg(STATUS2N, "Wrote '%s'", name);
-    break;
-
-  case FORMAT_LC_VOL:
-    ExportLcFieldOnVolume(&M, name);
+  case FORMAT_LC:
+    ExportLcField(&M, name);
     Msg(STATUS2N, "Wrote '%s'", name);
     break;
 
@@ -141,7 +140,8 @@ void CreateOutputFile(char *name, int format)
     CTX.print.gl_fonts = 1;
     if(format == FORMAT_JPEG || format == FORMAT_JPEGTEX){
       Msg(INFO, "Writing JPEG file '%s'", name);
-      create_jpeg(fp, width, height, CTX.print.jpeg_quality);
+      create_jpeg(fp, width, height, 
+		  CTX.print.jpeg_quality, CTX.print.jpeg_smoothing);
       Msg(INFO, "Wrote JPEG file '%s'", name);
     }
     else{
@@ -187,15 +187,10 @@ void CreateOutputFile(char *name, int format)
     break;
 
   case FORMAT_PS:
-  case FORMAT_PS_RASTER:
   case FORMAT_EPS:
-  case FORMAT_EPS_RASTER:
   case FORMAT_EPSTEX:
-  case FORMAT_EPSTEX_RASTER:
   case FORMAT_PDF:
-  case FORMAT_PDF_RASTER:
   case FORMAT_PDFTEX:
-  case FORMAT_PDFTEX_RASTER:
     if(!(fp = fopen(name, "wb"))) {
       Msg(GERROR, "Unable to open file '%s'", name);
       return;
@@ -203,13 +198,10 @@ void CreateOutputFile(char *name, int format)
 
     switch(format){
     case FORMAT_PDF:
-    case FORMAT_PDF_RASTER:
     case FORMAT_PDFTEX:
-    case FORMAT_PDFTEX_RASTER:
       psformat = GL2PS_PDF;
       break;
     case FORMAT_PS:
-    case FORMAT_PS_RASTER:
       psformat = GL2PS_PS;
       break;
     default:
@@ -218,21 +210,17 @@ void CreateOutputFile(char *name, int format)
     }
 
     {
-      int raster = 0;
       float *pixels = NULL;
-      if(format == FORMAT_PS_RASTER || 
-	 format == FORMAT_EPS_RASTER || format == FORMAT_EPSTEX_RASTER ||
-	 format == FORMAT_PDF_RASTER || format == FORMAT_PDFTEX_RASTER){
-	if(format == FORMAT_EPSTEX_RASTER || format == FORMAT_PDFTEX_RASTER)
+      if(CTX.print.eps_quality == 0){
+	if(format == FORMAT_EPSTEX || format == FORMAT_PDFTEX)
 	  CTX.print.gl_fonts = 0;
 	FillBuffer();
 	CTX.print.gl_fonts = 1;
 	pixels = new float[width * height * 3];
 	glReadPixels(0, 0, width, height, GL_RGB, GL_FLOAT, pixels);
-	raster = 1;
       }
       
-      pssort = (CTX.print.eps_quality == 1) ? GL2PS_SIMPLE_SORT : GL2PS_BSP_SORT;
+      pssort = (CTX.print.eps_quality == 2) ? GL2PS_BSP_SORT : GL2PS_SIMPLE_SORT;
       psoptions =
 	GL2PS_SIMPLE_LINE_OFFSET | GL2PS_SILENT | GL2PS_NO_BLENDING |
 	(CTX.print.eps_occlusion_culling ? GL2PS_OCCLUSION_CULL : 0) |
@@ -241,9 +229,7 @@ void CreateOutputFile(char *name, int format)
 	(CTX.print.eps_compress ? GL2PS_COMPRESS : 0) |
 	(CTX.print.eps_ps3shading ? 0 : GL2PS_NO_PS3_SHADING) |
 	(format == FORMAT_EPSTEX ? GL2PS_NO_TEXT : 0) |
-	(format == FORMAT_PDFTEX ? GL2PS_NO_TEXT : 0) |
-	(format == FORMAT_EPSTEX_RASTER ? GL2PS_NO_TEXT : 0) |
-	(format == FORMAT_PDFTEX_RASTER ? GL2PS_NO_TEXT : 0);
+	(format == FORMAT_PDFTEX ? GL2PS_NO_TEXT : 0);
       
       Msg(INFO, "Writing %s file '%s'", (psformat == GL2PS_PDF) ? "PDF" : "PS/EPS", name);
       
@@ -254,13 +240,22 @@ void CreateOutputFile(char *name, int format)
 	gl2psBeginPage(CTX.base_filename, "Gmsh", viewport, 
 		       psformat, pssort, psoptions, GL_RGBA, 0, NULL, 
 		       15, 20, 10, size3d, fp, name);
-	if(raster){
+	if(CTX.print.eps_quality == 0){
+	  double modelview[16], projection[16];
+	  glGetDoublev(GL_PROJECTION_MATRIX, projection);
+	  glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
 	  glMatrixMode(GL_PROJECTION);
 	  glLoadIdentity();
-	  glPushMatrix();
-	  glRasterPos2d(-1., -1.);
+	  glOrtho((double)CTX.viewport[0], (double)CTX.viewport[2],
+		  (double)CTX.viewport[1], (double)CTX.viewport[3], -1., 1.);
+	  glMatrixMode(GL_MODELVIEW);
+	  glLoadIdentity();
+	  glRasterPos2d(0, 0);
 	  gl2psDrawPixels(width, height, 0, 0, GL_RGB, GL_FLOAT, pixels);
-	  glPopMatrix();  
+	  glMatrixMode(GL_PROJECTION);
+	  glLoadMatrixd(projection);
+	  glMatrixMode(GL_MODELVIEW);
+	  glLoadMatrixd(modelview);
 	  delete [] pixels;
 	}
 	else{
