@@ -1,6 +1,6 @@
-// $Id: Draw.cpp,v 1.64 2004/11/01 15:10:36 geuzaine Exp $
+// $Id: Draw.cpp,v 1.76 2005/03/12 07:52:55 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,17 +34,50 @@ extern Mesh M;
 
 // Global Draw functions
 
+int NeedPolygonOffset()
+{
+  if(M.status == 2 &&
+     (CTX.mesh.surfaces_edges || CTX.geom.lines || CTX.geom.surfaces))
+    return 1;
+  if(M.status == 3 && 
+     (CTX.mesh.surfaces_edges || CTX.mesh.volumes_edges))
+    return 1;
+  for(int i = 0; i < List_Nbr(CTX.post.list); i++){
+    Post_View *v = *(Post_View**)List_Pointer(CTX.post.list, i);
+    if(v->Visible){
+      if(v->ShowElement || v->Axes)
+	return 1;
+      if((v->NbST || v->NbSQ) && (v->IntervalsType == DRAW_POST_ISO))
+	return 1;
+    }
+  }
+  return 0;
+}
+
 void Draw3d(void)
 {
-  glPolygonOffset(1.0, 1.0);
+  // We should only enable the polygon offset when there is a mix of
+  // lines and polygons to be drawn; enabling it all the time can lead
+  // to very small but annoying artifacts in the picture. Since there
+  // are so many ways in Gmsh to combine polygons and lines
+  // (geometries + meshes + views...), we do our best here to
+  // automatically detect if we should enable it. Note: the formula
+  // for the offset is "offset = factor*DZ+r*units", where DZ is a
+  // measurement of the change in depth relative to the screen area of
+  // the polygon, and r is the smallest value that is guaranteed to
+  // produce a resolvable offset for a given implementation.
+  glPolygonOffset(CTX.polygon_offset_factor, CTX.polygon_offset_units);
+  if(CTX.polygon_offset_factor || CTX.polygon_offset_units)
+    CTX.polygon_offset = CTX.polygon_offset_always ? 1 : NeedPolygonOffset();
+  else
+    CTX.polygon_offset = 0;
+
   glDepthFunc(GL_LESS);
   glEnable(GL_DEPTH_TEST);
 
   InitRenderModel();
 
-  glPushMatrix();
   Draw_Mesh(&M);
-  glPopMatrix();
 }
 
 void Draw2d(void)
@@ -55,13 +88,12 @@ void Draw2d(void)
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  // to draw directly in screen coords
   glOrtho((double)CTX.viewport[0], (double)CTX.viewport[2],
           (double)CTX.viewport[1], (double)CTX.viewport[3], -1., 1.);
+  // hack to make the 2D primitives appear "in front" in GL2PS
+  glTranslated(0., 0., CTX.clip_factor > 1. ? 1./CTX.clip_factor : CTX.clip_factor);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-
-  glPushMatrix();
   Draw_Graph2D();
   Draw_Text2D();
   Draw_OnScreenMessages();
@@ -69,7 +101,24 @@ void Draw2d(void)
     Draw_Scales();
   if(CTX.small_axes)
     Draw_SmallAxes();
-  glPopMatrix();
+}
+
+void DrawPlugin(void (*draw)(void))
+{
+  CTX.post.plugin_draw_function = draw;
+  int old = CTX.draw_bbox;
+  CTX.draw_bbox = 1;
+  if(CTX.fast_redraw){
+    CTX.post.draw = 0;
+    CTX.mesh.draw = 0;
+  }
+  if(!CTX.batch) 
+    Draw();
+  // this is reset in each plugin run/cancel callback:
+  // CTX.post.plugin_draw_function = NULL;
+  CTX.draw_bbox = old;
+  CTX.post.draw = 1;
+  CTX.mesh.draw = 1;
 }
 
 // Ortho
@@ -127,7 +176,7 @@ void Orthogonalize(int x, int y)
   if(CTX.ortho) {
     double maxz = MAX(fabs(CTX.min[2]), fabs(CTX.max[2]));
     if(maxz < CTX.lc) maxz = CTX.lc;
-    double clip = maxz * CTX.s[2] * 10;
+    double clip = maxz * CTX.s[2] * CTX.clip_factor;
     glOrtho(CTX.vxmin, CTX.vxmax, CTX.vymin, CTX.vymax, -clip, clip);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
@@ -193,12 +242,13 @@ void InitRenderModel(void)
   glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, CTX.shine_exponent);
 
   glShadeModel(GL_SMOOTH);
-#if defined(GL_VERSION_1_2)
-  // this is more efficient, since we already specify unit normals
-  glEnable(GL_RESCALE_NORMAL);
-#else
+  // Normalize the normals automatically. We could use the more
+  // efficient glEnable(GL_RESCALE_NORMAL) instead (since we initially
+  // specify unit normals), but GL_RESCALE_NORMAL does only work with
+  // isotropic scalings (and we allow anistotropic scalings in
+  // myZoom). Note that GL_RESCALE_NORMAL is only available in
+  // GL_VERSION_1_2.
   glEnable(GL_NORMALIZE);
-#endif
 
   // lighting is enabled/disabled for each particular primitive later
   glDisable(GL_LIGHTING);
@@ -216,7 +266,7 @@ void InitPosition(void)
 		 CTX.rotation_center[1],
 		 CTX.rotation_center[2]);
   
-  CTX.buildRotmatrix();
+  CTX.buildRotationMatrix();
   glMultMatrixd(CTX.rot);
 
   if(CTX.rotation_center_cg)
@@ -226,9 +276,11 @@ void InitPosition(void)
 		 -CTX.rotation_center[1],
 		 -CTX.rotation_center[2]);
 
-  // store the modelview and projection matrices
-  glGetDoublev(GL_MODELVIEW_MATRIX, CTX.mod);
+  // store the projection and modelview matrices at this precise
+  // moment (so that we can use them at any later time, even if the
+  // context has changed, i.e., even if we are out of Draw())
   glGetDoublev(GL_PROJECTION_MATRIX, CTX.proj);
+  glGetDoublev(GL_MODELVIEW_MATRIX, CTX.model);
 }
 
 // Entity selection
@@ -302,25 +354,6 @@ void Filter_SelectionBuffer(int n, GLuint * typ, GLuint * ient,
   }
 }
 
-// FIXME: this is notoriously wrong :-)
-
-void myZoom(GLdouble X1, GLdouble X2, GLdouble Y1, GLdouble Y2,
-            GLdouble Xc1, GLdouble Xc2, GLdouble Yc1, GLdouble Yc2)
-{
-  GLdouble xscale1 = CTX.s[0];
-  GLdouble yscale1 = CTX.s[1];
-  set_s(0, CTX.s[0] * (CTX.vxmax - CTX.vxmin) / (X2 - X1));
-  set_s(1, CTX.s[1] * (CTX.vymax - CTX.vymin) / (Y1 - Y2));
-  //set_s(2, 0.5 * (CTX.s[0] + CTX.s[1])); // bof, bof. bof: can cause normal clamping
-  set_s(2, MIN(CTX.s[0], CTX.s[1])); // better, but not great...
-  set_t(0, CTX.t[0] * (xscale1 / CTX.s[0]) - 
-	((Xc1 + Xc2) / 2.) * (1. - (xscale1 / CTX.s[0])));
-  set_t(1, CTX.t[1] * (yscale1 / CTX.s[1]) - 
-	((Yc1 + Yc2) / 2.) * (1. - (yscale1 / CTX.s[1])));
-  InitPosition();
-  Draw();
-}
-
 // Takes a cursor position in window coordinates and returns the line
 // (given by a point and a unit direction vector), in real space, that
 // corresponds to that cursor position
@@ -333,10 +366,13 @@ void unproject(double x, double y, double p[3], double d[3])
   y = viewport[3]-y;
 
   GLdouble x0, y0, z0, x1, y1, z1;
+
+  // we use CTX.model and CTX.proj instead of directly getGetDouble'ing
+  // the matrices since unproject can be called in or after Draw2D
   
-  if(!gluUnProject(x, y, 0.0, CTX.mod, CTX.proj, viewport, &x0, &y0, &z0))
+  if(!gluUnProject(x, y, 0.0, CTX.model, CTX.proj, viewport, &x0, &y0, &z0))
     Msg(WARNING, "unproject1 failed");
-  if(!gluUnProject(x, y, 1.0, CTX.mod, CTX.proj, viewport, &x1, &y1, &z1))
+  if(!gluUnProject(x, y, 1.0, CTX.model, CTX.proj, viewport, &x1, &y1, &z1))
     Msg(WARNING, "unproject2 failed");
   
   p[0] = x0;
@@ -350,3 +386,23 @@ void unproject(double x, double y, double p[3], double d[3])
   d[1] /= len;
   d[2] /= len;
 }
+
+void Viewport2World(double win[3], double xyz[3])
+{
+  GLint viewport[4];
+  GLdouble model[16], proj[16];
+  glGetIntegerv(GL_VIEWPORT, viewport);
+  glGetDoublev(GL_PROJECTION_MATRIX, proj);
+  glGetDoublev(GL_MODELVIEW_MATRIX, model);
+  gluUnProject(win[0], win[1], win[2], model, proj, viewport, &xyz[0], &xyz[1], &xyz[2]);
+}
+
+void World2Viewport(double xyz[3], double win[3])
+{
+  GLint viewport[4];
+  GLdouble model[16], proj[16];
+  glGetIntegerv(GL_VIEWPORT, viewport);
+  glGetDoublev(GL_PROJECTION_MATRIX, proj);
+  glGetDoublev(GL_MODELVIEW_MATRIX, model);
+  gluProject(xyz[0], xyz[1], xyz[2], model, proj, viewport, &win[0], &win[1], &win[2]);
+}
