@@ -1,7 +1,7 @@
 #ifndef _DRAW_H_
 #define _DRAW_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,11 +38,9 @@ void Orthogonalize(int x, int y);
 void ClearOpengl(void);
 void SetOpenglContext(void);
 
-void set_r(int i, double val);
-void set_t(int i, double val);
-void set_s(int i, double val);
-
 void unproject(double x, double y, double p[3], double d[3]);
+void Viewport2World(double win[3], double xyz[3]);
+void World2Viewport(double xyz[3], double win[3]);
 
 unsigned int PaletteContinuous(Post_View * View, double min, double max, double val);
 unsigned int PaletteContinuousLinear(Post_View * v, double min, double max, double val);
@@ -56,29 +54,38 @@ void ZeroHighlightEntityNum(int v, int c, int s);
 
 void Draw3d(void);
 void Draw2d(void);
+void DrawPlugin(void (*draw)(void));
 void Draw(void);
 
 void Draw_String(char *s);
+void Draw_String(char *s, double style);
+void Draw_String_Center(char *s);
+void Draw_String_Right(char *s);
 void Draw_Geom(Mesh *m);
 void Draw_Post(void);
 void Draw_Graph2D(void);
 void Draw_Text2D(void);
 void Draw_Text2D3D(int dim, int timestep, int nb, List_T *td, List_T *tc);
+int Fix2DCoordinates(double *x, double *y);
 void Draw_OnScreenMessages(void);
 void Draw_Scales(void);
-void Draw_Axes(double s);
-void Draw_SmallAxes(void);
 void Draw_Disk(double size, double rint, double x, double y, double z, int light);
 void Draw_Sphere(double size, double x, double y, double z, int light);
 void Draw_Cylinder(double width, double *x, double *y, double *z, int light);
 void Draw_Point(int type, double size, double *x, double *y, double *z, 
-		double Raise[3][8], int light);
+		int light);
 void Draw_Line(int type, double width, double *x, double *y, double *z,
-	       double Raise[3][8], int light);
+	       int light);
 void Draw_Vector(int Type, int Fill,
 		 double relHeadRadius, double relStemLength, double relStemRadius,
 		 double x, double y, double z, double dx, double dy, double dz,
-		 double Raise[3][8], int light);
+		 int light);
+void Draw_PlaneInBoundingBox(double xmin, double ymin, double zmin,
+			     double xmax, double ymax, double zmax,
+			     double a, double b, double c, double d);
+void Draw_SmallAxes(void);
+void Draw_Axes(int mode, int tics[3], char format[3][256], char label[3][256],
+	       double bbox[6]);
 
 void Draw_Mesh(Mesh *M);
 void Draw_Mesh_Volume(void *a, void *b);
@@ -95,10 +102,6 @@ void Draw_Mesh_Prism(void *a, void *b);
 void Draw_Mesh_Pyramid(void *a, void *b);
 void Draw_Mesh_Array(VertexArray *va, int faces, int edges);
 
-void Draw_PlaneInBoundingBox(double xmin, double ymin, double zmin,
-			     double xmax, double ymax, double zmax,
-			     double a, double b, double c, double d);
-
 #define ARGS Post_View *View, int preproNormals, \
              double ValMin, double ValMax, 	 \
              double *X, double *Y, double *Z, double *V
@@ -141,4 +144,7 @@ int GiveIndexFromValue_Lin(double ValMin, double ValMax, int NbIso, double Val);
 int GiveIndexFromValue_Log(double ValMin, double ValMax, int NbIso, double Val);
 int GiveIndexFromValue_DoubleLog(double ValMin, double ValMax, int NbIso, double Val);
 
+int GetValuesFromExternalView(Post_View *v, int type, int refcomp, 
+			      int *nbcomp, double **vals, int viewIndex);
+
 #endif
