@@ -1,6 +1,6 @@
-// $Id: Geom.cpp,v 1.73 2004/10/28 03:44:37 geuzaine Exp $
+// $Id: Geom.cpp,v 1.81 2005/02/20 06:36:54 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include "Interpolation.h"
 #include "Plugin.h"
 #include "PluginManager.h"
-#include "STL.h"
 #include "gl2ps.h"
 
 extern Context_T CTX;
@@ -150,12 +149,20 @@ void Draw_Curve(void *a, void *b)
       N = 10 * n;
       break;
     }
-    if(c->Typ == MSH_SEGM_DISCRETE) {
+    if(c->theSegmRep) {
+      glBegin(GL_LINE_STRIP);
+      for(int i = 0; i < List_Nbr(c->theSegmRep->points); i+=3) {
+        double *p = (double*)List_Pointer(c->theSegmRep->points, i);
+        glVertex3d(p[0], p[1], p[2]);
+      }
+      glEnd();
+    }
+    else if(c->Typ == MSH_SEGM_DISCRETE) {
       Simplex *s;
       List_T *temp = Tree2List(c->Simplexes);
       for(int i = 0; i < List_Nbr(temp); i++) {
         List_Read(temp, i, &s);
-        glBegin(GL_LINE_STRIP);
+        glBegin(GL_LINES);
         glVertex3d(s->V[0]->Pos.X, s->V[0]->Pos.Y, s->V[0]->Pos.Z);
         glVertex3d(s->V[1]->Pos.X, s->V[1]->Pos.Y, s->V[1]->Pos.Z);
         glEnd();
@@ -221,7 +228,7 @@ void Draw_Curve(void *a, void *b)
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
 		v.Pos.X, v.Pos.Y, v.Pos.Z,
-                dv.Pos.X, dv.Pos.Y, dv.Pos.Z, NULL, CTX.geom.light);
+                dv.Pos.X, dv.Pos.Y, dv.Pos.Z, CTX.geom.light);
   }
 
   if(CTX.render_mode == GMSH_SELECT) {
@@ -296,17 +303,19 @@ void getPlaneSurfaceNormal(Surface *s, double x, double y, double z, double n[3]
 
   if(List_Nbr(s->Generatrices) > 1){
     List_Read(s->Generatrices, 0, &c);
-    t1[0] = x - c->beg->Pos.X;
-    t1[1] = y - c->beg->Pos.Y;
-    t1[2] = z - c->beg->Pos.Z;
-    for(int i = 1; i < List_Nbr(s->Generatrices); i++){
-      List_Read(s->Generatrices, i, &c);
-      t2[0] = x - c->beg->Pos.X;
-      t2[1] = y - c->beg->Pos.Y;
-      t2[2] = z - c->beg->Pos.Z;
-      prodve(t1, t2, n);
-      if(norme(n))
-	break;
+    if(c->beg){
+      t1[0] = x - c->beg->Pos.X;
+      t1[1] = y - c->beg->Pos.Y;
+      t1[2] = z - c->beg->Pos.Z;
+      for(int i = 1; i < List_Nbr(s->Generatrices); i++){
+	List_Read(s->Generatrices, i, &c);
+	t2[0] = x - c->beg->Pos.X;
+	t2[1] = y - c->beg->Pos.Y;
+	t2[2] = z - c->beg->Pos.Z;
+	prodve(t1, t2, n);
+	if(norme(n))
+	  break;
+      }
     }
   }
   if(List_Nbr(s->Generatrices) < 2 || !norme(n)){
@@ -318,31 +327,42 @@ void getPlaneSurfaceNormal(Surface *s, double x, double y, double z, double n[3]
   }
 }
 
-void Draw_Triangulated_Surface(Surface * s)
+void Draw_Polygonal_Surface(Surface * s)
 {
-  int k = 0;
-  double *points, *p1, *p2, *p3;
-
   if(CTX.geom.surfaces) {
     if(CTX.geom.light) glEnable(GL_LIGHTING);
-    glEnable(GL_POLYGON_OFFSET_FILL);
-    glBegin(GL_TRIANGLES);
+    glEnable(GL_POLYGON_OFFSET_FILL); // always!
+    int k = 0;
     while (k < List_Nbr(s->thePolyRep->polygons)){
-      points = (double*)List_Pointer(s->thePolyRep->polygons,k);
+      double *points = (double*)List_Pointer(s->thePolyRep->polygons,k);
       k+= ((int)points[0] + 1);
       if (points[0] == 3){
-	p1 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 6*(int)points[1]);
-	p2 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 6*(int)points[2]);
-	p3 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 6*(int)points[3]);
+	glBegin(GL_TRIANGLES);
+	double *p1 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 
+					   6*(int)points[1]);
+	double *p2 = (double*)List_Pointer(s->thePolyRep->points_and_normals,
+					   6*(int)points[2]);
+	double *p3 = (double*)List_Pointer(s->thePolyRep->points_and_normals,
+					   6*(int)points[3]);
 	if(CTX.geom.light) glNormal3dv(&p1[3]);
 	glVertex3d(p1[0],p1[1],p1[2]);
 	if(CTX.geom.light) glNormal3dv(&p2[3]);
 	glVertex3d(p2[0],p2[1],p2[2]);
 	if(CTX.geom.light) glNormal3dv(&p3[3]);
 	glVertex3d(p3[0],p3[1],p3[2]);
+	glEnd();
+      }
+      else{
+	glBegin(GL_POLYGON);
+	for(int i = 0; i < (int)points[0]; i++){
+	  double *p = (double*)List_Pointer(s->thePolyRep->points_and_normals,
+					    6*(int)points[i+1]);
+	  if(CTX.geom.light) glNormal3dv(&p[3]);
+	  glVertex3d(p[0],p[1],p[2]);
+	}
+	glEnd();
       }
     }
-    glEnd();
     glDisable(GL_POLYGON_OFFSET_FILL);
     glDisable(GL_LIGHTING);
   }  
@@ -350,43 +370,40 @@ void Draw_Triangulated_Surface(Surface * s)
 
 void Draw_Plane_Surface(Surface * s)
 {
-  int i, j, k;
   Curve *c;
-  double minx = 0., miny = 0., maxx = 0., maxy = 0., t, n[3];
-  Vertex P1, P2, P3, V[4], vv, vv1, vv2;
-  char Num[100];
-  const int numPoints = 100;
-
-  if (s->thePolyRep) {
-    Draw_Triangulated_Surface(s);
-    return;
-  }
+  Vertex V[4], vv, vv1, vv2;
+  double n[3];
 
   if(!CTX.threads_lock && List_Nbr(s->Orientations) < 1) {
     CTX.threads_lock = 1;
 
     List_T *points = List_Create(10, 10, sizeof(Vertex *));
-    for(i = 0; i < List_Nbr(s->Generatrices); i++) {
+    for(int i = 0; i < List_Nbr(s->Generatrices); i++) {
       List_Read(s->Generatrices, i, &c);
-      for(j = 0; j < List_Nbr(c->Control_Points); j++) {
+      for(int j = 0; j < List_Nbr(c->Control_Points); j++) {
         List_Add(points, List_Pointer(c->Control_Points, j));
       }
     }
+
+    if(!List_Nbr(points)){
+      List_Delete(points);      
+      CTX.threads_lock = 0;
+      return;
+    }
     MeanPlane(points, s);
     List_Delete(points);
-
-    k = 0;
-
-    for(i = 0; i < List_Nbr(s->Generatrices); i++) {
+    
+    // compute (rough) bounding box of surface
+    double minx = 0., miny = 0., maxx = 0., maxy = 0.;
+    for(int i = 0; i < List_Nbr(s->Generatrices); i++) {
       List_Read(s->Generatrices, i, &c);
-      P1 = InterpolateCurve(c, 0.0, 0);
-      P2 = InterpolateCurve(c, 0.5, 0);
-      P3 = InterpolateCurve(c, 1.0, 0);
+      Vertex P1 = InterpolateCurve(c, 0.0, 0);
+      Vertex P2 = InterpolateCurve(c, 0.5, 0);
+      Vertex P3 = InterpolateCurve(c, 1.0, 0);
       Projette(&P1, s->plan);
       Projette(&P2, s->plan);
       Projette(&P3, s->plan);
-      if(!k) {
-        k = 1;
+      if(!i) {
         minx = maxx = P1.Pos.X;
         miny = maxy = P1.Pos.Y;
       }
@@ -405,7 +422,7 @@ void Draw_Plane_Surface(Surface * s)
     V[3].Pos.X = minx;
     V[3].Pos.Y = maxy;
 
-    for(i = 0; i < 4; i++) {
+    for(int i = 0; i < 4; i++) {
       V[i].Pos.Z = 0.0;
       putZ(&V[i], s);
     }
@@ -415,73 +432,64 @@ void Draw_Plane_Surface(Surface * s)
     n[2] = s->plan[2][2];
     norme(n);
 
-    k = 0;
-    for(i = 0; i < numPoints; i++) {
-      t = (double)i / (double)(numPoints-1);
-      vv.Pos.X = t * 0.5 * (V[0].Pos.X + V[1].Pos.X) + (1. - t) *
-        0.5 * (V[2].Pos.X + V[3].Pos.X);
-      vv.Pos.Y = t * 0.5 * (V[0].Pos.Y + V[1].Pos.Y) + (1. - t) *
-        0.5 * (V[2].Pos.Y + V[3].Pos.Y);
-      vv.Pos.Z = t * 0.5 * (V[0].Pos.Z + V[1].Pos.Z) + (1. - t) *
-        0.5 * (V[2].Pos.Z + V[3].Pos.Z);
-
-      if(isPointOnPlanarSurface(s, vv.Pos.X, vv.Pos.Y, vv.Pos.Z, n)) {
-        if(!k) {
-          List_Add(s->Orientations, &vv);
-          k = 1;
-        }
-      }
-      else {
-        if(k) {
-          List_Add(s->Orientations, &vv);
-          k = 0;
-        }
+    // compute which parts of the "middle cross" lie inside the surface
+    const int numPoints = 200;
+    for(int cross = 0; cross < 2; cross++) {
+      int k = 0;
+      for(int i = 0; i < numPoints; i++) {
+	double t = (double)i / (double)(numPoints-1);
+	if(!cross){
+	  vv.Pos.X = t * 0.5 * (V[0].Pos.X + V[1].Pos.X) + 
+	    (1. - t) * 0.5 * (V[2].Pos.X + V[3].Pos.X);
+	  vv.Pos.Y = t * 0.5 * (V[0].Pos.Y + V[1].Pos.Y) + 
+	    (1. - t) * 0.5 * (V[2].Pos.Y + V[3].Pos.Y);
+	  vv.Pos.Z = t * 0.5 * (V[0].Pos.Z + V[1].Pos.Z) + 
+	    (1. - t) * 0.5 * (V[2].Pos.Z + V[3].Pos.Z);
+	}
+	else{
+	  vv.Pos.X = t * .5 * (V[0].Pos.X + V[3].Pos.X) + 
+	    (1. - t) * .5 * (V[2].Pos.X + V[1].Pos.X);
+	  vv.Pos.Y = t * .5 * (V[0].Pos.Y + V[3].Pos.Y) + 
+	    (1. - t) * .5 * (V[2].Pos.Y + V[1].Pos.Y);
+	  vv.Pos.Z = t * .5 * (V[0].Pos.Z + V[3].Pos.Z) + 
+	    (1. - t) * .5 * (V[2].Pos.Z + V[1].Pos.Z);
+	}
+	if(isPointOnPlanarSurface(s, vv.Pos.X, vv.Pos.Y, vv.Pos.Z, n)) {
+	  if(!k) {
+	    List_Add(s->Orientations, &vv);
+	    k = 1;
+	  }
+	}
+	else {
+	  if(k) {
+	    List_Add(s->Orientations, &vv);
+	    k = 0;
+	  }
+	}
       }
+      if(k)
+	List_Add(s->Orientations, &vv);
     }
-    if(k)
-      List_Add(s->Orientations, &vv);
 
-    k = 0;
-    for(i = 0; i < numPoints; i++) {
-      t = (double)i / (double)(numPoints-1);
-      vv.Pos.X = t * .5 * (V[0].Pos.X + V[3].Pos.X) + 
-	(1. - t) * .5 * (V[2].Pos.X + V[1].Pos.X);
-      vv.Pos.Y = t * .5 * (V[0].Pos.Y + V[3].Pos.Y) + 
-	(1. - t) * .5 * (V[2].Pos.Y + V[1].Pos.Y);
-      vv.Pos.Z = t * .5 * (V[0].Pos.Z + V[3].Pos.Z) + 
-	(1. - t) * .5 * (V[2].Pos.Z + V[1].Pos.Z);
-      if(isPointOnPlanarSurface(s, vv.Pos.X, vv.Pos.Y, vv.Pos.Z, n)) {
-        if(!k) {
-          List_Add(s->Orientations, &vv);
-          k = 1;
-        }
-      }
-      else {
-        if(k) {
-          List_Add(s->Orientations, &vv);
-          k = 0;
-        }
-      }
+    Msg(STATUS2, "Plane Surface %d (%d cross points)", 
+	s->Num, List_Nbr(s->Orientations));
+    
+    if(!List_Nbr(s->Orientations)){ // add dummy
+      List_Add(s->Orientations, &vv);
+      //printf("surf %d: min=%g %g max=%g %g\n", s->Num, minx, miny, maxx, maxy);
     }
-    if(k)
-      List_Add(s->Orientations, &vv);
-
-    Msg(STATUS2, "Plane Surface %d (%d points)", s->Num, List_Nbr(s->Orientations));
-
-    if(!List_Nbr(s->Orientations)) // add dummy
-      List_Add(s->Orientations, &vv);
 
     CTX.threads_lock = 0;
   }
-
+  
   if(List_Nbr(s->Orientations) > 1) {
 
     if(CTX.geom.surfaces) {
       glEnable(GL_LINE_STIPPLE);
-      glLineStipple(1, 0x0F0F);
+      glLineStipple(1, 0x1F1F);
       gl2psEnable(GL2PS_LINE_STIPPLE);
       glBegin(GL_LINES);
-      for(i = 0; i < List_Nbr(s->Orientations); i++) {
+      for(int i = 0; i < List_Nbr(s->Orientations); i++) {
 	List_Read(s->Orientations, i, &vv);
 	glVertex3d(vv.Pos.X, vv.Pos.Y, vv.Pos.Z);
       }
@@ -493,6 +501,7 @@ void Draw_Plane_Surface(Surface * s)
     if(CTX.geom.surfaces_num) {
       List_Read(s->Orientations, 0, &vv1);
       List_Read(s->Orientations, 1, &vv2);
+      char Num[100];
       sprintf(Num, "%d", s->Num);
       double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
       glRasterPos3d((vv2.Pos.X + vv1.Pos.X) / 2. + offset / CTX.s[0],
@@ -514,7 +523,7 @@ void Draw_Plane_Surface(Surface * s)
       glColor4ubv((GLubyte *) & CTX.color.geom.normals);
       Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		  CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius, 
-		  x, y, z, n[0], n[1], n[2], NULL, CTX.geom.light);
+		  x, y, z, n[0], n[1], n[2], CTX.geom.light);
     }
 
   }
@@ -525,7 +534,7 @@ void Draw_NonPlane_Surface(Surface * s)
   if(CTX.geom.surfaces) {
     if(s->Typ == MSH_SURF_NURBS) {
       if(CTX.geom.light) glEnable(GL_LIGHTING);
-      glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       GLUnurbsObj *nurb;
       nurb = gluNewNurbsRenderer();
 #if defined(GLU_VERSION_1_3)
@@ -543,7 +552,7 @@ void Draw_NonPlane_Surface(Surface * s)
     }
     else{
       glEnable(GL_LINE_STIPPLE);
-      glLineStipple(1, 0x0F0F);
+      glLineStipple(1, 0x1F1F);
       gl2psEnable(GL2PS_LINE_STIPPLE);
       int N = 50;
       glBegin(GL_LINE_STRIP);
@@ -580,22 +589,17 @@ void Draw_NonPlane_Surface(Surface * s)
     Vertex v1 = InterpolateSurface(s, 0.5, 0.5, 0, 0);
     Vertex v2 = InterpolateSurface(s, 0.6, 0.5, 0, 0);
     Vertex v3 = InterpolateSurface(s, 0.5, 0.6, 0, 0);
-    double t1[3], t2[3], n[3];
-    t1[0] = v2.Pos.X - v1.Pos.X;
-    t1[1] = v2.Pos.Y - v1.Pos.Y;
-    t1[2] = v2.Pos.Z - v1.Pos.Z;
-    t2[0] = v3.Pos.X - v1.Pos.X;
-    t2[1] = v3.Pos.Y - v1.Pos.Y;
-    t2[2] = v3.Pos.Z - v1.Pos.Z;
-    prodve(t1, t2, n);
-    norme(n);
+    double n[3];
+    normal3points(v1.Pos.X, v1.Pos.Y, v1.Pos.Z, 
+		  v2.Pos.X, v2.Pos.Y, v2.Pos.Z, 
+		  v3.Pos.X, v3.Pos.Y, v3.Pos.Z, n);
     n[0] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[0];
     n[1] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[1];
     n[2] *= CTX.geom.normals * CTX.pixel_equiv_x / CTX.s[2];
     glColor4ubv((GLubyte *) & CTX.color.geom.normals);
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		v1.Pos.X, v1.Pos.Y, v1.Pos.Z, n[0], n[1], n[2], NULL,
+		v1.Pos.X, v1.Pos.Y, v1.Pos.Z, n[0], n[1], n[2],
 		CTX.geom.light);
   }
 }
@@ -613,23 +617,21 @@ void Draw_Surface(void *a, void *b)
   }
 
   if(s->ipar[4]) {
-    glLineWidth(CTX.geom.line_sel_width);
-    gl2psLineWidth(CTX.geom.line_sel_width *
+    glLineWidth(CTX.geom.line_sel_width / 2.);
+    gl2psLineWidth(CTX.geom.line_sel_width / 2. *
 		   CTX.print.eps_line_width_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.surface_sel);
   }
   else {
-    glLineWidth(CTX.geom.line_width);
-    gl2psLineWidth(CTX.geom.line_width * CTX.print.eps_line_width_factor);
+    glLineWidth(CTX.geom.line_width / 2.);
+    gl2psLineWidth(CTX.geom.line_width / 2. * CTX.print.eps_line_width_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.surface);
   }
 
-  if(s->Typ == MSH_SURF_STL) {
-    Tree_Action(s->STL->Simplexes, Draw_Mesh_Triangle);
-  }
-  else if(s->Typ == MSH_SURF_DISCRETE) {
+  if(s->thePolyRep) // always draw the poly reprentation if available
+    Draw_Polygonal_Surface(s);
+  else if(s->Typ == MSH_SURF_DISCRETE)
     Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
-  }
   else if(s->Typ == MSH_SURF_PLAN)
     Draw_Plane_Surface(s);
   else
@@ -692,7 +694,10 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
       Draw_Curve(&c,NULL);
     }
     else{
-      Msg(STATUS1N, "Curve %d  {%d->%d}", c->Num, c->beg->Num, c->end->Num);
+      if(c->beg && c->end)
+	Msg(STATUS1N, "Curve %d {%d->%d}", c->Num, c->beg->Num, c->end->Num);
+      else
+	Msg(STATUS1N, "Curve %d", c->Num);
     }
   }
   else if(s) {
@@ -701,23 +706,28 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
       Draw_Surface(&s,NULL);
     }
     else{
-      sprintf(Message, "Surface %d {", s->Num);
       int nbg = List_Nbr(s->Generatrices);
-      if(nbg < 10) {
-	for(int i = 0; i < nbg; i++) {
-	  List_Read(s->Generatrices, i, &cc);
-	  if(!i)
-	    sprintf(temp, "%d", cc->Num);
-	  else
-	    sprintf(temp, ",%d", cc->Num);
-	  strcat(Message, temp);
+      if(nbg){
+	sprintf(Message, "Surface %d {", s->Num);
+	if(nbg < 10) {
+	  for(int i = 0; i < nbg; i++) {
+	    List_Read(s->Generatrices, i, &cc);
+	    if(!i)
+	      sprintf(temp, "%d", cc->Num);
+	    else
+	      sprintf(temp, ",%d", cc->Num);
+	    strcat(Message, temp);
+	  }
 	}
+	else {
+	  strcat(Message, "...");
+	}
+	strcat(Message, "}");
+	Msg(STATUS1N, Message);
       }
-      else {
-	strcat(Message, "...");
+      else{
+	Msg(STATUS1N, "Surface %d", s->Num);
       }
-      strcat(Message, "}");
-      Msg(STATUS1N, Message);
     }
   }
   else{
