@@ -1,6 +1,6 @@
-// $Id: Graph2D.cpp,v 1.37 2004/10/11 19:18:59 geuzaine Exp $
+// $Id: Graph2D.cpp,v 1.51 2005/03/14 19:03:55 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -92,20 +92,19 @@ static void addval(Post_View * v, double Abs, double Val,
 }
 
 
-static void Draw_Graph2D(Post_View * v,
-                         double xx, double yy, double width, double height,
-                         double tic, double bb[4])
+static void Draw_Graph2D(Post_View * v, double xtop, double ytop, 
+			 double width, double height, double tic)
 {
   char label[1024];
   float font_h, font_a;
   int i, i_inc, i_max, j, j_inc, j_max, k, nb;
   double dx, dy, dv;
-  double xtop = xx;
-  double ytop = CTX.viewport[3] - yy;
   double ybot = ytop - height;
   double Abs, Val, ValMin = 0., ValMax = 0., AbsMin, AbsMax;
   double p1[3], p2[3];
 
+  glColor4ubv((GLubyte *) & v->color.axes);
+
   gl_font(CTX.gl_font_enum, CTX.gl_fontsize);
   font_h = gl_height(); // total font height
   font_a = gl_height() - gl_descent();  // height above ref pt
@@ -139,13 +138,12 @@ static void Draw_Graph2D(Post_View * v,
 
   // axes
 
-  if(v->Grid) {
-    glColor4ubv((GLubyte *) & CTX.color.fg);
+  if(v->Axes) {
     glBegin(GL_LINE_STRIP);
     glVertex2d(xtop, ytop);
     glVertex2d(xtop, ytop - height);
     glVertex2d(xtop + width, ybot);
-    if(v->Grid > 1) {
+    if(v->Axes > 1) {
       glVertex2d(xtop + width, ytop);
       glVertex2d(xtop, ytop);
     }
@@ -161,21 +159,19 @@ static void Draw_Graph2D(Post_View * v,
   dy = height / (double)nb;
   dv = (ValMax - ValMin) / (double)nb;
   for(i = 0; i < nb + 1; i++) {
-    if(v->Grid > 0) {
-      glColor4ubv((GLubyte *) & CTX.color.fg);
+    if(v->Axes > 0) {
       glBegin(GL_LINES);
       glVertex2d(xtop, ytop - i * dy);
       glVertex2d(xtop + tic, ytop - i * dy);
-      if(v->Grid > 1) {
+      if(v->Axes > 1) {
         glVertex2d(xtop + width - tic, ytop - i * dy);
         glVertex2d(xtop + width, ytop - i * dy);
       }
       glEnd();
-      if(v->Grid > 2 && i != 0 && i != nb) {
+      if(v->Axes > 2 && i != 0 && i != nb) {
         glEnable(GL_LINE_STIPPLE);
         glLineStipple(1, 0x1111);
         gl2psEnable(GL2PS_LINE_STIPPLE);
-        glLineWidth(1.);
         gl2psLineWidth(1. * CTX.print.eps_line_width_factor);
         glBegin(GL_LINES);
         glVertex2d(xtop, ytop - i * dy);
@@ -183,29 +179,28 @@ static void Draw_Graph2D(Post_View * v,
         glEnd();
         glDisable(GL_LINE_STIPPLE);
         gl2psDisable(GL2PS_LINE_STIPPLE);
-        glLineWidth(CTX.line_width);
         gl2psLineWidth(CTX.line_width * CTX.print.eps_line_width_factor);
       }
     }
     if(v->ShowScale) {
-      glColor4ubv((GLubyte *) & CTX.color.text);
       sprintf(label, v->Format, (i == nb) ? ValMin : (ValMax - i * dv));
-      glRasterPos2d(xtop - gl_width(label) - 2 * tic,
-		    ytop - i * dy - font_a / 3.);
-      Draw_String(label);
+      glRasterPos2d(xtop - 2 * tic, ytop - i * dy - font_a / 3.);
+      Draw_String_Right(label);
     }
   }
 
   if(v->ShowScale) {
-    glColor4ubv((GLubyte *) & CTX.color.text);
     if(v->Type == DRAW_POST_2D_SPACE &&
        ((v->ShowTime == 1 && List_Nbr(v->Time) > 1) ||
-	(v->ShowTime == 2 && List_Nbr(v->Time) > 0)))
-      sprintf(label, "%s (%g)", v->Name, *(double *)List_Pointer(v->Time, v->TimeStep));
+	(v->ShowTime == 2 && List_Nbr(v->Time) > 0))){
+      char tmp[256];
+      sprintf(tmp, v->Format, *(double *)List_Pointer(v->Time, v->TimeStep));
+      sprintf(label, "%s (%s)", v->Name, tmp);
+    }
     else
       sprintf(label, "%s", v->Name);
-    glRasterPos2d(xtop - gl_width(label) / 2., ytop + font_h + tic);
-    Draw_String(label);
+    glRasterPos2d(xtop, ytop + font_h + tic);
+    Draw_String_Center(label);
   }
 
   // x tics + labels
@@ -230,37 +225,30 @@ static void Draw_Graph2D(Post_View * v,
     AbsMax = *(double *)List_Pointer(v->Time, List_Nbr(v->Time) - 1);
   }
 
-  nb = v->NbAbscissa;
-  if(v->ShowScale) {
-    sprintf(label, v->AbscissaFormat, AbsMin);
-    double ww = gl_width(label);
-    sprintf(label, v->AbscissaFormat, AbsMax);
-    if(gl_width(label) > ww)
-      ww = gl_width(label);
-    if((nb - 1) * (ww + 2) > width)
-      nb = (int)floor(width / (ww + 2)) + 1;
+  nb = v->AxesTics[0];
+  if(v->Axes) {
+    sprintf(label, v->AxesFormat[0], -M_PI/1.e4);
+    if((nb-1) * gl_width(label) > width)
+      nb = (int)(width / gl_width(label)) + 1;
   }
-  if(nb == 1)
-    dx = width;
-  else
-    dx = width / (double)(nb - 1);
+  if(nb == 1) nb++;
 
+  dx = width / (double)(nb - 1);
+  
   for(i = 0; i < nb; i++) {
-    if(v->Grid > 0) {
-      glColor4ubv((GLubyte *) & CTX.color.fg);
+    if(v->Axes) {
       glBegin(GL_LINES);
       glVertex2d(xtop + i * dx, ybot);
       glVertex2d(xtop + i * dx, ybot + tic);
-      if(v->Grid > 1) {
+      if(v->Axes > 1) {
         glVertex2d(xtop + i * dx, ytop);
         glVertex2d(xtop + i * dx, ytop - tic);
       }
       glEnd();
-      if(v->Grid > 2 && i != 0 && i != nb - 1) {
+      if(v->Axes > 2 && i != 0 && i != nb - 1) {
         glEnable(GL_LINE_STIPPLE);
         glLineStipple(1, 0x1111);
         gl2psEnable(GL2PS_LINE_STIPPLE);
-        glLineWidth(1.);
         gl2psLineWidth(1. * CTX.print.eps_line_width_factor);
         glBegin(GL_LINES);
         glVertex2d(xtop + i * dx, ytop);
@@ -268,27 +256,23 @@ static void Draw_Graph2D(Post_View * v,
         glEnd();
         glDisable(GL_LINE_STIPPLE);
         gl2psDisable(GL2PS_LINE_STIPPLE);
-        glLineWidth(CTX.line_width);
         gl2psLineWidth(CTX.line_width * CTX.print.eps_line_width_factor);
       }
-    }
-    if(v->ShowScale) {
-      glColor4ubv((GLubyte *) & CTX.color.text);
+
       if(nb == 1)
-        sprintf(label, v->AbscissaFormat, AbsMin);
+        sprintf(label, v->AxesFormat[0], AbsMin);
       else
-        sprintf(label, v->AbscissaFormat,
+        sprintf(label, v->AxesFormat[0],
                 AbsMin + i * (AbsMax - AbsMin) / (double)(nb - 1));
-      glRasterPos2d(xtop + i * dx - gl_width(label) / 2.,
-                    ybot - font_h - tic);
-      Draw_String(label);
+      glRasterPos2d(xtop + i * dx, ybot - font_h - tic);
+      Draw_String_Center(label);
     }
   }
-  if(v->ShowScale) {
-    glColor4ubv((GLubyte *) & CTX.color.text);
-    sprintf(label, "%s", v->AbscissaName);
-    glRasterPos2d(xtop + width / 2 - gl_width(label) / 2., ybot - 2 * font_h - 2 * tic);
-    Draw_String(label);
+
+  if(v->Axes) {
+    sprintf(label, "%s", v->AxesLabel[0]);
+    glRasterPos2d(xtop + width / 2, ybot - 2 * font_h - 2 * tic);
+    Draw_String_Center(label);
   }
 
   // the curve(s)
@@ -349,76 +333,110 @@ static void Draw_Graph2D(Post_View * v,
 
 }
 
-void getbb(double pos[2], double width, double height,
-           double dx, double dy, double tic, double space,
-           double bbtot[4], double bb[4])
-{
-  bbtot[0] = pos[0] - dx - space;       //topleft x
-  bb[0] = bbtot[0];
-
-  //don't recompute bbtot[1].
-  bb[1] = pos[1] - 1.5 * dy - space;    //topleft y
-
-  bbtot[2] = MAX(bbtot[2], pos[0] + width + (dx - tic) / 2 + space);    //bottomright x
-  bb[2] = pos[0] + width + (dx - tic) / 2 + space;
-
-  bbtot[3] = pos[1] + height + 2 * dy + space;  //bottomright y
-  bb[3] = bbtot[3];
-}
-
 void Draw_Graph2D(void)
 {
-  int i, nbauto = 0;
-  Post_View *v;
-  double dx, dy, bb[4], bbtot[4] = { 0., 0., 0., 0. }, pos[2] = {0., 0.}, tic;
-  double space = 10.;
-  char label[1024];
+  static List_T *todraw = NULL;
 
   if(!CTX.post.list)
     return;
 
-  for(i = 0; i < List_Nbr(CTX.post.list); i++) {
-    v = (Post_View *) List_Pointer(CTX.post.list, i);
-    if(v->Visible && !v->Dirty && v->NbSP && v->Type != DRAW_POST_3D) {
-      tic = 5;
-      dx = dy = 0.;
-      if(v->ShowScale) {
-        gl_font(CTX.gl_font_enum, CTX.gl_fontsize);
-        sprintf(label, v->AbscissaFormat, v->CustomMin);
-        if(gl_width(label) + tic > dx)
-          dx = gl_width(label) + tic;
-        sprintf(label, v->AbscissaFormat, v->CustomMax);
-        if(gl_width(label) + tic > dx)
-          dx = gl_width(label) + tic;
-        dy = 1.5 * gl_height(); //2 below and & above!
+  // 2d graphs to draw?
+
+  if(!todraw)
+    todraw = List_Create(5, 5, sizeof(Post_View *));
+  else
+    List_Reset(todraw);
+
+  gl_font(CTX.gl_font_enum, CTX.gl_fontsize);
+  char label[1024];
+  double largest_number = 0.;
+
+  for(int i = 0; i < List_Nbr(CTX.post.list); i++) {
+    Post_View *v = *(Post_View **) List_Pointer(CTX.post.list, i);
+    if(v->Visible && !v->Dirty && v->NbSP && v->Type != DRAW_POST_3D){
+      List_Add(todraw, &v);
+      sprintf(label, v->Format, -M_PI/1.e4);
+      if(largest_number < gl_width(label))
+	largest_number = gl_width(label);
+    }
+  }
+  
+  if(!List_Nbr(todraw))
+    return;
+
+  const double tic = 5;
+  double xsep = largest_number;
+  double ysep = 5 * gl_height();
+  
+  int num = 0;
+
+  for(int i = 0; i < List_Nbr(todraw); i++) {
+    Post_View *v = *(Post_View **) List_Pointer(todraw, i);
+
+    if(!v->AutoPosition) {
+      double x = v->Position[0], y = v->Position[1];
+      int center = Fix2DCoordinates(&x, &y);
+      Draw_Graph2D(v, 
+		   x - (center & 1 ? v->Size[0]/2. : 0), 
+		   y + (center & 2 ? v->Size[1]/2. : 0), 
+		   v->Size[0], v->Size[1], tic);
+    }
+    else{
+      double winw = CTX.viewport[2] - CTX.viewport[0];
+      double winh = CTX.viewport[3] - CTX.viewport[1];
+      if(List_Nbr(todraw) == 1){
+	double fracw = 0.75, frach = 0.75;
+	double w = fracw * winw - xsep;
+	double h = frach * winh - ysep;
+	double xmin = CTX.viewport[0] + (1-fracw)/2. * winw;
+	double ymin = CTX.viewport[1] + (1-frach)/2. * winh;
+	Draw_Graph2D(v, xmin + 0.95*xsep, CTX.viewport[3] - (ymin + 0.4*ysep), w, h, tic);
       }
-      if(!v->AutoPosition || !nbauto) {
-        pos[0] = v->Position[0];
-        pos[1] = v->Position[1];
-        bbtot[1] = pos[1] - 1.5 * dy - space;   //top y
-        getbb(pos, v->Size[0], v->Size[1], dx, dy, tic, space, bbtot, bb);
-        Draw_Graph2D(v, pos[0], pos[1], v->Size[0], v->Size[1], tic, bb);
+      else if(List_Nbr(todraw) == 2){
+	double fracw = 0.75, frach = 0.85;
+	double w = fracw * winw - xsep;
+	double h = frach * winh / 2. - ysep;
+	double xmin = CTX.viewport[0] + (1-fracw)/2. * winw;
+	double ymin = CTX.viewport[1] + (1-frach)/3. * winh;
+	if(num == 1) ymin += (h + ysep + (1-frach)/3. * winh);
+	Draw_Graph2D(v, xmin + 0.95*xsep, CTX.viewport[3] - (ymin + 0.4*ysep), w, h, tic);
+	num++;
       }
-      else {
-        if(bbtot[3] + v->Size[1] + 3 * dy + 2 * space < CTX.viewport[3]) {      //try to put below
-          pos[1] = bbtot[3] + 1.5 * dy + space;
-          getbb(pos, v->Size[0], v->Size[1], dx, dy, tic, space, bbtot, bb);
-          Draw_Graph2D(v, pos[0], pos[1], v->Size[0], v->Size[1], tic, bb);
-        }
-        else {  //start a new column
-          pos[0] = bbtot[2] + dx + space;
-          pos[1] = bbtot[1] + 1.5 * dy + space;
-          getbb(pos, v->Size[0], v->Size[1], dx, dy, tic, space, bbtot, bb);
-          Draw_Graph2D(v, pos[0], pos[1], v->Size[0], v->Size[1], tic, bb);
-        }
+      else{
+	double fracw = 0.85, frach = 0.85;
+	double w = fracw * winw / 2. - xsep;
+	double h = frach * winh / 2. - ysep;
+	double xmin = CTX.viewport[0] + (1-fracw)/3. * winw;
+	if(num == 1 || num == 3) xmin += (w + xsep + (1-fracw)/3. * winw);
+	double ymin = CTX.viewport[1] + (1-frach)/3. * winh;
+	if(num == 2 || num == 3) ymin += (h + ysep + (1-frach)/3. * winh);
+	Draw_Graph2D(v, xmin + 0.95*xsep, CTX.viewport[3] - (ymin + 0.4*ysep), w, h, tic);
+	num++;
       }
-      nbauto++;
     }
   }
 }
 
 // Text strings
 
+int Fix2DCoordinates(double *x, double *y)
+{
+  int ret = (*x > 99999 && *y > 99999) ? 3 : (*y > 99999) ? 2 : (*x > 99999) ? 1 : 0;
+
+  if(*x < 0) // measure from right border
+    *x = CTX.viewport[2] + *x;
+  else if(*x > 99999) // by convention, x-centered
+    *x = CTX.viewport[2]/2;
+
+  if(*y < 0) // measure from bottom border
+    *y = -(*y);
+  else if(*y > 99999) // by convention, y-centered
+    *y = CTX.viewport[3]/2.;
+  else
+    *y = CTX.viewport[3] - *y;
+  return ret;
+}
+
 // Parser format: T2(x,y,style){"str","str",...};
 // T2D list of double : x,y,style,index,x,y,style,index,...
 // T2C list of chars  : string\0,string\0,string\0,string\0,...
@@ -458,12 +476,9 @@ void Draw_Text2D3D(int dim, int timestep, int nb, List_T * td, List_T * tc)
     d2 = (double *)List_Pointer_Test(td, (j + 1) * nbd);
     if(dim == 2) {
       x = d1[0];
-      if(x < 0)
-        x = CTX.viewport[2] + x;
-      y = CTX.viewport[3] - d1[1];
-      if(d1[1] < 0)
-        y = -d1[1];
+      y = d1[1];
       z = 0.;
+      Fix2DCoordinates(&x, &y);
       style = d1[2];
       index = (int)d1[3];
       if(d2)
@@ -490,27 +505,23 @@ void Draw_Text2D3D(int dim, int timestep, int nb, List_T * td, List_T * tc)
         l++;
     }
     if(k < nbchar && l == timestep)
-      Draw_String(&c[k]);
+      Draw_String(&c[k], style);
     else
-      Draw_String(c);
+      Draw_String(c, style);
   }
 }
 
 
 void Draw_Text2D(void)
 {
-  int i;
-  Post_View *v;
-
   if(!CTX.post.list)
     return;
 
-  glColor4ubv((GLubyte *) & CTX.color.text);
-
-  for(i = 0; i < List_Nbr(CTX.post.list); i++) {
-    v = (Post_View *) List_Pointer(CTX.post.list, i);
-    if(v->Visible && !v->Dirty && v->DrawStrings)
+  for(int i = 0; i < List_Nbr(CTX.post.list); i++) {
+    Post_View *v = *(Post_View **) List_Pointer(CTX.post.list, i);
+    if(v->Visible && !v->Dirty && v->DrawStrings){
+      glColor4ubv((GLubyte *) & v->color.text2d);
       Draw_Text2D3D(2, v->TimeStep, v->NbT2, v->T2D, v->T2C);
+    }
   }
-
 }
