@@ -1,6 +1,6 @@
-// $Id: Iso.cpp,v 1.30 2004/08/07 06:59:16 geuzaine Exp $
+// $Id: Iso.cpp,v 1.33 2005/01/01 19:35:29 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -54,14 +54,8 @@ void IsoTriangle(Post_View * View, double *X, double *Y, double *Z,
     nb++;
   }
   
-  if(nb == 2) {
-    double Raise[3][8];
-    for(int i = 0; i < 3; i++)
-      for(int l = 0; l < 2; l++)
-	Raise[i][l] = View->Raise[i] * V;
-    Draw_Line(View->LineType, View->LineWidth, Xp, Yp, Zp,
-	      Raise, View->Light);
-  }
+  if(nb == 2)
+    Draw_Line(View->LineType, View->LineWidth, Xp, Yp, Zp, View->Light);
 }
 
 // Compute the polygon between the two iso-lines V1 and V2 in a
@@ -238,10 +232,7 @@ void IsoLine(Post_View *View, double *X, double *Y, double *Z,
 
   if((Val[0] >= V && Val[1] <= V) || (Val[1] >= V && Val[0] <= V)) {
     InterpolateIso(X, Y, Z, Val, V, 0, 1, Xp, Yp, Zp);
-    double Raise[3][8];
-    for(int i = 0; i < 3; i++)
-      Raise[i][0] = View->Raise[i] * V;
-    Draw_Point(View->PointType, View->PointSize, Xp, Yp, Zp, Raise, View->Light);
+    Draw_Point(View->PointType, View->PointSize, Xp, Yp, Zp, View->Light);
   }
 }
 
@@ -490,17 +481,11 @@ void IsoSimplex(Post_View * View, int preproNormals,
 
   if(View->TriVertexArray && View->TriVertexArray->fill){
     for(int i = 2; i < nb; i++){
-      View->TriVertexArray->add(Xp[0] + View->Raise[0] * V,
-				Yp[0] + View->Raise[1] * V,
-				Zp[0] + View->Raise[2] * V,
+      View->TriVertexArray->add(Xp[0], Yp[0], Zp[0],
 				norms[0], norms[1], norms[2], color);
-      View->TriVertexArray->add(Xp[i-1] + View->Raise[0] * V,
-				Yp[i-1] + View->Raise[1] * V,
-				Zp[i-1] + View->Raise[2] * V,
+      View->TriVertexArray->add(Xp[i-1], Yp[i-1], Zp[i-1],
 				norms[3*(i-1)], norms[3*(i-1)+1], norms[3*(i-1)+2], color);
-      View->TriVertexArray->add(Xp[i] + View->Raise[0] * V,
-				Yp[i] + View->Raise[1] * V,
-				Zp[i] + View->Raise[2] * V,
+      View->TriVertexArray->add(Xp[i], Yp[i], Zp[i],
 				norms[3*i], norms[3*i+1], norms[3*i+2], color);
       View->TriVertexArray->num++;
     }
@@ -511,17 +496,11 @@ void IsoSimplex(Post_View * View, int preproNormals,
     glBegin(GL_TRIANGLES);
     for(int i = 2; i < nb; i++){
       if(View->Light) glNormal3dv(&norms[0]);
-      glVertex3d(Xp[0] + View->Raise[0] * V, 
-		 Yp[0] + View->Raise[1] * V, 
-		 Zp[0] + View->Raise[2] * V);
+      glVertex3d(Xp[0], Yp[0], Zp[0]);
       if(View->Light) glNormal3dv(&norms[3*(i-1)]);
-      glVertex3d(Xp[i-1] + View->Raise[0] * V, 
-		 Yp[i-1] + View->Raise[1] * V, 
-		 Zp[i-1] + View->Raise[2] * V);
+      glVertex3d(Xp[i-1], Yp[i-1], Zp[i-1]);
       if(View->Light) glNormal3dv(&norms[3*i]);
-      glVertex3d(Xp[i] + View->Raise[0] * V, 
-		 Yp[i] + View->Raise[1] * V, 
-		 Zp[i] + View->Raise[2] * V);
+      glVertex3d(Xp[i], Yp[i], Zp[i]);
     }
     glEnd();
     glDisable(GL_LIGHTING);
