@@ -1,6 +1,6 @@
-// $Id: Mesh.cpp,v 1.112 2004/10/30 03:07:29 geuzaine Exp $
+// $Id: Mesh.cpp,v 1.125 2005/03/15 15:43:31 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -140,13 +140,19 @@ void Draw_Mesh(Mesh * M)
   if(CTX.render_mode == GMSH_SELECT)
     return;
 
-  // draw the bounding box of the mesh if we are in fast redraw mode
-  // and there is no geometry
- 
-  if((CTX.draw_bbox && (Tree_Nbr(M->Vertices) || Tree_Nbr(M->Points))) ||
-     (!CTX.mesh.draw && Tree_Nbr(M->Vertices) && !Tree_Nbr(M->Points))) {
+  // draw the bounding box if we asked for it and we have a geometry
+  // or a mesh, or if we are in fast redraw mode and there is no
+  // geometry but there is a mesh
+  
+  int somegeo = 
+    Tree_Nbr(M->Points) || Tree_Nbr(M->Curves) || Tree_Nbr(M->Surfaces);
+
+  if((CTX.draw_bbox && (somegeo || Tree_Nbr(M->Vertices))) ||
+     (!CTX.mesh.draw && !Tree_Nbr(M->Points) && Tree_Nbr(M->Vertices))) {
     glColor4ubv((GLubyte *) & CTX.color.fg);
     glLineWidth(CTX.line_width);
+    gl2psLineWidth(CTX.line_width * CTX.print.eps_line_width_factor);
+
     glBegin(GL_LINE_LOOP);
     glVertex3d(CTX.min[0], CTX.min[1], CTX.min[2]);
     glVertex3d(CTX.max[0], CTX.min[1], CTX.min[2]);
@@ -182,6 +188,7 @@ void Draw_Mesh(Mesh * M)
     sprintf(label, "(%g,%g,%g)", CTX.max[0], CTX.max[1], CTX.max[2]);
     Draw_String(label);
 
+    glColor3d(1.,0.,0.);
     for(int i = 0; i < 6; i++)
       if(CTX.clip[i] & 1 || CTX.clip[i] & 2)
 	Draw_PlaneInBoundingBox(CTX.min[0], CTX.min[1], CTX.min[2],
@@ -194,6 +201,24 @@ void Draw_Mesh(Mesh * M)
 			      CTX.mesh.cut_planea, CTX.mesh.cut_planeb, 
 			      CTX.mesh.cut_planec, CTX.mesh.cut_planed);
   }
+
+  // draw the axes
+
+  if(CTX.axes){
+    glColor4ubv((GLubyte *) & CTX.color.axes);
+    glLineWidth(CTX.line_width);
+    gl2psLineWidth(CTX.line_width * CTX.print.eps_line_width_factor);
+    if(!CTX.axes_auto_position){
+      Draw_Axes(CTX.axes, CTX.axes_tics, CTX.axes_format, CTX.axes_label, 
+		CTX.axes_position);
+    }
+    else if(somegeo || Tree_Nbr(M->Vertices)){
+      double bb[6] = { CTX.min[0], CTX.max[0],
+		       CTX.min[1], CTX.max[1],
+		       CTX.min[2], CTX.max[2] };
+      Draw_Axes(CTX.axes, CTX.axes_tics, CTX.axes_format, CTX.axes_label, bb);
+    }
+  }
   
   // draw the mesh
 
@@ -244,11 +269,6 @@ void Draw_Mesh(Mesh * M)
       glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   }
 
-  // draw the big moving axes
-
-  if(CTX.axes)
-    Draw_Axes(CTX.lc_middle / 4.);
-
   // draw any plugin-specific stuff
   
   if(CTX.post.plugin_draw_function)
@@ -285,6 +305,7 @@ void Draw_Mesh_Volume(void *a, void *b)
       v->QuadVertexArray = new VertexArray(4, 1000);
       v->QuadVertexArray->fill = 1;
       Tree_Action(v->Simplexes, Draw_Mesh_Tetrahedron);
+      Tree_Action(v->SimplexesBase, Draw_Mesh_Tetrahedron);
       Tree_Action(v->Hexahedra, Draw_Mesh_Hexahedron);
       Tree_Action(v->Prisms, Draw_Mesh_Prism);
       Tree_Action(v->Pyramids, Draw_Mesh_Pyramid);
@@ -312,6 +333,7 @@ void Draw_Mesh_Volume(void *a, void *b)
      CTX.mesh.normals){
     Msg(DEBUG, "classic volume data path");
     Tree_Action(v->Simplexes, Draw_Mesh_Tetrahedron);
+    Tree_Action(v->SimplexesBase, Draw_Mesh_Tetrahedron);
     Tree_Action(v->Hexahedra, Draw_Mesh_Hexahedron);
     Tree_Action(v->Prisms, Draw_Mesh_Prism);
     Tree_Action(v->Pyramids, Draw_Mesh_Pyramid);
@@ -336,6 +358,7 @@ void Draw_Mesh_Surface(void *a, void *b)
     s->normals = new smooth_normals;
     preproNormals = 1;
     Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
+    Tree_Action(s->SimplexesBase, Draw_Mesh_Triangle);
     Tree_Action(s->Quadrangles, Draw_Mesh_Quadrangle);
     preproNormals = 0;
   }
@@ -345,9 +368,11 @@ void Draw_Mesh_Surface(void *a, void *b)
       Msg(DEBUG, "regenerate surface mesh vertex arrays");
       // triangles
       if(s->TriVertexArray) delete s->TriVertexArray;
-      s->TriVertexArray = new VertexArray(3, Tree_Nbr(s->Simplexes));
+      s->TriVertexArray = new VertexArray(3, Tree_Nbr(s->Simplexes) + 
+					  Tree_Nbr(s->SimplexesBase));
       s->TriVertexArray->fill = 1;
       Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
+      Tree_Action(s->SimplexesBase, Draw_Mesh_Triangle);
       if(s->TriVertexArray){
 	Msg(DEBUG, "%d triangles in surface vertex array", s->TriVertexArray->num);
 	s->TriVertexArray->fill = 0;
@@ -374,6 +399,7 @@ void Draw_Mesh_Surface(void *a, void *b)
      CTX.mesh.points_per_element || CTX.mesh.normals){
     Msg(DEBUG, "classic triangle data path");
     Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
+    Tree_Action(s->SimplexesBase, Draw_Mesh_Triangle);
   }
 
   if(!s->QuadVertexArray || CTX.mesh.dual || CTX.mesh.surfaces_num ||
@@ -405,6 +431,7 @@ void Draw_Mesh_Curve(void *a, void *b)
   thePhysical = getFirstPhysical(MSH_PHYSICAL_LINE, c->Num);
 
   Tree_Action(c->Simplexes, Draw_Mesh_Line);
+  Tree_Action(c->SimplexesBase, Draw_Mesh_Line);
 }
 
 void Draw_Mesh_Point(int num, double x, double y, double z, int degree, int visible)
@@ -454,7 +481,7 @@ void Draw_Mesh_Line(void *a, void *b)
   double Xc = 0.0, Yc = 0.0, Zc = 0.0, m[3];
   char Num[100];
 
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
 
   if(!(s->Visible & VIS_MESH))
     return;
@@ -463,6 +490,12 @@ void Draw_Mesh_Line(void *a, void *b)
   if(part && !(*part)->Visible)
     return;
 
+  if(CTX.mesh.radius_sup) {
+    double r = s->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
+      return;
+  }
+
   if(!CTX.mesh.cut_plane_only_volume && intersectCutPlane(2, s->V) < 0)
     return;
 
@@ -533,27 +566,10 @@ void Draw_Mesh_Line(void *a, void *b)
     m[2] *= CTX.mesh.tangents * CTX.pixel_equiv_x / CTX.s[2];
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		Xc, Yc, Zc, m[0], m[1], m[2], NULL, CTX.mesh.light);
+		Xc, Yc, Zc, m[0], m[1], m[2], CTX.mesh.light);
   }
 }
 
-void _normal3points(double x0, double y0, double z0,
-		    double x1, double y1, double z1,
-		    double x2, double y2, double z2,
-		    double n[3])
-{
-  double x1x0 = x1 - x0;
-  double y1y0 = y1 - y0;
-  double z1z0 = z1 - z0;
-  double x2x0 = x2 - x0;
-  double y2y0 = y2 - y0;
-  double z2z0 = z2 - z0;
-  n[0] = y1y0 * z2z0 - z1z0 * y2y0;
-  n[1] = z1z0 * x2x0 - x1x0 * z2z0;
-  n[2] = x1x0 * y2y0 - y1y0 * x2x0;
-  norme(n);
-}
-
 void _triFace(double x0, double y0, double z0,
 	      double x1, double y1, double z1,
 	      double x2, double y2, double z2)
@@ -561,7 +577,7 @@ void _triFace(double x0, double y0, double z0,
   double n[3], ns[3];
 
   if(CTX.mesh.light || (theSurface && preproNormals)){
-    _normal3points(x0, y0, z0, x1, y1, z1, x2, y2, z2, n);
+    normal3points(x0, y0, z0, x1, y1, z1, x2, y2, z2, n);
     if(theSurface && preproNormals){
       theSurface->normals->add(x0, y0, z0, n[0], n[1], n[2]);
       theSurface->normals->add(x1, y1, z1, n[0], n[1], n[2]);
@@ -613,9 +629,9 @@ void _quadFace(double *x, double *y, double *z,
   double n[3], ns[3];
 
   if(CTX.mesh.light || (theSurface && preproNormals)){
-    _normal3points(x[i0], y[i0], z[i0],
-		   x[i1], y[i1], z[i1],
-		   x[i2], y[i2], z[i2], n);
+    normal3points(x[i0], y[i0], z[i0],
+		  x[i1], y[i1], z[i1],
+		  x[i2], y[i2], z[i2], n);
     if(theSurface && preproNormals){
       theSurface->normals->add(x[i0], y[i0], z[i0], n[0], n[1], n[2]);
       theSurface->normals->add(x[i1], y[i1], z[i1], n[0], n[1], n[2]);
@@ -688,12 +704,11 @@ void Draw_Mesh_Array(VertexArray *va, int faces, int edges)
   glEnableClientState(GL_NORMAL_ARRAY);
 
   if(faces){
-    if(edges)
-      glEnable(GL_POLYGON_OFFSET_FILL);
     if(CTX.mesh.light)
       glEnable(GL_LIGHTING);
     else
       glDisableClientState(GL_NORMAL_ARRAY);
+    if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
     glDrawArrays((va->type == 3) ? GL_TRIANGLES : GL_QUADS, 0, va->type * va->num);
     glDisable(GL_POLYGON_OFFSET_FILL);
     glDisable(GL_LIGHTING);
@@ -721,7 +736,7 @@ void Draw_Mesh_Triangle(void *a, void *b)
   double n[3];
   char Num[256];
 
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
 
   if(!s->V[2]) // the old extrusion algo puts lines in the simp_surf tree...
     return;
@@ -733,6 +748,12 @@ void Draw_Mesh_Triangle(void *a, void *b)
   if(part && !(*part)->Visible)
     return;
 
+  if(CTX.mesh.radius_sup) {
+    double r = s->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
+      return;
+  }
+
   if(!CTX.mesh.cut_plane_only_volume && intersectCutPlane(3, s->V) < 0)
     return;
 
@@ -779,9 +800,9 @@ void Draw_Mesh_Triangle(void *a, void *b)
 
   if(theSurface && theSurface->TriVertexArray){
     if(preproNormals || theSurface->TriVertexArray->fill)
-      _normal3points(X[0], Y[0], Z[0], 
-		     X[1], Y[1], Z[1],
-		     X[2], Y[2], Z[2], n);
+     normal3points(X[0], Y[0], Z[0], 
+		   X[1], Y[1], Z[1],
+		   X[2], Y[2], Z[2], n);
     if(preproNormals){
       for(int i = 0; i < 3; i++)
 	theSurface->normals->add(X[i], Y[i], Z[i], n[0], n[1], n[2]);
@@ -815,7 +836,7 @@ void Draw_Mesh_Triangle(void *a, void *b)
     if(CTX.mesh.surfaces_faces) {
       glColor4ubv((GLubyte *) & col);      
       if(CTX.mesh.light) glEnable(GL_LIGHTING);
-      if(CTX.mesh.surfaces_edges) glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       if(!s->VSUP) {
 	glBegin(GL_TRIANGLES);
 	_triFace(X[0], Y[0], Z[0], X[1], Y[1], Z[1], X[2], Y[2], Z[2]);
@@ -858,16 +879,16 @@ void Draw_Mesh_Triangle(void *a, void *b)
   }
 
   if(CTX.mesh.normals) {
-    _normal3points(X[0], Y[0], Z[0], 
-		   X[1], Y[1], Z[1],
-		   X[2], Y[2], Z[2], n);
+    normal3points(X[0], Y[0], Z[0], 
+		  X[1], Y[1], Z[1],
+		  X[2], Y[2], Z[2], n);
     glColor4ubv((GLubyte *) & CTX.color.mesh.normals);
     n[0] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[0];
     n[1] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[1];
     n[2] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[2];
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		Xc, Yc, Zc, n[0], n[1], n[2], NULL, CTX.mesh.light);
+		Xc, Yc, Zc, n[0], n[1], n[2], CTX.mesh.light);
   }
 }
 
@@ -886,6 +907,12 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
   if(part && !(*part)->Visible)
     return;
 
+  if(CTX.mesh.radius_sup) {
+    double r = q->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
+      return;
+  }
+
   if(!CTX.mesh.cut_plane_only_volume && intersectCutPlane(4, q->V) < 0)
     return;
 
@@ -935,9 +962,9 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
 
   if(theSurface && theSurface->QuadVertexArray){
     if(preproNormals || theSurface->QuadVertexArray->fill)
-      _normal3points(X[0], Y[0], Z[0], 
-		     X[1], Y[1], Z[1],
-		     X[2], Y[2], Z[2], n);
+      normal3points(X[0], Y[0], Z[0], 
+		    X[1], Y[1], Z[1],
+		    X[2], Y[2], Z[2], n);
     if(preproNormals){
       for(int i = 0; i < 4; i++)
 	theSurface->normals->add(X[i], Y[i], Z[i], n[0], n[1], n[2]);
@@ -971,7 +998,7 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
     if(CTX.mesh.surfaces_faces) {
       glColor4ubv((GLubyte *) & col);
       if(CTX.mesh.light) glEnable(GL_LIGHTING);
-      if(CTX.mesh.surfaces_edges) glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       if(!q->VSUP) {
 	glBegin(GL_QUADS);
 	_quadFace(X, Y, Z, 0, 1, 2, 3);
@@ -1015,15 +1042,15 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
 
   if(CTX.mesh.normals) {
     glColor4ubv((GLubyte *) & CTX.color.mesh.normals);
-    _normal3points(X[0], Y[0], Z[0], 
-		   X[1], Y[1], Z[1],
-		   X[2], Y[2], Z[2], n);
+    normal3points(X[0], Y[0], Z[0], 
+		  X[1], Y[1], Z[1],
+		  X[2], Y[2], Z[2], n);
     n[0] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[0];
     n[1] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[1];
     n[2] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[2];
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		Xc, Yc, Zc, n[0], n[1], n[2], NULL, CTX.mesh.light);
+		Xc, Yc, Zc, n[0], n[1], n[2], CTX.mesh.light);
   }
 }
 
@@ -1032,7 +1059,7 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
   char Num[100];
   double tmp, X[4], Y[4], Z[4], X2[6], Y2[6], Z2[6];
 
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
 
   if(!s->V[3] || !(s->Visible & VIS_MESH))
     return;
@@ -1048,10 +1075,11 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
   }
 
   if(CTX.mesh.radius_sup) {
-    if(s->Radius < CTX.mesh.radius_inf || s->Radius > CTX.mesh.radius_sup)
+    double r = s->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
       return;
   }
-
+  
   int edges = CTX.mesh.volumes_edges;
   int faces = CTX.mesh.volumes_faces;
 
@@ -1111,7 +1139,7 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
 	int b = trifaces_tetra[i][1];
 	int c = trifaces_tetra[i][2];
 	double n[3];
-	_normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
+	normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
 	theVolume->TriVertexArray->add(X[a], Y[a], Z[a], n[0], n[1], n[2], col);
 	theVolume->TriVertexArray->add(X[b], Y[b], Z[b], n[0], n[1], n[2], col);
 	theVolume->TriVertexArray->add(X[c], Y[c], Z[c], n[0], n[1], n[2], col);
@@ -1142,7 +1170,7 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
     if(faces){
       glColor4ubv((GLubyte *) & col);
       if(CTX.mesh.light) glEnable(GL_LIGHTING);
-      if(CTX.mesh.surfaces_edges || edges) glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       if(!s->VSUP){
 	glBegin(GL_TRIANGLES);
 	_triFace(X[0], Y[0], Z[0], X[2], Y[2], Z[2], X[1], Y[1], Z[1]);
@@ -1213,6 +1241,12 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
   if(part && !(*part)->Visible)
     return;
 
+  if(CTX.mesh.radius_sup) {
+    double r = h->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
+      return;
+  }
+
   int edges = CTX.mesh.volumes_edges;
   int faces = CTX.mesh.volumes_faces;
 
@@ -1275,7 +1309,7 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
 	int c = quadfaces_hexa[i][2];
 	int d = quadfaces_hexa[i][3];
 	double n[3];
-	_normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
+	normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
 	theVolume->QuadVertexArray->add(X[a], Y[a], Z[a], n[0], n[1], n[2], col);
 	theVolume->QuadVertexArray->add(X[b], Y[b], Z[b], n[0], n[1], n[2], col);
 	theVolume->QuadVertexArray->add(X[c], Y[c], Z[c], n[0], n[1], n[2], col);
@@ -1307,7 +1341,7 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
     if(faces){
       glColor4ubv((GLubyte *) & col);
       if(CTX.mesh.light) glEnable(GL_LIGHTING);
-      if(CTX.mesh.surfaces_edges || edges) glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       if(!h->VSUP){
 	glBegin(GL_QUADS);
 	_quadFace(X, Y, Z, 0, 3, 2, 1);
@@ -1388,6 +1422,12 @@ void Draw_Mesh_Prism(void *a, void *b)
   if(part && !(*part)->Visible)
     return;
 
+  if(CTX.mesh.radius_sup) {
+    double r = p->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
+      return;
+  }
+
   int edges = CTX.mesh.volumes_edges;
   int faces = CTX.mesh.volumes_faces;
 
@@ -1454,7 +1494,7 @@ void Draw_Mesh_Prism(void *a, void *b)
 	int b = trifaces_prism[i][1];
 	int c = trifaces_prism[i][2];
 	double n[3];
-	_normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
+	normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
 	theVolume->TriVertexArray->add(X[a], Y[a], Z[a], n[0], n[1], n[2], col);
 	theVolume->TriVertexArray->add(X[b], Y[b], Z[b], n[0], n[1], n[2], col);
 	theVolume->TriVertexArray->add(X[c], Y[c], Z[c], n[0], n[1], n[2], col);
@@ -1468,7 +1508,7 @@ void Draw_Mesh_Prism(void *a, void *b)
 	int c = quadfaces_prism[i][2];
 	int d = quadfaces_prism[i][3];
 	double n[3];
-	_normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
+	normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
 	theVolume->QuadVertexArray->add(X[a], Y[a], Z[a], n[0], n[1], n[2], col);
 	theVolume->QuadVertexArray->add(X[b], Y[b], Z[b], n[0], n[1], n[2], col);
 	theVolume->QuadVertexArray->add(X[c], Y[c], Z[c], n[0], n[1], n[2], col);
@@ -1500,7 +1540,7 @@ void Draw_Mesh_Prism(void *a, void *b)
     if(faces){
       glColor4ubv((GLubyte *) & col);
       if(CTX.mesh.light) glEnable(GL_LIGHTING);
-      if(CTX.mesh.surfaces_edges || edges) glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       if(!p->VSUP){
 	glBegin(GL_TRIANGLES);
 	_triFace(X[0], Y[0], Z[0], X[2], Y[2], Z[2], X[1], Y[1], Z[1]);
@@ -1578,6 +1618,12 @@ void Draw_Mesh_Pyramid(void *a, void *b)
   if(part && !(*part)->Visible)
     return;
 
+  if(CTX.mesh.radius_sup) {
+    double r = p->maxEdge();
+    if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
+      return;
+  }
+
   int edges = CTX.mesh.volumes_edges;
   int faces = CTX.mesh.volumes_faces;
 
@@ -1644,7 +1690,7 @@ void Draw_Mesh_Pyramid(void *a, void *b)
 	int b = trifaces_pyramid[i][1];
 	int c = trifaces_pyramid[i][2];
 	double n[3];
-	_normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
+	normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
 	theVolume->TriVertexArray->add(X[a], Y[a], Z[a], n[0], n[1], n[2], col);
 	theVolume->TriVertexArray->add(X[b], Y[b], Z[b], n[0], n[1], n[2], col);
 	theVolume->TriVertexArray->add(X[c], Y[c], Z[c], n[0], n[1], n[2], col);
@@ -1657,7 +1703,7 @@ void Draw_Mesh_Pyramid(void *a, void *b)
       int c = quadfaces_pyramid[0][2];
       int d = quadfaces_pyramid[0][3];
       double n[3];
-      _normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
+      normal3points(X[a], Y[a], Z[a], X[b], Y[b], Z[b], X[c], Y[c], Z[c], n);
       theVolume->QuadVertexArray->add(X[a], Y[a], Z[a], n[0], n[1], n[2], col);
       theVolume->QuadVertexArray->add(X[b], Y[b], Z[b], n[0], n[1], n[2], col);
       theVolume->QuadVertexArray->add(X[c], Y[c], Z[c], n[0], n[1], n[2], col);
@@ -1688,7 +1734,7 @@ void Draw_Mesh_Pyramid(void *a, void *b)
     if(faces){
       glColor4ubv((GLubyte *) & col);
       if(CTX.mesh.light) glEnable(GL_LIGHTING);
-      if(CTX.mesh.surfaces_edges || edges) glEnable(GL_POLYGON_OFFSET_FILL);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
       if(!p->VSUP){
 	glBegin(GL_QUADS);
 	_quadFace(X, Y, Z, 0, 3, 2, 1);
