@@ -1,6 +1,6 @@
-// $Id: Post.cpp,v 1.83 2004/11/01 15:10:36 geuzaine Exp $
+// $Id: Post.cpp,v 1.99 2005/03/14 18:55:22 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -126,24 +126,115 @@ unsigned int PaletteDiscrete(Post_View * v, int nbi, int i)
   return v->CT.table[index];
 }
 
-// Compute node coordinates taking Offset and Explode into account
+// Get values from a compatible external view (i.e., a view that has
+// the same number of elements as the {type,refcomp} list in the
+// reference view, and the same number of time steps)
 
-void Get_Coords(double Explode, double *Offset, int nbnod,
-                double *x1, double *y1, double *z1,
-                double *x2, double *y2, double *z2)
+int GetValuesFromExternalView(Post_View *v, int type, int refcomp, 
+			      int *nbcomp, double **vals, int viewIndex)
 {
-  int i;
-  double xc = 0., yc = 0., zc = 0.;
+  Post_View **vv = (Post_View **) List_Pointer_Test(CTX.post.list, viewIndex);  
 
-  if(Explode == 1) {
-    for(i = 0; i < nbnod; i++) {
-      x2[i] = x1[i] + Offset[0];
-      y2[i] = y1[i] + Offset[1];
-      z2[i] = z1[i] + Offset[2];
+  if(!vv){
+    if(!v->ExternalElementIndex)
+      Msg(GERROR, "Nonexistent external view: drawing self instead");
+    return 0;
+  }
+
+  Post_View *v2 = *vv;
+  int nbelm = 0, comp = 0, nbnod = 0, ref;
+  List_T *l = NULL;
+
+  switch(type){
+  case POST_POINT:
+    nbnod = 1;
+    ref = (refcomp == 1) ? v->NbSP : ((refcomp == 3) ? v->NbVP : v->NbTP);
+    if(ref == v2->NbSP){ nbelm = v2->NbSP; l = v2->SP; comp = 1; }
+    else if(ref == v2->NbVP){ nbelm = v2->NbVP; l = v2->VP; comp = 3; }
+    else if(ref == v2->NbTP){ nbelm = v2->NbTP; l = v2->TP; comp = 9; }
+    break;
+  case POST_LINE:
+    nbnod = 2;
+    ref = (refcomp == 1) ? v->NbSL : ((refcomp == 3) ? v->NbVL : v->NbTL);
+    if(ref == v2->NbSL){ nbelm = v2->NbSL; l = v2->SL; comp = 1; } 
+    else if(ref == v2->NbVL){ nbelm = v2->NbVL; l = v2->VL; comp = 3; } 
+    else if(ref == v2->NbTL){ nbelm = v2->NbTL; l = v2->TL; comp = 9; } 
+    break;
+  case POST_TRIANGLE:
+    nbnod = 3;
+    ref = (refcomp == 1) ? v->NbST : ((refcomp == 3) ? v->NbVT : v->NbTT);
+    if(ref == v2->NbST){ nbelm = v2->NbST; l = v2->ST; comp = 1;  } 
+    else if(ref == v2->NbVT){ nbelm = v2->NbVT; l = v2->VT; comp = 3;  } 
+    else if(ref == v2->NbTT){ nbelm = v2->NbTT; l = v2->TT; comp = 9;  } 
+    break;
+  case POST_QUADRANGLE:
+    nbnod = 4;
+    ref = (refcomp == 1) ? v->NbSQ : ((refcomp == 3) ? v->NbVQ : v->NbTQ);
+    if(ref == v2->NbSQ){ nbelm = v2->NbSQ; l = v2->SQ; comp = 1;  } 
+    else if(ref == v2->NbVQ){ nbelm = v2->NbVQ; l = v2->VQ; comp = 3;  } 
+    else if(ref == v2->NbTQ){ nbelm = v2->NbTQ; l = v2->TQ; comp = 9;  } 
+    break;
+  case POST_TETRAHEDRON:
+    nbnod = 4;
+    ref = (refcomp == 1) ? v->NbSS : ((refcomp == 3) ? v->NbVS : v->NbTS);
+    if(ref == v2->NbSS){ nbelm = v2->NbSS; l = v2->SS; comp = 1;  } 
+    else if(ref == v2->NbVS){ nbelm = v2->NbVS; l = v2->VS; comp = 3;  } 
+    else if(ref == v2->NbTS){ nbelm = v2->NbTS; l = v2->TS; comp = 9;  } 
+    break;
+  case POST_HEXAHEDRON:
+    nbnod = 8;
+    ref = (refcomp == 1) ? v->NbSH : ((refcomp == 3) ? v->NbVH : v->NbTH);
+    if(ref == v2->NbSH){ nbelm = v2->NbSH; l = v2->SH; comp = 1;  }
+    else if(ref == v2->NbVH){ nbelm = v2->NbVH; l = v2->VH; comp = 3;  }
+    else if(ref == v2->NbTH){ nbelm = v2->NbTH; l = v2->TH; comp = 9;  }
+    break;
+  case POST_PRISM:
+    nbnod = 6;
+    ref = (refcomp == 1) ? v->NbSI : ((refcomp == 3) ? v->NbVI : v->NbTI);
+    if(ref == v2->NbSI){ nbelm = v2->NbSI; l = v2->SI; comp = 1;  }
+    else if(ref == v2->NbVI){ nbelm = v2->NbVI; l = v2->VI; comp = 3;  }
+    else if(ref == v2->NbTI){ nbelm = v2->NbTI; l = v2->TI; comp = 9;  }
+    break;
+  case POST_PYRAMID:
+    nbnod = 5;
+    ref = (refcomp == 1) ? v->NbSY : ((refcomp == 3) ? v->NbVY : v->NbTY);
+    if(ref == v2->NbSY){ nbelm = v2->NbSY; l = v2->SY; comp = 1;  } 
+    else if(ref == v2->NbVY){ nbelm = v2->NbVY; l = v2->VY; comp = 3;  } 
+    else if(ref == v2->NbTY){ nbelm = v2->NbTY; l = v2->TY; comp = 9;  } 
+    break;
+  }
+
+  if(!l || !nbelm || v2->NbTimeStep != v->NbTimeStep ||
+     v->ExternalElementIndex < 0 || v->ExternalElementIndex >= nbelm){
+    if(!v->ExternalElementIndex)
+      Msg(GERROR, "Incompatible external view: drawing self instead");
+    return 0;
+  }
+
+  int nb = List_Nbr(l) / nbelm;
+  *nbcomp = comp;
+  *vals = (double *)List_Pointer(l, v->ExternalElementIndex * nb + 
+				 3 * nbnod + comp * nbnod * v->TimeStep);
+  return 1;
+}
+
+// Compute node coordinates taking Offset, Raise and Explode into account
+
+void Get_Coords(Post_View *v, int type, int nbnod, int nbcomp,
+                double *x1, double *y1, double *z1, double *vals,
+                double *x2, double *y2, double *z2,
+		int offset, int raise, int transform)
+{
+  if(v->Explode == 1.) {
+    for(int i = 0; i < nbnod; i++) {
+      x2[i] = x1[i];
+      y2[i] = y1[i];
+      z2[i] = z1[i];
     }
   }
   else {
-    for(i = 0; i < nbnod; i++) {
+    double xc = 0., yc = 0., zc = 0.;
+    for(int i = 0; i < nbnod; i++) {
       xc += x1[i];
       yc += y1[i];
       zc += z1[i];
@@ -151,12 +242,64 @@ void Get_Coords(double Explode, double *Offset, int nbnod,
     xc /= (double)nbnod;
     yc /= (double)nbnod;
     zc /= (double)nbnod;
-    for(i = 0; i < nbnod; i++) {
-      x2[i] = xc + Explode * (x1[i] - xc) + Offset[0];
-      y2[i] = yc + Explode * (y1[i] - yc) + Offset[1];
-      z2[i] = zc + Explode * (z1[i] - zc) + Offset[2];
+    for(int i = 0; i < nbnod; i++) {
+      x2[i] = xc + v->Explode * (x1[i] - xc);
+      y2[i] = yc + v->Explode * (y1[i] - yc);
+      z2[i] = zc + v->Explode * (z1[i] - zc);
     }
   }
+
+  if(transform){
+    for(int i = 0; i < nbnod; i++) {
+      double x = x2[i], y = y2[i], z = z2[i];
+      x2[i] = v->Transform[0][0] * x + v->Transform[0][1] * y + v->Transform[0][2] * z;
+      y2[i] = v->Transform[1][0] * x + v->Transform[1][1] * y + v->Transform[1][2] * z;
+      z2[i] = v->Transform[2][0] * x + v->Transform[2][1] * y + v->Transform[2][2] * z;
+    }
+  }
+  
+  if(offset){
+    for(int i = 0; i < nbnod; i++) {
+      x2[i] += v->Offset[0];
+      y2[i] += v->Offset[1];
+      z2[i] += v->Offset[2];
+    }
+  }
+
+  if(raise){
+    for(int i = 0; i < nbnod; i++){
+      double norm = 0.;
+      if(nbcomp == 1)
+	norm = vals[i];
+      else if(nbcomp == 3)
+	norm = sqrt(vals[3*i] * vals[3*i] + 
+		    vals[3*i+1] * vals[3*i+1] + 
+		    vals[3*i+2] * vals[3*i+2]);
+      else if(nbcomp == 9)
+	norm = ComputeVonMises(vals + 9*i);
+      x2[i] += v->Raise[0] * norm;
+      y2[i] += v->Raise[1] * norm;
+      z2[i] += v->Raise[2] * norm;
+    }
+  }
+
+  if(v->UseGenRaise){
+    int ext_nbcomp = nbcomp;
+    double *ext_vals = vals;
+    if(v->ViewIndexForGenRaise >= 0)
+      GetValuesFromExternalView(v, type, nbcomp, &ext_nbcomp, &ext_vals, 
+				v->ViewIndexForGenRaise);
+    ApplyGeneralizedRaise(v, nbnod, ext_nbcomp, ext_vals, x2, y2, z2);
+  }
+
+  for(int i = 0; i < nbnod; i++){
+    if(x2[i] < v->TmpBBox[0]) v->TmpBBox[0] = x2[i];
+    if(x2[i] > v->TmpBBox[1]) v->TmpBBox[1] = x2[i];
+    if(y2[i] < v->TmpBBox[2]) v->TmpBBox[2] = y2[i];
+    if(y2[i] > v->TmpBBox[3]) v->TmpBBox[3] = y2[i];
+    if(z2[i] < v->TmpBBox[4]) v->TmpBBox[4] = z2[i];
+    if(z2[i] > v->TmpBBox[5]) v->TmpBBox[5] = z2[i];
+  }
 }
 
 // Compare barycenters with viewpoint (eye)
@@ -181,8 +324,6 @@ int changedEye()
   return 0;
 }
 
-// to be rigorous, we should take Raise into account
-
 int compareEye(double *q, double *w, int nbnodes)
 {
   double d, dq, dw, cgq[3] = { 0., 0., 0. }, cgw[3] = { 0., 0., 0.};
@@ -231,14 +372,28 @@ int compareEye8Nodes(const void *a, const void *b)
 
 // Draw_Post
 
-void Draw_List(Post_View * v, double ValMin, double ValMax,
-	       List_T * list, int nbelm, int nbnod,
+void Draw_List(Post_View * v, double ValMin, double ValMax, int type,
+	       List_T * list, int nbelm, int nbnod, int nbcomp,
 	       void (*draw) (Post_View *, int, double, double, double *, 
 			     double *, double *, double *))
 {
-  int i, nb;
+  int i, nb, offset = 0, raise = 0, transform = 0;
   double X[8], Y[8], Z[8];
 
+  // do we need to apply an offset?
+  if(v->Offset[0] || v->Offset[1] || v->Offset[2])
+    offset = 1;
+
+  // do we need to apply a simple raise?
+  if(v->Raise[0] || v->Raise[1] || v->Raise[2])
+    raise = 1;
+
+  // do we need to apply a general transformation?
+  if(v->Transform[0][0] != 1. || v->Transform[0][1] != 0. || v->Transform[0][2] != 0. ||
+     v->Transform[1][0] != 0. || v->Transform[1][1] != 1. || v->Transform[1][2] != 0. ||
+     v->Transform[2][0] != 0. || v->Transform[2][1] != 0. || v->Transform[2][2] != 1.)
+    transform = 1;
+
   if(nbelm) {
     nb = List_Nbr(list) / nbelm;
 
@@ -246,10 +401,13 @@ void Draw_List(Post_View * v, double ValMin, double ValMax,
       Msg(DEBUG, "Preprocessing of normals in View[%d]", v->Index);
       v->ExternalElementIndex = 0;
       for(i = 0; i < List_Nbr(list); i += nb) {
-        Get_Coords(v->Explode, v->Offset, nbnod,
+        Get_Coords(v, type, nbnod, nbcomp,
                    (double *)List_Pointer_Fast(list, i),
                    (double *)List_Pointer_Fast(list, i + nbnod),
-                   (double *)List_Pointer_Fast(list, i + 2 * nbnod), X, Y, Z);
+                   (double *)List_Pointer_Fast(list, i + 2 * nbnod), 
+		   (double *)List_Pointer_Fast(list, i + 3 * nbnod +
+					       v->TimeStep * nbnod * nbcomp),
+		   X, Y, Z, offset, raise, transform);
         draw(v, 1, ValMin, ValMax, X, Y, Z,
              (double *)List_Pointer_Fast(list, i + 3 * nbnod));
 	v->ExternalElementIndex++;
@@ -258,10 +416,13 @@ void Draw_List(Post_View * v, double ValMin, double ValMax,
 
     v->ExternalElementIndex = 0;
     for(i = 0; i < List_Nbr(list); i += nb) {
-      Get_Coords(v->Explode, v->Offset, nbnod,
+      Get_Coords(v, type, nbnod, nbcomp,
                  (double *)List_Pointer_Fast(list, i),
                  (double *)List_Pointer_Fast(list, i + nbnod),
-                 (double *)List_Pointer_Fast(list, i + 2 * nbnod), X, Y, Z);
+                 (double *)List_Pointer_Fast(list, i + 2 * nbnod),
+		 (double *)List_Pointer_Fast(list, i + 3 * nbnod +
+					     v->TimeStep * nbnod * nbcomp),
+		 X, Y, Z, offset, raise, transform);
       draw(v, 0, ValMin, ValMax, X, Y, Z,
            (double *)List_Pointer_Fast(list, i + 3 * nbnod));
       v->ExternalElementIndex++;
@@ -280,7 +441,7 @@ void Draw_Post(void)
 
   if(CTX.draw_bbox || !CTX.post.draw) {  // draw only the bbox of the visible views
     for(int iView = 0; iView < List_Nbr(CTX.post.list); iView++) {
-      v = (Post_View *) List_Pointer(CTX.post.list, iView);
+      v = *(Post_View **) List_Pointer(CTX.post.list, iView);
       if(v->Visible && v->Type == DRAW_POST_3D) {
         glColor4ubv((GLubyte *) & CTX.color.fg);
 	glLineWidth(CTX.line_width);
@@ -319,6 +480,7 @@ void Draw_Post(void)
 	sprintf(label, "(%g,%g,%g)", v->BBox[1], v->BBox[3], v->BBox[5]);
 	Draw_String(label);
 
+	glColor3d(1.,0.,0.);
 	for(int i = 0; i < 6; i++)
 	  if(CTX.clip[i] & (1<<(2+iView)))
 	    Draw_PlaneInBoundingBox(v->BBox[0], v->BBox[2], v->BBox[4],
@@ -334,7 +496,7 @@ void Draw_Post(void)
 
   for(int iView = 0; iView < List_Nbr(CTX.post.list); iView++) {
 
-    v = (Post_View *) List_Pointer(CTX.post.list, iView);
+    v = *(Post_View **) List_Pointer(CTX.post.list, iView);
 
     if(v->Visible && !v->Dirty) {
 
@@ -396,9 +558,19 @@ void Draw_Post(void)
       if(v->Light && v->SmoothNormals && v->Changed)
 	v->reset_normals();
 
+      if(v->UseGenRaise)
+	InitGeneralizedRaise(v);
+
+      if(v->Changed){
+	for(int i = 0; i < 3; i++) {
+	  v->TmpBBox[2 * i] = VAL_INF;
+	  v->TmpBBox[2 * i + 1] = -VAL_INF;
+	}
+      }
+
       // initialize alpha blending for transparency
       if(CTX.alpha && ColorTable_IsAlpha(&v->CT)){
-	if(CTX.fake_transparency){
+	if(v->FakeTransparency){
 	  // simple additive blending "a la xpost":
 	  glBlendFunc(GL_SRC_ALPHA, GL_ONE); // glBlendEquation(GL_FUNC_ADD);
 	  // maximum intensity projection "a la volsuite":
@@ -446,19 +618,25 @@ void Draw_Post(void)
 
       if(v->DrawPoints) {
 	if(v->Type == DRAW_POST_3D && v->DrawScalars)
-	  Draw_List(v, ValMin, ValMax, v->SP, v->NbSP, 1, Draw_ScalarPoint);
+	  Draw_List(v, ValMin, ValMax, POST_POINT, v->SP, v->NbSP, 1, 1, 
+		    Draw_ScalarPoint);
 	if(v->DrawVectors)
-	  Draw_List(v, ValMin, ValMax, v->VP, v->NbVP, 1, Draw_VectorPoint);
+	  Draw_List(v, ValMin, ValMax, POST_POINT, v->VP, v->NbVP, 1, 3, 
+		    Draw_VectorPoint);
 	if(v->DrawTensors)
-	  Draw_List(v, ValMin, ValMax, v->TP, v->NbTP, 1, Draw_TensorPoint);
+	  Draw_List(v, ValMin, ValMax, POST_POINT, v->TP, v->NbTP, 1, 9, 
+		    Draw_TensorPoint);
       }
       if(v->DrawLines) {
 	if(v->DrawScalars)
-	  Draw_List(v, ValMin, ValMax, v->SL, v->NbSL, 2, Draw_ScalarLine);
+	  Draw_List(v, ValMin, ValMax, POST_LINE, v->SL, v->NbSL, 2, 1,
+		    Draw_ScalarLine);
 	if(v->DrawVectors)
-	  Draw_List(v, ValMin, ValMax, v->VL, v->NbVL, 2, Draw_VectorLine);
+	  Draw_List(v, ValMin, ValMax, POST_LINE, v->VL, v->NbVL, 2, 3, 
+		    Draw_VectorLine);
 	if(v->DrawTensors)
-	  Draw_List(v, ValMin, ValMax, v->TL, v->NbTL, 2, Draw_TensorLine);
+	  Draw_List(v, ValMin, ValMax, POST_LINE, v->TL, v->NbTL, 2, 9, 
+		    Draw_TensorLine);
       }
 
       for(int pass = 0; pass < 2; pass++){
@@ -478,7 +656,7 @@ void Draw_Post(void)
 	else{
 	  // don't even enter the classic data path if we don't have to
 	  if(v->TriVertexArray){
-	    if(v->Boundary < 1 && !v->ShowElement &&
+	    if(v->Boundary < 1 && !v->ShowElement && !v->Normals &&
 	       v->IntervalsType != DRAW_POST_NUMERIC && v->IntervalsType != DRAW_POST_ISO){
 	      Msg(DEBUG, "View[%d]: skiping 2D scalar pass alltogether", v->Index);
 	      skip_2d = 1;
@@ -494,51 +672,69 @@ void Draw_Post(void)
       pass_0:
 	if(v->DrawTriangles) {
 	  if(!skip_2d && v->DrawScalars)
-	    Draw_List(v, ValMin, ValMax, v->ST, v->NbST, 3, Draw_ScalarTriangle);
+	    Draw_List(v, ValMin, ValMax, POST_TRIANGLE, v->ST, v->NbST, 3, 1,
+		      Draw_ScalarTriangle);
 	  if(v->DrawVectors)
-	    Draw_List(v, ValMin, ValMax, v->VT, v->NbVT, 3, Draw_VectorTriangle);
+	    Draw_List(v, ValMin, ValMax, POST_TRIANGLE, v->VT, v->NbVT, 3, 3,
+		      Draw_VectorTriangle);
 	  if(v->DrawTensors)
-	    Draw_List(v, ValMin, ValMax, v->TT, v->NbTT, 3, Draw_TensorTriangle);
+	    Draw_List(v, ValMin, ValMax, POST_TRIANGLE, v->TT, v->NbTT, 3, 9,
+		      Draw_TensorTriangle);
 	}
 	if(v->DrawQuadrangles) {
 	  if(!skip_2d && v->DrawScalars)
-	    Draw_List(v, ValMin, ValMax, v->SQ, v->NbSQ, 4, Draw_ScalarQuadrangle);
+	    Draw_List(v, ValMin, ValMax, POST_QUADRANGLE, v->SQ, v->NbSQ, 4, 1,
+		      Draw_ScalarQuadrangle);
 	  if(v->DrawVectors)
-	    Draw_List(v, ValMin, ValMax, v->VQ, v->NbVQ, 4, Draw_VectorQuadrangle);
+	    Draw_List(v, ValMin, ValMax, POST_QUADRANGLE, v->VQ, v->NbVQ, 4, 3,
+		      Draw_VectorQuadrangle);
 	  if(v->DrawTensors)
-	    Draw_List(v, ValMin, ValMax, v->TQ, v->NbTQ, 4, Draw_TensorQuadrangle);
+	    Draw_List(v, ValMin, ValMax, POST_QUADRANGLE, v->TQ, v->NbTQ, 4, 9,
+		      Draw_TensorQuadrangle);
 	}
 	if(v->DrawTetrahedra) {
 	  if(!skip_3d && v->DrawScalars)
-	    Draw_List(v, ValMin, ValMax, v->SS, v->NbSS, 4, Draw_ScalarTetrahedron);
+	    Draw_List(v, ValMin, ValMax, POST_TETRAHEDRON, v->SS, v->NbSS, 4, 1,
+		      Draw_ScalarTetrahedron);
 	  if(v->DrawVectors)
-	    Draw_List(v, ValMin, ValMax, v->VS, v->NbVS, 4, Draw_VectorTetrahedron);
+	    Draw_List(v, ValMin, ValMax, POST_TETRAHEDRON, v->VS, v->NbVS, 4, 3,
+		      Draw_VectorTetrahedron);
 	  if(v->DrawTensors)
-	    Draw_List(v, ValMin, ValMax, v->TS, v->NbTS, 4, Draw_TensorTetrahedron);
+	    Draw_List(v, ValMin, ValMax, POST_TETRAHEDRON, v->TS, v->NbTS, 4, 9,
+		      Draw_TensorTetrahedron);
 	}
 	if(v->DrawHexahedra) {
 	  if(!skip_3d && v->DrawScalars)
-	    Draw_List(v, ValMin, ValMax, v->SH, v->NbSH, 8, Draw_ScalarHexahedron);
+	    Draw_List(v, ValMin, ValMax, POST_HEXAHEDRON, v->SH, v->NbSH, 8, 1,
+		      Draw_ScalarHexahedron);
 	  if(v->DrawVectors)
-	    Draw_List(v, ValMin, ValMax, v->VH, v->NbVH, 8, Draw_VectorHexahedron);
+	    Draw_List(v, ValMin, ValMax, POST_HEXAHEDRON, v->VH, v->NbVH, 8, 3,
+		      Draw_VectorHexahedron);
 	  if(v->DrawTensors)
-	    Draw_List(v, ValMin, ValMax, v->TH, v->NbTH, 8, Draw_TensorHexahedron);
+	    Draw_List(v, ValMin, ValMax, POST_HEXAHEDRON, v->TH, v->NbTH, 8, 9,
+		      Draw_TensorHexahedron);
 	}
 	if(v->DrawPrisms) {
 	  if(!skip_3d && v->DrawScalars)
-	    Draw_List(v, ValMin, ValMax, v->SI, v->NbSI, 6, Draw_ScalarPrism);
+	    Draw_List(v, ValMin, ValMax, POST_PRISM, v->SI, v->NbSI, 6, 1,
+		      Draw_ScalarPrism);
 	  if(v->DrawVectors)
-	    Draw_List(v, ValMin, ValMax, v->VI, v->NbVI, 6, Draw_VectorPrism);
+	    Draw_List(v, ValMin, ValMax, POST_PRISM, v->VI, v->NbVI, 6, 3,
+		      Draw_VectorPrism);
 	  if(v->DrawTensors)
-	    Draw_List(v, ValMin, ValMax, v->TI, v->NbTI, 6, Draw_TensorPrism);
+	    Draw_List(v, ValMin, ValMax, POST_PRISM, v->TI, v->NbTI, 6, 9,
+		      Draw_TensorPrism);
 	}
 	if(v->DrawPyramids) {
 	  if(!skip_3d && v->DrawScalars)
-	    Draw_List(v, ValMin, ValMax, v->SY, v->NbSY, 5, Draw_ScalarPyramid);
+	    Draw_List(v, ValMin, ValMax, POST_PYRAMID, v->SY, v->NbSY, 5, 1,
+		      Draw_ScalarPyramid);
 	  if(v->DrawVectors)
-	    Draw_List(v, ValMin, ValMax, v->VY, v->NbVY, 5, Draw_VectorPyramid);
+	    Draw_List(v, ValMin, ValMax, POST_PYRAMID, v->VY, v->NbVY, 5, 3,
+		      Draw_VectorPyramid);
 	  if(v->DrawTensors)
-	    Draw_List(v, ValMin, ValMax, v->TY, v->NbTY, 5, Draw_TensorPyramid);
+	    Draw_List(v, ValMin, ValMax, POST_PYRAMID, v->TY, v->NbTY, 5, 9,
+		      Draw_TensorPyramid);
 	}
 
       pass_1:
@@ -550,7 +746,7 @@ void Draw_Post(void)
 
       if(v->TriVertexArray && v->TriVertexArray->num){
 
-	if(CTX.alpha && ColorTable_IsAlpha(&v->CT) && !CTX.fake_transparency &&
+	if(CTX.alpha && ColorTable_IsAlpha(&v->CT) && !v->FakeTransparency &&
 	   (changedEye() || v->Changed)){
 	  Msg(DEBUG, "Sorting View[%d] for transparency (WITH vertex array)", v->Index);
 	  v->TriVertexArray->sort(storedEye);
@@ -568,7 +764,7 @@ void Draw_Post(void)
 	  glEnable(GL_LIGHTING);
 	else
 	  glDisableClientState(GL_NORMAL_ARRAY);
-	if(v->ShowElement) glEnable(GL_POLYGON_OFFSET_FILL);
+	if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
 	glDrawArrays(GL_TRIANGLES, 0, 3 * v->TriVertexArray->num);
 	glDisable(GL_POLYGON_OFFSET_FILL);
 	glDisable(GL_LIGHTING);
@@ -579,19 +775,39 @@ void Draw_Post(void)
       }
 
       if(v->DrawStrings) {
-	glColor4ubv((GLubyte *) & CTX.color.text);
+	glColor4ubv((GLubyte *) & v->color.text3d);
 	Draw_Text2D3D(3, v->TimeStep, v->NbT3, v->T3D, v->T3C);
       }
 
+      for(int i = 0; i < 6; i++)
+	glDisable((GLenum)(GL_CLIP_PLANE0 + i));
+
+      if(v->Axes && v->Type == DRAW_POST_3D){
+	glColor4ubv((GLubyte *) & v->color.axes);
+	glLineWidth(CTX.line_width);
+	gl2psLineWidth(CTX.line_width * CTX.print.eps_line_width_factor);
+	if(!v->AxesAutoPosition){
+	  Draw_Axes(v->Axes, v->AxesTics, v->AxesFormat, v->AxesLabel, v->AxesPosition);
+	}
+	else{
+	  int ok = 1;
+	  for(int i = 0; i < 6; i++) {
+	    if(fabs(v->TmpBBox[i]) == VAL_INF){
+	      ok = 0;
+	      break;
+	    }
+	  }
+	  if(ok)
+	    Draw_Axes(v->Axes, v->AxesTics, v->AxesFormat, v->AxesLabel, v->TmpBBox);
+	}
+      }
+      
       // reset alpha blending
       if(CTX.alpha){
 	glDisable(GL_BLEND);
 	glEnable(GL_DEPTH_TEST);
       }
 
-      for(int i = 0; i < 6; i++)
-	glDisable((GLenum)(GL_CLIP_PLANE0 + i));
-      
       v->Changed = 0;
     }
   }
