@@ -1,6 +1,6 @@
-// $Id: Scale.cpp,v 1.52 2004/10/26 00:43:23 geuzaine Exp $
+// $Id: Scale.cpp,v 1.60 2005/03/13 09:10:35 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -166,7 +166,7 @@ void draw_scale(Post_View * v,
   int nbv;
   double cv_box;
   if(horizontal){
-    sprintf(label, v->Format, -100*M_PI);
+    sprintf(label, v->Format, -M_PI/1.e4);
     double estim = gl_width(label);
     nbv = (width/estim > v->NbIso) ? v->NbIso : ((width/estim > 2.) ? -2 : -1);
     cv_box = width / nbv;
@@ -182,42 +182,55 @@ void draw_scale(Post_View * v,
     if(v->IntervalsType == DRAW_POST_DISCRETE ||
        v->IntervalsType == DRAW_POST_CONTINUOUS) {
       sprintf(label, v->Format, ValMin);
-      if(horizontal)
-	glRasterPos2d(xmin - gl_width(label)/2., ymin + height + tic);
-      else
+      if(horizontal){
+	glRasterPos2d(xmin, ymin + height + tic);
+	Draw_String_Center(label);
+      }
+      else{
 	glRasterPos2d(xmin + width + tic, ymin - font_a / 3.);
-      Draw_String(label);
-
+	Draw_String(label);
+      }
       sprintf(label, v->Format, ValMax);
-      if(horizontal)
-	glRasterPos2d(xmin + width - gl_width(label)/2., ymin + height + tic);
-      else
+      if(horizontal){
+	glRasterPos2d(xmin + width, ymin + height + tic);
+	Draw_String_Center(label);
+      }
+      else{
 	glRasterPos2d(xmin + width + tic, ymin + height - font_a / 3.);
-      Draw_String(label);
+	Draw_String(label);
+      }
     }
     else {
       sprintf(label, v->Format, ValMin);
-      if(horizontal)
-	glRasterPos2d(xmin + (box / 2) - gl_width(label)/2., ymin + height + tic);
-      else
+      if(horizontal){
+	glRasterPos2d(xmin + (box / 2), ymin + height + tic);
+	Draw_String_Center(label);
+      }
+      else{
 	glRasterPos2d(xmin + width + tic, ymin + (box / 2) - font_a / 3.);
-      Draw_String(label);
-
+	Draw_String(label);
+      }
       sprintf(label, v->Format, ValMax);
-      if(horizontal)
-	glRasterPos2d(xmin + width - (box / 2) - gl_width(label)/2., ymin + height + tic);
-      else
+      if(horizontal){
+	glRasterPos2d(xmin + width - (box / 2), ymin + height + tic);
+	Draw_String_Center(label);
+      }
+      else{
 	glRasterPos2d(xmin + width + tic, ymin + height - (box / 2) - font_a / 3.);
-      Draw_String(label);
+	Draw_String(label);
+      }
     }
     if(nbv == -2){
       double Val = v->GVFI(ValMin, ValMax, v->NbIso, v->NbIso/2);
       sprintf(label, v->Format, Val);
-      if(horizontal)
-	glRasterPos2d(xmin + width/2. - gl_width(label)/2., ymin + height + tic);
-      else
+      if(horizontal){
+	glRasterPos2d(xmin + width/2., ymin + height + tic);
+	Draw_String_Center(label);
+      }
+      else{
 	glRasterPos2d(xmin + width + tic, ymin + height/2 - font_a / 3.);
-      Draw_String(label);
+	Draw_String(label);
+      }
     }
   }
   else {
@@ -226,23 +239,28 @@ void draw_scale(Post_View * v,
       for(int i = 0; i < nbv + 1; i++) {
         double Val = v->GVFI(ValMin, ValMax, nbv + 1, i);
         sprintf(label, v->Format, Val);
-	if(horizontal)
-	  glRasterPos2d(xmin + i * cv_box - gl_width(label) / 2., ymin + height + tic);
-	else
+	if(horizontal){
+	  glRasterPos2d(xmin + i * cv_box, ymin + height + tic);
+	  Draw_String_Center(label);
+	}
+	else{
 	  glRasterPos2d(xmin + width + tic, ymin + i * cv_box - font_a / 3.);
-        Draw_String(label);
+	  Draw_String(label);
+	}
       }
     }
     else {
       for(int i = 0; i < nbv; i++) {
         double Val = v->GVFI(ValMin, ValMax, nbv, i);
         sprintf(label, v->Format, Val);
-	if(horizontal)
-	  glRasterPos2d(xmin + (2 * i + 1) * (cv_box / 2) - gl_width(label) / 2., 
-			ymin + height + tic);
-	else
+	if(horizontal){
+	  glRasterPos2d(xmin + (2 * i + 1) * (cv_box / 2), ymin + height + tic);
+	  Draw_String_Center(label);
+	}
+	else{
 	  glRasterPos2d(xmin + width + tic, ymin + (2 * i + 1) * (cv_box / 2) - font_a / 3.);
-        Draw_String(label);
+	  Draw_String(label);
+	}
       }
     }
   }
@@ -250,17 +268,22 @@ void draw_scale(Post_View * v,
   // the label
 
   if((v->ShowTime == 1 && List_Nbr(v->Time) > 1) ||
-     (v->ShowTime == 2 && List_Nbr(v->Time) > 0))
-    sprintf(label, "%s (%g)", v->Name, *(double *)List_Pointer(v->Time, v->TimeStep));
+     (v->ShowTime == 2 && List_Nbr(v->Time) > 0)){
+    char tmp[256];
+    sprintf(tmp, v->Format, *(double *)List_Pointer(v->Time, v->TimeStep));
+    sprintf(label, "%s (%s)", v->Name, tmp);
+  }
   else
     sprintf(label, "%s", v->Name);
 
-  if(horizontal)
-    glRasterPos2d(xmin + width / 2. - gl_width(label) / 2., ymin + height + tic + 1.4*font_h);
-  else
+  if(horizontal){
+    glRasterPos2d(xmin + width / 2., ymin + height + tic + 1.4*font_h);
+    Draw_String_Center(label);
+  }
+  else{
     glRasterPos2d(xmin, ymin - 2 * font_h);
-
-  Draw_String(label);
+    Draw_String(label);
+  }
 }
 
 void Draw_Scales(void)
@@ -278,7 +301,7 @@ void Draw_Scales(void)
     List_Reset(todraw);
 
   for(int i = 0; i < List_Nbr(CTX.post.list); i++) {
-    Post_View *v = (Post_View *) List_Pointer(CTX.post.list, i);
+    Post_View *v = *(Post_View **) List_Pointer(CTX.post.list, i);
     if(v->Visible && !v->Dirty && v->ShowScale &&
        v->Type == DRAW_POST_3D && !v->TextOnly)
       List_Add(todraw, &v);
@@ -296,20 +319,21 @@ void Draw_Scales(void)
   double largest_number = 0.;
   for(int i = 0; i < List_Nbr(todraw); i++) {
     Post_View *v = *(Post_View **) List_Pointer(todraw, i);
-    sprintf(label, v->Format, -100*M_PI);
+    sprintf(label, v->Format, -M_PI/1.e4);
     if(largest_number < gl_width(label))
       largest_number = gl_width(label);
   }
 
   for(int i = 0; i < List_Nbr(todraw); i++) {
     Post_View *v = *(Post_View **) List_Pointer(todraw, i);
-
+    
     if(!v->AutoPosition) {
-	draw_scale(v, 
-		   v->Position[0], 
-		   CTX.viewport[3] - v->Size[1] - v->Position[1], 
-		   v->Size[0], v->Size[1], 
-		   tic, CTX.post.horizontal_scales);
+      double x = v->Position[0], y = v->Position[1];
+      int center = Fix2DCoordinates(&x, &y);
+      draw_scale(v, 
+		 x - ((center & 1) ? v->Size[0]/2. : 0.), 
+		 y - v->Size[1] + ((center & 2) ? v->Size[1]/2. : 0.),
+		 v->Size[0], v->Size[1], tic, CTX.post.horizontal_scales);
     }
     else{
       if(CTX.post.horizontal_scales){
@@ -356,10 +380,13 @@ void Draw_Scales(void)
 	}
 	// compute width
 	width_prev = width;
-	sprintf(label, v->Format, -100*M_PI);
+	sprintf(label, v->Format, -M_PI/1.e4);
 	width = bar_size + tic + gl_width(label);
-	if(List_Nbr(v->Time) > 1 && v->ShowTime)
-	  sprintf(label, "%s (%g)", v->Name, *(double *)List_Pointer(v->Time, v->TimeStep));
+	if(List_Nbr(v->Time) > 1 && v->ShowTime){
+	  char tmp[256];
+	  sprintf(tmp, v->Format, *(double *)List_Pointer(v->Time, v->TimeStep));
+	  sprintf(label, "%s (%s)", v->Name, tmp);
+	}
 	else
 	  sprintf(label, "%s", v->Name);
 	if(gl_width(label) > width)
