@@ -1,6 +1,6 @@
-// $Id: 1D_Mesh.cpp,v 1.40 2004/05/26 00:33:37 geuzaine Exp $
+// $Id: 1D_Mesh.cpp,v 1.44 2005/02/20 07:11:04 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ double F_One(double t)
 double F_Transfini(double t)
 {
   Vertex der;
-  double d, a, b, val;
+  double d, a, b, val, r;
   int i;
 
   der = InterpolateCurve(THEC, t, 1);
@@ -66,16 +66,16 @@ double F_Transfini(double t)
     switch (abs(THEC->ipar[1])) {
 
     case 1:    // Geometric progression ar^i; Sum of n terms = THEC->l = a (r^n-1)/(r-1)
-      if(THEC->dpar[0] == 1.)
+      if(sign(THEC->ipar[1]) >= 0)
+	r = THEC->dpar[0];
+      else
+	r = 1. / THEC->dpar[0];
+      if(r == 1.)
         a = THEC->l / (double)THEC->ipar[0];
       else
-        a =
-          THEC->l * (THEC->dpar[0] -
-                     1.) / (pow(THEC->dpar[0], THEC->ipar[0]) - 1.);
-      i =
-        (int)(log(t * THEC->l / a * (THEC->dpar[0] - 1.) + 1.) /
-              log(THEC->dpar[0]));
-      val = d / (a * pow(THEC->dpar[0], (double)i));
+        a = THEC->l * (r - 1.) / (pow(r, THEC->ipar[0]) - 1.);
+      i = (int)(log(t * THEC->l / a * (r - 1.) + 1.) / log(r));
+      val = d / (a * pow(r, (double)i));
       break;
 
     case 2:    // Bump
@@ -144,6 +144,10 @@ void Maillage_Curve(void *data, void *dummy)
 
   Msg(STATUS3, "Meshing curve %d", c->Num);
 
+  if(MeshDiscreteCurve(c)){
+    return;
+  }
+  
   Points = List_Create(10, 10, sizeof(IntPoint));
   c->l = Integration(c->ubeg, c->uend, F_One, Points, 1.e-4);
   List_Delete(Points);
@@ -178,7 +182,6 @@ void Maillage_Curve(void *data, void *dummy)
     }
   }
   b = a / (double)(N - 1);
-  c->Vertices = List_Create(N, 2, sizeof(Vertex *));
   
   v = &c->beg;
   if((vexist = (Vertex **) Tree_PQuery(THEM->Vertices, v))) {
