@@ -1,6 +1,6 @@
-// $Id: 2D_Mesh.cpp,v 1.68 2004/07/14 22:42:26 geuzaine Exp $
+// $Id: 2D_Mesh.cpp,v 1.73 2005/01/20 19:05:09 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -275,7 +275,7 @@ int mesh_domain(ContourPeek * ListContours, int numcontours,
 
   if(!numcontours) {
     Msg(GERROR, "No contour");
-    return 0;
+    return -1;
   }
 
   numact = 0;
@@ -325,7 +325,6 @@ int mesh_domain(ContourPeek * ListContours, int numcontours,
       List_Add(del_L, &del_P);
   }
   doc->numTriangles = List_Nbr(del_L);
-
   verify_edges(del_L, ListContours, numcontours);
   verify_inside(doc->delaunay, doc->numTriangles);
 
@@ -625,9 +624,10 @@ void Maillage_Automatique_VieuxCode(Surface * pS, Mesh * m, int ori)
     liste[i] = cp;
   }
 
+  int res_mesh_domain = 1;
   if(pS->Method)
-    mesh_domain(liste, List_Nbr(pS->Contours), &M, &N,
-                (CTX.mesh.initial_only == 2));
+    res_mesh_domain = mesh_domain(liste, List_Nbr(pS->Contours), &M, &N,
+				  (CTX.mesh.initial_only == 2));
 
   for(i = 0; i < M.numpoints; i++) {
     if(gPointArray[i].initial < 0) {
@@ -682,7 +682,9 @@ void Maillage_Automatique_VieuxCode(Surface * pS, Mesh * m, int ori)
     //xxxxFree(M.listdel[i]);
 
   }
-  Free(M.listdel);
+  
+  if(res_mesh_domain >= 0)
+    Free(M.listdel);
   Free(gPointArray);
 }
 
@@ -855,11 +857,6 @@ void Maillage_Surface(void *data, void *dum)
     return;
   }
 
-  if(s->Typ == MSH_SURF_STL) {
-    Msg(GERROR, "Remeshing of STL surfaces is not implemented (yet)");
-    return;
-  }
-
   THESUPPORT = s->Support;
   THESURFACE = s;
 
@@ -872,7 +869,8 @@ void Maillage_Surface(void *data, void *dum)
 
   Msg(STATUS3, "Meshing surface %d", s->Num);
 
-  if(MeshTransfiniteSurface(s) ||
+  if(MeshDiscreteSurface(s) ||
+     MeshTransfiniteSurface(s) ||
      MeshEllipticSurface(s) ||
      MeshCylindricalSurface(s) ||
      MeshParametricSurface(s) ||
@@ -882,6 +880,12 @@ void Maillage_Surface(void *data, void *dum)
     Tree_Action(s->Vertices, Add_In_Mesh);
   }
   else{
+
+    if(!List_Nbr(s->Generatrices)){
+      Msg(GERROR, "Cannot mesh surface with no boundary");
+      return;
+    }
+
     int TypSurface = s->Typ;
     Plan_Moyen(pS, dum);
     Tree_Action(THEM->Points, Freeze_Vertex);
