@@ -1,6 +1,6 @@
-// $Id: 3D_Extrude.cpp,v 1.84 2004/07/02 04:56:30 geuzaine Exp $
+// $Id: 3D_Extrude.cpp,v 1.87 2005/02/20 07:11:04 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -378,7 +378,6 @@ void Create_HexPri(int iEnt, Vertex * v[8])
           v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7]);
       return;
     }
-    printf("aaa %d %d\n", iEnt, THEV->Num);
     newp->iEnt = (iEnt && ep->useZonLayer()) ? iEnt : THEV->Num;
     Tree_Add(THEV->Prisms, &newp);
   }
@@ -735,8 +734,6 @@ void copy_mesh(Curve * from, Curve * to, int direction)
     return;
   }
 
-  to->Vertices = List_Create(List_Nbr(from->Vertices), 2, sizeof(Vertex *));
-
   v = to->beg;
   if((vexist = (Vertex **) Tree_PQuery(THEM->Vertices, &v))) {
     (*vexist)->u = to->ubeg;
@@ -830,7 +827,6 @@ int Extrude_Mesh(Curve * c)
     Extrude_Vertex(&c->beg, NULL);
     L = getnxl(c->beg, DIM);
     if(!L) return false;
-    c->Vertices = List_Create(List_Nbr(L), 2, sizeof(Vertex *));
 
     v = c->beg;
     if((vexist = (Vertex **) Tree_PQuery(THEM->Vertices, &v))) {
