@@ -1,6 +1,6 @@
-// $Id: Create.cpp,v 1.63 2004/08/12 16:57:32 geuzaine Exp $
+// $Id: Create.cpp,v 1.69 2005/02/26 04:53:13 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -467,17 +467,21 @@ void End_Curve(Curve * c)
 
   }
 
-  if(c->cp)
+  if(c->cp){
     Free(c->cp);
-  c->cp = (float *)Malloc(4 * List_Nbr(c->Control_Points) * sizeof(float));
-  for(i = 0; i < List_Nbr(c->Control_Points); i++) {
-    List_Read(c->Control_Points, i, &v[0]);
-    c->cp[4 * i] = v[0]->Pos.X;
-    c->cp[4 * i + 1] = v[0]->Pos.Y;
-    c->cp[4 * i + 2] = v[0]->Pos.Z;
-    c->cp[4 * i + 3] = v[0]->w;
+    c->cp = NULL;
   }
 
+  if(List_Nbr(c->Control_Points)){
+    c->cp = (float *)Malloc(4 * List_Nbr(c->Control_Points) * sizeof(float));
+    for(i = 0; i < List_Nbr(c->Control_Points); i++) {
+      List_Read(c->Control_Points, i, &v[0]);
+      c->cp[4 * i] = v[0]->Pos.X;
+      c->cp[4 * i + 1] = v[0]->Pos.Y;
+      c->cp[4 * i + 2] = v[0]->Pos.Z;
+      c->cp[4 * i + 3] = v[0]->w;
+    }
+  }
 }
 
 void End_Surface(Surface * s, int reset_orientations)
@@ -528,12 +532,14 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
   pC->Color.type = 0;
   pC->Visible = VIS_GEOM | VIS_MESH;
   pC->cp = NULL;
-  pC->Vertices = NULL;
+  pC->Vertices = List_Create(2, 20, sizeof(Vertex *));
   pC->Extrude = NULL;
+  pC->theSegmRep = 0;
   pC->Typ = Typ;
   pC->Num = Num;
   THEM->MaxLineNum = IMAX(THEM->MaxLineNum, Num);
   pC->Simplexes = Tree_Create(sizeof(Simplex *), compareSimplex);
+  pC->SimplexesBase = Tree_Create(sizeof(SimplexBase *), compareSimplexBase);
   pC->Method = LIBRE;
   pC->degre = Order;
   pC->Circle.n[0] = 0.0;
@@ -592,6 +598,8 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
   }
   else {
     pC->Control_Points = NULL;
+    pC->beg = NULL;
+    pC->end = NULL;
     return pC;
   }
 
@@ -630,9 +638,13 @@ void Free_Curve(void *a, void *b)
     List_Delete(pC->Vertices);
     Tree_Action(pC->Simplexes, Free_Simplex);
     Tree_Delete(pC->Simplexes);
+    Tree_Action(pC->SimplexesBase, Free_SimplexBase);
+    Tree_Delete(pC->SimplexesBase);
     Free(pC->k);
     List_Delete(pC->Control_Points);
     Free(pC->cp);
+    if(pC->theSegmRep)
+      delete pC->theSegmRep;
     Free(pC);
     pC = NULL;
   }
@@ -658,6 +670,7 @@ Surface *Create_Surface(int Num, int Typ)
   pS->RecombineAngle = 75;
   pS->TrsfPoints = List_Create(4, 4, sizeof(Vertex *));
   pS->Simplexes = Tree_Create(sizeof(Simplex *), compareQuality);
+  pS->SimplexesBase = Tree_Create(sizeof(SimplexBase *), compareSimplexBase);
   pS->Quadrangles = Tree_Create(sizeof(Quadrangle *), compareQuadrangle);
   pS->Vertices = Tree_Create(sizeof(Vertex *), compareVertex);
   pS->TrsfVertices = List_Create(1, 10, sizeof(Vertex *));
@@ -668,7 +681,6 @@ Surface *Create_Surface(int Num, int Typ)
   pS->Generatrices = NULL;
   pS->Edges = NULL;
   pS->Extrude = NULL;
-  pS->STL = NULL;
   pS->TriVertexArray = NULL;
   pS->QuadVertexArray = NULL;
   pS->normals = new smooth_normals;
@@ -682,6 +694,8 @@ void Free_Surface(void *a, void *b)
     List_Delete(pS->TrsfPoints);
     Tree_Action(pS->Simplexes, Free_Simplex);
     Tree_Delete(pS->Simplexes);
+    Tree_Action(pS->SimplexesBase, Free_SimplexBase);
+    Tree_Delete(pS->SimplexesBase);
     Tree_Action(pS->Quadrangles, Free_Quadrangle);
     Tree_Delete(pS->Quadrangles);
     Tree_Delete(pS->Vertices);
@@ -700,6 +714,8 @@ void Free_Surface(void *a, void *b)
       delete pS->QuadVertexArray;
     if(pS->normals)
       delete pS->normals;
+    if(pS->thePolyRep)
+      delete pS->thePolyRep;
     Free(pS);
     pS = NULL;
   }
@@ -724,6 +740,7 @@ Volume *Create_Volume(int Num, int Typ)
   pV->Surfaces = List_Create(1, 2, sizeof(Surface *));
   pV->SurfacesOrientations = List_Create(1, 2, sizeof(int));
   pV->Simplexes = Tree_Create(sizeof(Simplex *), compareQuality);
+  pV->SimplexesBase = Tree_Create(sizeof(Simplex *), compareSimplexBase);
   pV->Vertices = Tree_Create(sizeof(Vertex *), compareVertex);
   pV->Hexahedra = Tree_Create(sizeof(Hexahedron *), compareHexahedron);
   pV->Prisms = Tree_Create(sizeof(Prism *), comparePrism);
@@ -744,6 +761,7 @@ void Free_Volume(void *a, void *b)
   Volume *pV = *(Volume **) a;
   if(pV) {
     Tree_Action(pV->Simplexes, Free_Simplex);
+    Tree_Action(pV->SimplexesBase, Free_SimplexBase);
     Tree_Action(pV->Hexahedra, Free_Hexahedron);
     Tree_Action(pV->Prisms, Free_Prism);
     Tree_Action(pV->Pyramids, Free_Pyramid);
@@ -760,6 +778,7 @@ void Free_Volume_But_Not_Elements(void *a, void *b)
     List_Delete(pV->Surfaces);  // surfaces freed elsewhere
     List_Delete(pV->SurfacesOrientations);
     Tree_Delete(pV->Simplexes);
+    Tree_Delete(pV->SimplexesBase);
     Tree_Delete(pV->Simp_Surf); // for old extrusion mesh generator
     Tree_Delete(pV->Quad_Surf); // for old extrusion mesh generator
     Tree_Delete(pV->Vertices);  // vertices freed elsewhere
