@@ -1,6 +1,6 @@
-// $Id: Generator.cpp,v 1.60 2004/08/13 20:59:44 geuzaine Exp $
+// $Id: Generator.cpp,v 1.63 2005/01/01 19:35:30 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ void GetStatistics(double stat[50])
       Surface *s;
       List_Read(surfaces, i, &s);
       stat[5] += Tree_Nbr(s->Vertices);
-      stat[7] += Tree_Nbr(s->Simplexes);
+      stat[7] += Tree_Nbr(s->Simplexes) + Tree_Nbr(s->SimplexesBase);
       stat[8] += Tree_Nbr(s->Quadrangles);
     }
     List_Delete(surfaces);
@@ -82,7 +82,7 @@ void GetStatistics(double stat[50])
       Volume *v;
       List_Read(volumes, i, &v);
       stat[6] += Tree_Nbr(v->Vertices);
-      stat[9] += Tree_Nbr(v->Simplexes);
+      stat[9] += Tree_Nbr(v->Simplexes) + Tree_Nbr(v->SimplexesBase);
       stat[10] += Tree_Nbr(v->Hexahedra);
       stat[11] += Tree_Nbr(v->Prisms);
       stat[12] += Tree_Nbr(v->Pyramids);
@@ -121,7 +121,7 @@ void GetStatistics(double stat[50])
 
   stat[26] = List_Nbr(CTX.post.list);
   for(int i = 0; i < List_Nbr(CTX.post.list); i++) {
-    Post_View *v = (Post_View *) List_Pointer(CTX.post.list, i);
+    Post_View *v = *(Post_View **) List_Pointer(CTX.post.list, i);
     stat[27] += v->NbSP + v->NbVP + v->NbTP;
     stat[28] += v->NbSL + v->NbVL + v->NbTL;
     stat[29] += v->NbST + v->NbVT + v->NbTT;
