@@ -1,7 +1,7 @@
 #ifndef _MESH_H_
 #define _MESH_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #include "Simplex.h"
 #include "Edge.h"
 #include "ExtrudeParams.h"
-#include "STL.h"
+#include "DiscreteSurface.h"
 #include "VertexArray.h"
 #include "SmoothNormals.h"
 
@@ -56,13 +56,9 @@
 #define FORMAT_PNGTEX        23
 #define FORMAT_PDF           24
 #define FORMAT_PDFTEX        25
-#define FORMAT_PS_RASTER     26
-#define FORMAT_EPS_RASTER    27
-#define FORMAT_PDF_RASTER    28
-#define FORMAT_EPSTEX_RASTER 29
-#define FORMAT_PDFTEX_RASTER 30
-#define FORMAT_LC_SUR        31
-#define FORMAT_LC_VOL        32
+#define FORMAT_LC            26
+#define FORMAT_STL           27
+#define FORMAT_P3D           28
 
 #define CONV_VALUE    0.8
 
@@ -242,7 +238,7 @@ struct _Surf{
   double a, b, c, d;
   List_T *Orientations;
   List_T *Contours;
-  Tree_T *Simplexes;
+  Tree_T *Simplexes, *SimplexesBase;
   Tree_T *Quadrangles;
   Tree_T *Vertices;
   List_T *TrsfVertices;
@@ -253,7 +249,6 @@ struct _Surf{
   CylParam Cyl;
   Grid_T Grid;  // fast search grid
   ExtrudeParams *Extrude;
-  STL_Data *STL; // stl representation of the surface
   POLY_rep *thePolyRep;
   int Dirty; // flag to prevent any meshing
   DrawingColor Color;
@@ -307,7 +302,7 @@ typedef struct {
   Tree_T *Vertices;
   Tree_T *Edges;
   Tree_T *Faces;
-  Tree_T *Simplexes;
+  Tree_T *Simplexes, *SimplexesBase;
   Tree_T *Simp_Surf; // for old extrusion mesh generator
   Tree_T *Quad_Surf; // for old extrusion mesh generator
   Tree_T *Hexahedra;
@@ -371,7 +366,7 @@ typedef struct{
   double ubeg, uend;
   List_T *Control_Points;
   List_T *Vertices;
-  Tree_T *Simplexes;
+  Tree_T *Simplexes, *SimplexesBase;
   ExtrudeParams *Extrude;
   float *k, *cp;
   int degre;
@@ -379,6 +374,7 @@ typedef struct{
   char functu[256], functv[256], functw[256];
   int Dirty; // flag to prevent any meshing
   DrawingColor Color;
+  SEGM_rep *theSegmRep;
 }Curve;
 
 typedef struct{
@@ -443,7 +439,7 @@ void mai3d(Mesh *M, int Asked);
 
 void Init_Mesh(Mesh *M);
 void Create_BgMesh(int i, double d, Mesh *m);
-void Print_Geo(Mesh *M, char *c);
+void Print_Geo(Mesh *M, char *c, int discrete_curves=0, int discrete_surf=0);
 void Print_Mesh(Mesh *M, char *c, int Type);
 void Read_Mesh(Mesh *M, FILE *fp, char *filename, int Type);
 void GetStatistics(double s[50]);
@@ -468,6 +464,8 @@ int MeshTransfiniteVolume(Volume *vol);
 int MeshCylindricalSurface(Surface *s);
 int MeshParametricSurface(Surface *s);
 int MeshEllipticSurface(Surface *sur);
+int MeshDiscreteSurface(Surface *sur);
+int MeshDiscreteCurve(Curve *c);
 
 int AlgorithmeMaillage2DAnisotropeModeJF(Surface *s);
 void Maillage_Automatique_VieuxCode(Surface *pS, Mesh *m, int ori);
@@ -492,8 +490,7 @@ void ActionLissSurf(void *data, void *dummy);
 int Recombine(Tree_T *TreeAllVert, Tree_T *TreeAllSimp, Tree_T *TreeAllQuad,
 		double a);
 void ApplyLcFactor(Mesh *M);
-void ExportLcFieldOnVolume(Mesh *M, char *filename);
-void ExportLcFieldOnSurfaces(Mesh *M, char *filename);
+void ExportLcField(Mesh *M, char *filename, int volume=1, int surface=1);
 
 void Degre1();
 void Degre2(int dim);
