@@ -1,6 +1,6 @@
-// $Id: Print_Mesh.cpp,v 1.55 2004/06/08 02:10:32 geuzaine Exp $
+// $Id: Print_Mesh.cpp,v 1.60 2005/02/28 23:57:59 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ static void _msh_print_simplex(void *a, void *b)
 {
   int i, type, nbn, nbs = 0;
 
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
 
   if(MSH_VOL_NUM && (MSH_VOL_NUM != s->iEnt))
     return;
@@ -428,6 +428,7 @@ static void _msh_print_elements(Mesh *M)
 	    MSH_LIN_NUM = abs(Num);
 	    MSH_PHYSICAL_ORI = sign(Num);
 	    Tree_Action(pc->Simplexes, _msh_print_simplex);
+	    Tree_Action(pc->SimplexesBase, _msh_print_simplex);
 	  }
 	}
       }
@@ -454,6 +455,7 @@ static void _msh_print_elements(Mesh *M)
 	    MSH_SUR_NUM = abs(Num);
 	    MSH_PHYSICAL_ORI = sign(Num);
 	    Tree_Action(ps->Simplexes, _msh_print_simplex);
+	    Tree_Action(ps->SimplexesBase, _msh_print_simplex);
 	    Tree_Action(ps->Quadrangles, _msh_print_quadrangle);
 	  }
 	}
@@ -468,6 +470,7 @@ static void _msh_print_elements(Mesh *M)
           MSH_VOL_NUM = abs(Num);
           MSH_PHYSICAL_ORI = sign(Num);
           Tree_Action(pV->Simplexes, _msh_print_simplex);
+          Tree_Action(pV->SimplexesBase, _msh_print_simplex);
           Tree_Action(pV->Hexahedra, _msh_print_hexahedron);
           Tree_Action(pV->Prisms, _msh_print_prism);
           Tree_Action(pV->Pyramids, _msh_print_pyramid);
@@ -491,12 +494,14 @@ static void _msh_print_all_curves(void *a, void *b)
 {
   Curve *c = *(Curve **) a;
   Tree_Action(c->Simplexes, _msh_print_simplex);
+  Tree_Action(c->SimplexesBase, _msh_print_simplex);
 }
 
 static void _msh_print_all_surfaces(void *a, void *b)
 {
   Surface *s = *(Surface **) a;
   Tree_Action(s->Simplexes, _msh_print_simplex);
+  Tree_Action(s->SimplexesBase, _msh_print_simplex);
   Tree_Action(s->Quadrangles, _msh_print_quadrangle);
 }
 
@@ -511,6 +516,7 @@ static void _msh_print_all_volumes(void *a, void *b)
 {
   Volume *v = *(Volume **) a;
   Tree_Action(v->Simplexes, _msh_print_simplex);
+  Tree_Action(v->SimplexesBase, _msh_print_simplex);
   Tree_Action(v->Hexahedra, _msh_print_hexahedron);
   Tree_Action(v->Prisms, _msh_print_prism);
   Tree_Action(v->Pyramids, _msh_print_pyramid);
@@ -622,25 +628,27 @@ static FILE *UNVFILE;
 
 static void _unv_process_nodes(Mesh *M)
 {
-  int nbnod;
-  double x, y, z;
-  int i, idnod;
   Vertex *v;
-
   List_T *Nodes = Tree2List(M->Vertices);
 
   fprintf(UNVFILE, "%6d\n", -1);
   fprintf(UNVFILE, "%6d\n", NODES);
-  nbnod = List_Nbr(Nodes);
+  int nbnod = List_Nbr(Nodes);
 
-  for(i = 0; i < nbnod; i++) {
+  for(int i = 0; i < nbnod; i++) {
     List_Read(Nodes, i, &v);
-    idnod = v->Num;
-    x = v->Pos.X * CTX.mesh.scaling_factor;
-    y = v->Pos.Y * CTX.mesh.scaling_factor;
-    z = v->Pos.Z * CTX.mesh.scaling_factor;
+    int idnod = v->Num;
+    double x = v->Pos.X * CTX.mesh.scaling_factor;
+    double y = v->Pos.Y * CTX.mesh.scaling_factor;
+    double z = v->Pos.Z * CTX.mesh.scaling_factor;
     fprintf(UNVFILE, "%10d%10d%10d%10d\n", idnod, 1, 1, 11);
-    fprintf(UNVFILE, "%21.16fD+00 %21.16fD+00 %21.16fD+00\n", x, y, z);
+    char tmp[128];
+    // ugly hack to print number with 'D+XX' exponents
+    sprintf(tmp, "%25.16E%25.16E%25.16E\n", x, y, z);
+    tmp[21] = 'D';
+    tmp[46] = 'D';
+    tmp[71] = 'D';
+    fprintf(UNVFILE, tmp);    
   }
 
   List_Delete(Nodes);
@@ -677,7 +685,7 @@ static void _unv_process_1D_elements(Mesh *m)
   List_T *AllCurves = List_Create(2, 2, sizeof(Surface *));
   List_T *ListSurfaces = Tree2List(m->Surfaces);
   List_T *Elements;
-  Simplex *sx;
+  SimplexBase *sx;
   Curve *c;
   Surface *surf;
 
@@ -685,10 +693,10 @@ static void _unv_process_1D_elements(Mesh *m)
     List_Read(ListSurfaces, i, &surf);
     for(int j = 0; j < List_Nbr(surf->Generatrices); j++) {
       List_Read(surf->Generatrices, j, &c);
-      if(Tree_Nbr(c->Simplexes))
+      if(Tree_Nbr(c->Simplexes) || Tree_Nbr(c->SimplexesBase))
         List_Add(AllCurves, &c);
       c = FindCurve(-c->Num, m);
-      if(Tree_Nbr(c->Simplexes))
+      if(Tree_Nbr(c->Simplexes) || Tree_Nbr(c->SimplexesBase))
         List_Add(AllCurves, &c);
     }
   }
@@ -696,15 +704,17 @@ static void _unv_process_1D_elements(Mesh *m)
   for(int i = 0; i < List_Nbr(ListCurves); i++) {
     List_Read(ListCurves, i, &c);
     if(!List_Search(AllCurves, &c, compareCurve)) {
-      Elements = Tree2List(c->Simplexes);
-      for(int j = 0; j < List_Nbr(Elements); j++) {
-        List_Read(Elements, j, &sx);
-        if(sx->VSUP)
-	  _unv_print_record(sx->Num, BEAM2, c->Num, 2, 2, &sx->V[0], sx->VSUP);
-	else 
-	  _unv_print_record(sx->Num, BEAM, c->Num, 2, 0, &sx->V[0], NULL);
+      for(int simtype = 0; simtype < 2; simtype ++){
+	Elements = (!simtype) ? Tree2List(c->Simplexes) : Tree2List(c->SimplexesBase);
+	for(int j = 0; j < List_Nbr(Elements); j++) {
+	  List_Read(Elements, j, &sx);
+	  if(sx->VSUP)
+	    _unv_print_record(sx->Num, BEAM2, c->Num, 2, 2, &sx->V[0], sx->VSUP);
+	  else 
+	    _unv_print_record(sx->Num, BEAM, c->Num, 2, 0, &sx->V[0], NULL);
+	}
+	List_Delete(Elements);
       }
-      List_Delete(Elements);
     }
   }
 
@@ -721,14 +731,15 @@ static void _unv_process_2D_elements(Mesh *m)
   List_T *Elements;
   Volume *vol;
   Surface *s;
-  Simplex *sx;
+  SimplexBase *sx;
   Quadrangle *qx;
 
   for(int i = 0; i < List_Nbr(ListVolumes); i++) {
     List_Read(ListVolumes, i, &vol);
     for(int j = 0; j < List_Nbr(vol->Surfaces); j++) {
       List_Read(vol->Surfaces, j, &s);
-      if(Tree_Nbr(s->Simplexes) || Tree_Nbr(s->Quadrangles))
+      if(Tree_Nbr(s->Simplexes) || Tree_Nbr(s->SimplexesBase) ||
+	 Tree_Nbr(s->Quadrangles))
         List_Add(AllSurfaces, &s);
     }
   }
@@ -738,16 +749,18 @@ static void _unv_process_2D_elements(Mesh *m)
     if(!List_Search(AllSurfaces, &s, compareSurface)) {
       
       // triangles
-      Elements = Tree2List(s->Simplexes);
-      for(int j = 0; j < List_Nbr(Elements); j++) {
-        List_Read(Elements, j, &sx);
-	if(sx->VSUP)
-	  _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 3, &sx->V[0], sx->VSUP);
-	else
-	  _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 0, &sx->V[0], NULL);
+      for(int simtype = 0; simtype < 2; simtype++){
+	Elements = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
+	for(int j = 0; j < List_Nbr(Elements); j++) {
+	  List_Read(Elements, j, &sx);
+	  if(sx->VSUP)
+	    _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 3, &sx->V[0], sx->VSUP);
+	  else
+	    _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 0, &sx->V[0], NULL);
+	}
+	List_Delete(Elements);
       }
-      List_Delete(Elements);
-      
+
       // quadrangles
       Elements = Tree2List(s->Quadrangles);
       for(int j = 0; j < List_Nbr(Elements); j++) {
@@ -769,7 +782,7 @@ static void _unv_process_3D_elements(Mesh *m)
 {
   List_T *ListVolumes = Tree2List(m->Volumes);
   List_T *Elements;
-  Simplex *sx;
+  SimplexBase *sx;
   Hexahedron *hx;
   Prism *px;
   Volume *v;
@@ -778,29 +791,31 @@ static void _unv_process_3D_elements(Mesh *m)
     List_Read(ListVolumes, i, &v);
 
     // Tets
-    Elements = Tree2List(v->Simplexes);
-    for(int j = 0; j < List_Nbr(Elements); j++) {
-      List_Read(Elements, j, &sx);
-      if(sx->Volume_Simplexe() < 0) {
-        Vertex *temp;
-        temp = sx->V[0];
-        sx->V[0] = sx->V[1];
-        sx->V[1] = temp;
-	if(sx->VSUP){
-	  temp = sx->VSUP[1];
-	  sx->VSUP[1] = sx->VSUP[2];
-	  sx->VSUP[2] = temp;
-	  temp = sx->VSUP[5];
-	  sx->VSUP[5] = sx->VSUP[3];
-	  sx->VSUP[3] = temp;
+    for(int simtype = 0; simtype < 2; simtype++){
+      Elements = (!simtype) ? Tree2List(v->Simplexes) : Tree2List(v->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Elements); j++) {
+	List_Read(Elements, j, &sx);
+	if(sx->Volume_Simplexe() < 0) {
+	  Vertex *temp;
+	  temp = sx->V[0];
+	  sx->V[0] = sx->V[1];
+	  sx->V[1] = temp;
+	  if(sx->VSUP){
+	    temp = sx->VSUP[1];
+	    sx->VSUP[1] = sx->VSUP[2];
+	    sx->VSUP[2] = temp;
+	    temp = sx->VSUP[5];
+	    sx->VSUP[5] = sx->VSUP[3];
+	    sx->VSUP[3] = temp;
+	  }
 	}
+	if(sx->VSUP)
+	  _unv_print_record(ELEMENT_ID++, SOLIDFEM, v->Num, 4, 6, &sx->V[0], sx->VSUP);
+	else
+	  _unv_print_record(ELEMENT_ID++, SOLIDFEM, v->Num, 4, 0, &sx->V[0], NULL);
       }
-      if(sx->VSUP)
-	_unv_print_record(ELEMENT_ID++, SOLIDFEM, v->Num, 4, 6, &sx->V[0], sx->VSUP);
-      else
-	_unv_print_record(ELEMENT_ID++, SOLIDFEM, v->Num, 4, 0, &sx->V[0], NULL);
+      List_Delete(Elements);
     }
-    List_Delete(Elements);
 
     // Prisms
     Elements = Tree2List(v->Prisms);
@@ -834,14 +849,14 @@ static void _unv_add_vertex(void *a, void *b)
     return;
   Tree_Add(tree, &v->Num);
   fprintf(UNVFILE, "%10d%10d%2d%2d%2d%2d%2d%2d\n", v->Num, 1, 0, 1, 0, 0, 0, 0);
-  fprintf(UNVFILE, "%21.16fD+00 %21.16fD+00 %21.16fD+00\n", 0., 1., 0.);
-  fprintf(UNVFILE, "%21.16fD+00 %21.16fD+00 %21.16fD+00\n", 0., 0., 0.);
+  fprintf(UNVFILE, "   0.0000000000000000D+00   1.0000000000000000D+00   0.0000000000000000D+00\n");
+  fprintf(UNVFILE, "   0.0000000000000000D+00   0.0000000000000000D+00   0.0000000000000000D+00\n");
   fprintf(UNVFILE, "%10d%10d%10d%10d%10d%10d\n", 0, 0, 0, 0, 0, 0);
 }
 
 static void _unv_add_simplex_vertices(void *a, void *b)
 {
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
   if(s->iEnt != UNV_VOL_NUM)
     return;
   for(int i = 0; i < 4; i++)
@@ -903,6 +918,7 @@ static void _unv_process_groups(Mesh *m)
         for(int j = 0; j < List_Nbr(p->Entities); j++) {
           List_Read(p->Entities, j, &UNV_VOL_NUM);
           Tree_Action(pV->Simplexes, _unv_add_simplex_vertices);
+          Tree_Action(pV->SimplexesBase, _unv_add_simplex_vertices);
           Tree_Action(pV->Hexahedra, _unv_add_hexahedron_vertices);
           Tree_Action(pV->Prisms, _unv_add_prism_vertices);
           Tree_Action(pV->Pyramids, _unv_add_pyramid_vertices);
@@ -964,7 +980,7 @@ static FILE *GREFFILE;
 static void _gref_consecutive_nodes(Mesh *M, Tree_T *ConsecutiveNTree,
 				    Tree_T *ConsecutiveETree)
 {
-  Simplex *sx;
+  SimplexBase *sx;
   Quadrangle *qx;
   Surface *s;
   int nbnod, nbedges, nbdof;
@@ -974,17 +990,21 @@ static void _gref_consecutive_nodes(Mesh *M, Tree_T *ConsecutiveNTree,
   List_T *ListSurfaces = Tree2List(M->Surfaces);
   for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    List_T *Triangles = Tree2List(s->Simplexes);
-    for(int j = 0; j < List_Nbr(Triangles); j++) {
-      List_Read(Triangles, j, &sx);
-      for(int k = 0; k < 3; k++) {
-        if(sx->V[k]->Frozen >= 0) {
-          sx->V[k]->Frozen = --newnum;
-          Tree_Insert(ConsecutiveNTree, &(sx->V[k]));
-        }
+    // triangles
+    for(int simtype = 0; simtype < 2; simtype++){
+      List_T *Triangles = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Triangles); j++) {
+	List_Read(Triangles, j, &sx);
+	for(int k = 0; k < 3; k++) {
+	  if(sx->V[k]->Frozen >= 0) {
+	    sx->V[k]->Frozen = --newnum;
+	    Tree_Insert(ConsecutiveNTree, &(sx->V[k]));
+	  }
+	}
       }
+      List_Delete(Triangles);
     }
-    List_Delete(Triangles);
+    // quads
     List_T *Quadrangles = Tree2List(s->Quadrangles);
     for(int j = 0; j < List_Nbr(Quadrangles); j++) {
       List_Read(Quadrangles, j, &qx);
@@ -1002,17 +1022,21 @@ static void _gref_consecutive_nodes(Mesh *M, Tree_T *ConsecutiveNTree,
 
   for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    List_T *Triangles = Tree2List(s->Simplexes);
-    for(int j = 0; j < List_Nbr(Triangles); j++) {
-      List_Read(Triangles, j, &sx);
-      for(int k = 0; k < 3; k++) {
-	if(sx->VSUP[k]->Frozen >= 0) {
-	  sx->VSUP[k]->Frozen = --newnum;
-	  Tree_Insert(ConsecutiveETree, &(sx->VSUP[k]));
+    // triangles
+    for(int simtype = 0; simtype < 2; simtype++){
+      List_T *Triangles = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Triangles); j++) {
+	List_Read(Triangles, j, &sx);
+	for(int k = 0; k < 3; k++) {
+	  if(sx->VSUP[k]->Frozen >= 0) {
+	    sx->VSUP[k]->Frozen = --newnum;
+	    Tree_Insert(ConsecutiveETree, &(sx->VSUP[k]));
+	  }
 	}
       }
+      List_Delete(Triangles);
     }
-    List_Delete(Triangles);
+    // quads
     List_T *Quadrangles = Tree2List(s->Quadrangles);
     for(int j = 0; j < List_Nbr(Quadrangles); j++) {
       List_Read(Quadrangles, j, &qx);
@@ -1035,22 +1059,26 @@ static void _gref_consecutive_nodes(Mesh *M, Tree_T *ConsecutiveNTree,
 
 static void _gref_end_consecutive_nodes(Mesh *M)
 {
-  Simplex *sx;
+  SimplexBase *sx;
   Quadrangle *qx;
   Surface *s;
 
   List_T *ListSurfaces = Tree2List(M->Surfaces);
   for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    List_T *Triangles = Tree2List(s->Simplexes);
-    for(int j = 0; j < List_Nbr(Triangles); j++) {
-      List_Read(Triangles, j, &sx);
-      for(int k = 0; k < 3; k++)
-        sx->V[k]->Frozen = 0;
-      for(int k = 0; k < 3; k++)
-	sx->VSUP[k]->Frozen = 0;
+    // triangles
+    for(int simtype = 0; simtype < 2; simtype++){
+      List_T *Triangles = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Triangles); j++) {
+	List_Read(Triangles, j, &sx);
+	for(int k = 0; k < 3; k++)
+	  sx->V[k]->Frozen = 0;
+	for(int k = 0; k < 3; k++)
+	  sx->VSUP[k]->Frozen = 0;
+      }
+      List_Delete(Triangles);
     }
-    List_Delete(Triangles);
+    // quads
     List_T *Quadrangles = Tree2List(s->Quadrangles);
     for(int j = 0; j < List_Nbr(Quadrangles); j++) {
       List_Read(Quadrangles, j, &qx);
@@ -1087,7 +1115,7 @@ static int _gref_process_nodes(Mesh *M, Tree_T *ConsecutiveNTree,
   nbtriqua = 0;
   for(i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    nbtriqua += Tree_Nbr(s->Simplexes);
+    nbtriqua += Tree_Nbr(s->Simplexes) + Tree_Nbr(s->SimplexesBase);
     nbtriqua += Tree_Nbr(s->Quadrangles);
   }
   List_Delete(ListSurfaces);
@@ -1181,7 +1209,7 @@ static void _gref_process_boundary_conditions(Mesh *M, Tree_T *TRN, Tree_T *TRE)
 
 static void _gref_process_elements(Mesh *M, int nn)
 {
-  Simplex *sx;
+  SimplexBase *sx;
   Quadrangle *qx;
   Surface *s;
 
@@ -1189,13 +1217,17 @@ static void _gref_process_elements(Mesh *M, int nn)
 
   for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    List_T *Triangles = Tree2List(s->Simplexes);
-    for(int j = 0; j < List_Nbr(Triangles); j++) {
-      List_Read(Triangles, j, &sx);
-      fprintf(GREFFILE, "%d %d %d\n", -sx->V[0]->Frozen,
-	      -sx->V[1]->Frozen, -sx->V[2]->Frozen);
+    // triangles
+    for(int simtype = 0; simtype < 2; simtype++){
+      List_T *Triangles = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Triangles); j++) {
+	List_Read(Triangles, j, &sx);
+	fprintf(GREFFILE, "%d %d %d\n", -sx->V[0]->Frozen,
+		-sx->V[1]->Frozen, -sx->V[2]->Frozen);
+      }
+      List_Delete(Triangles);
     }
-    List_Delete(Triangles);
+    // quads
     List_T *Quadrangles = Tree2List(s->Quadrangles);
     for(int j = 0; j < List_Nbr(Quadrangles); j++) {
       List_Read(Quadrangles, j, &qx);
@@ -1208,23 +1240,27 @@ static void _gref_process_elements(Mesh *M, int nn)
   // Degres de Liberte
   for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    List_T *Triangles = Tree2List(s->Simplexes);
-    for(int j = 0; j < List_Nbr(Triangles); j++) {
-      List_Read(Triangles, j, &sx);
-      fprintf(GREFFILE, "%d %d %d %d %d %d %d %d %d %d %d %d\n",
-	      -2 * sx->V[0]->Frozen - 1,
-	      -2 * sx->V[0]->Frozen,
-	      -2 * sx->VSUP[0]->Frozen - 1,
-	      -2 * sx->VSUP[0]->Frozen,
-	      -2 * sx->V[1]->Frozen - 1,
-	      -2 * sx->V[1]->Frozen,
-	      -2 * sx->VSUP[1]->Frozen - 1,
-	      -2 * sx->VSUP[1]->Frozen,
-	      -2 * sx->V[2]->Frozen - 1,
-	      -2 * sx->V[2]->Frozen,
-	      -2 * sx->VSUP[2]->Frozen - 1, -2 * sx->VSUP[2]->Frozen);
+    // triangles
+    for(int simtype = 0; simtype < 2; simtype++){
+      List_T *Triangles = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Triangles); j++) {
+	List_Read(Triangles, j, &sx);
+	fprintf(GREFFILE, "%d %d %d %d %d %d %d %d %d %d %d %d\n",
+		-2 * sx->V[0]->Frozen - 1,
+		-2 * sx->V[0]->Frozen,
+		-2 * sx->VSUP[0]->Frozen - 1,
+		-2 * sx->VSUP[0]->Frozen,
+		-2 * sx->V[1]->Frozen - 1,
+		-2 * sx->V[1]->Frozen,
+		-2 * sx->VSUP[1]->Frozen - 1,
+		-2 * sx->VSUP[1]->Frozen,
+		-2 * sx->V[2]->Frozen - 1,
+		-2 * sx->V[2]->Frozen,
+		-2 * sx->VSUP[2]->Frozen - 1, -2 * sx->VSUP[2]->Frozen);
+      }
+      List_Delete(Triangles);
     }
-    List_Delete(Triangles);
+    // quads
     List_T *Quadrangles = Tree2List(s->Quadrangles);
     for(int j = 0; j < List_Nbr(Quadrangles); j++) {
       List_Read(Quadrangles, j, &qx);
@@ -1295,7 +1331,7 @@ static void _wrl_process_nodes(Mesh *M)
 
 static void _wrl_print_line(void *a, void *b)
 {
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
   for(int i = 0; i < 2; i++){
     if(s->V[i]){
       int j = List_ISearch(wrlnodes, &s->V[i]->Num, fcmp_int);
@@ -1310,7 +1346,7 @@ static void _wrl_print_line(void *a, void *b)
 
 static void _wrl_print_triangle(void *a, void *b)
 {
-  Simplex *s = *(Simplex **) a;
+  SimplexBase *s = *(SimplexBase **) a;
   for(int i = 0; i < 3; i++){
     if(s->V[i]){
       int j = List_ISearch(wrlnodes, &s->V[i]->Num, fcmp_int);
@@ -1346,6 +1382,7 @@ static void _wrl_print_all_curves(void *a, void *b)
   fprintf(WRLFILE, "DEF Curve%d IndexedLineSet {\n", c->Num);
   fprintf(WRLFILE, "  coordIndex [\n");
   Tree_Action(c->Simplexes, _wrl_print_line);
+  Tree_Action(c->SimplexesBase, _wrl_print_line);
   fprintf(WRLFILE, "  ]\n");
   fprintf(WRLFILE, "}\n");
 }
@@ -1356,6 +1393,7 @@ static void _wrl_print_all_surfaces(void *a, void *b)
   fprintf(WRLFILE, "DEF Surface%d IndexedFaceSet {\n", s->Num);
   fprintf(WRLFILE, "  coordIndex [\n");
   Tree_Action(s->Simplexes, _wrl_print_triangle);
+  Tree_Action(s->SimplexesBase, _wrl_print_triangle);
   Tree_Action(s->Quadrangles, _wrl_print_quadrangle);
   fprintf(WRLFILE, "  ]\n");
   fprintf(WRLFILE, "}\n");
@@ -1378,6 +1416,71 @@ void Print_Mesh_WRL(Mesh *M, FILE *fp)
   _wrl_process_elements(M);
 }
 
+// Write surface mesh in STL format
+
+static FILE *STLFILE;
+
+static void _stl_print_triangle(void *a, void *b)
+{
+  SimplexBase *s = *(SimplexBase **) a;
+  
+  if(!s->V[2]) return;
+  
+  double n[3];
+  normal3points(s->V[0]->Pos.X, s->V[0]->Pos.Y, s->V[0]->Pos.Z, 
+		s->V[1]->Pos.X, s->V[1]->Pos.Y, s->V[1]->Pos.Z, 
+		s->V[2]->Pos.X, s->V[2]->Pos.Y, s->V[2]->Pos.Z, n);
+
+  fprintf(STLFILE, "facet normal %g %g %g\n", n[0], n[1], n[2]);
+  fprintf(STLFILE, "  outer loop\n");
+  fprintf(STLFILE, "    vertex %g %g %g\n", s->V[0]->Pos.X, s->V[0]->Pos.Y, s->V[0]->Pos.Z);
+  fprintf(STLFILE, "    vertex %g %g %g\n", s->V[1]->Pos.X, s->V[1]->Pos.Y, s->V[1]->Pos.Z);
+  fprintf(STLFILE, "    vertex %g %g %g\n", s->V[2]->Pos.X, s->V[2]->Pos.Y, s->V[2]->Pos.Z);
+  fprintf(STLFILE, "  endloop\n");
+  fprintf(STLFILE, "endfacet\n");
+}
+
+static void _stl_print_quadrangle(void *a, void *b)
+{
+  Quadrangle *q = *(Quadrangle **) a;
+  
+  double n[3];
+  normal3points(q->V[0]->Pos.X, q->V[0]->Pos.Y, q->V[0]->Pos.Z, 
+		q->V[1]->Pos.X, q->V[1]->Pos.Y, q->V[1]->Pos.Z, 
+		q->V[2]->Pos.X, q->V[2]->Pos.Y, q->V[2]->Pos.Z, n);
+
+  fprintf(STLFILE, "facet normal %g %g %g\n", n[0], n[1], n[2]);
+  fprintf(STLFILE, "  outer loop\n");
+  fprintf(STLFILE, "    vertex %g %g %g\n", q->V[0]->Pos.X, q->V[0]->Pos.Y, q->V[0]->Pos.Z);
+  fprintf(STLFILE, "    vertex %g %g %g\n", q->V[1]->Pos.X, q->V[1]->Pos.Y, q->V[1]->Pos.Z);
+  fprintf(STLFILE, "    vertex %g %g %g\n", q->V[2]->Pos.X, q->V[2]->Pos.Y, q->V[2]->Pos.Z);
+  fprintf(STLFILE, "  endloop\n");
+  fprintf(STLFILE, "endfacet\n");
+  fprintf(STLFILE, "facet normal %g %g %g\n", n[0], n[1], n[2]);
+  fprintf(STLFILE, "  outer loop\n");
+  fprintf(STLFILE, "    vertex %g %g %g\n", q->V[0]->Pos.X, q->V[0]->Pos.Y, q->V[0]->Pos.Z);
+  fprintf(STLFILE, "    vertex %g %g %g\n", q->V[2]->Pos.X, q->V[2]->Pos.Y, q->V[2]->Pos.Z);
+  fprintf(STLFILE, "    vertex %g %g %g\n", q->V[3]->Pos.X, q->V[3]->Pos.Y, q->V[3]->Pos.Z);
+  fprintf(STLFILE, "  endloop\n");
+  fprintf(STLFILE, "endfacet\n");
+}
+
+static void _stl_print_all_surfaces(void *a, void *b)
+{
+  Surface *s = *(Surface **) a;
+  Tree_Action(s->Simplexes, _stl_print_triangle);
+  Tree_Action(s->SimplexesBase, _stl_print_triangle);
+  Tree_Action(s->Quadrangles, _stl_print_quadrangle);
+}
+
+void Print_Mesh_STL(Mesh *M, FILE *fp)
+{
+  STLFILE = fp;
+  fprintf(STLFILE, "solid Created by Gmsh\n");
+  Tree_Action(M->Surfaces, _stl_print_all_surfaces);
+  fprintf(STLFILE, "endsolid Created by Gmsh\n");
+}
+
 // Write mesh in DMG format
 
 static int _dmg_is_topologic(Vertex *v, List_T *curves)
@@ -1501,6 +1604,216 @@ void Print_Mesh_DMG(Mesh *m, FILE *fp)
   // write the volumes (2 b continued...)
 }
 
+// Write mesh in Plot3D format, ASCII structured, multi-zone
+// FIXME: still need to implement this for extruded grids
+
+static FILE *P3DFILE;
+
+int _p3d_cmp_entities(const void *a, const void *b)
+{
+  Element **e1 = (Element **) a;
+  Element **e2 = (Element **) b;
+
+  return (*e1)->iEnt - (*e2)->iEnt;
+}
+
+
+int _p3d_cmp_surf_num(const void *a, const void *b)
+{
+  Surface **e1 = (Surface **) a;
+  Surface **e2 = (Surface **) b;
+
+  return (*e1)->Num - (*e2)->Num;
+}
+
+int _p3d_cmp_vol_num(const void *a, const void *b)
+{
+  Volume **e1 = (Volume **) a;
+  Volume **e2 = (Volume **) b;
+
+  return (*e1)->Num - (*e2)->Num;
+}
+
+void _p3d_print_quads(List_T *ListQuads, int Nu, int Nv)
+{
+  int i = 0, j = 0, c = 1, curQuad = 0;
+  double coord;
+  Quadrangle *pQ;
+
+  for (c = 0; c < 3; c++) {
+    for (j = 0; j < (Nu - 1); j++) {
+      for (i = 0; i < (Nv - 1); i++) {
+        curQuad = i + (j * (Nv - 1));
+        List_Read(ListQuads, curQuad, &pQ);
+        coord = (c==0) ? pQ->V[0]->Pos.X : ((c==1) ? pQ->V[0]->Pos.Y : pQ->V[0]->Pos.Z);
+        fprintf(P3DFILE, "%g ", coord * CTX.mesh.scaling_factor);
+      }
+      coord = (c==0) ? pQ->V[3]->Pos.X : ((c==1) ? pQ->V[3]->Pos.Y : pQ->V[3]->Pos.Z);
+      fprintf(P3DFILE, "%g\n", coord * CTX.mesh.scaling_factor);
+    } j--;
+    for (i = 0; i < (Nv - 1); i++) {
+      curQuad = i + (j * (Nv - 1));
+      List_Read(ListQuads, curQuad, &pQ);
+      coord = (c==0) ? pQ->V[1]->Pos.X : ((c==1) ? pQ->V[1]->Pos.Y : pQ->V[1]->Pos.Z);
+      fprintf(P3DFILE, "%g ", coord * CTX.mesh.scaling_factor);
+    }
+    coord = (c==0) ? pQ->V[2]->Pos.X : ((c==1) ? pQ->V[2]->Pos.Y : pQ->V[2]->Pos.Z);
+    fprintf(P3DFILE, "%g\n", coord * CTX.mesh.scaling_factor);
+  }
+}
+
+void _p3d_print_hex(List_T *ListHex, int Nu, int Nv, int Nw)
+{
+  int i = 0, j = 0, k = 0, c = 0, curHex = 0;
+  double coord;
+  Hexahedron *pH;
+
+  
+  for (c = 0; c < 3; c++) {
+    for (k = 0; k < (Nu - 1); k++) {
+      for (j = 0; j < (Nv - 1); j++) {
+        for (i = 0; i < (Nw - 1); i++) {
+          curHex = i + (j * (Nw - 1)) + (k * (Nw - 1) * (Nv - 1));
+          List_Read(ListHex, curHex, &pH);
+          coord = (c==0) ? pH->V[0]->Pos.X : ((c==1) ? pH->V[0]->Pos.Y : pH->V[0]->Pos.Z);
+          fprintf(P3DFILE, "%g ", coord * CTX.mesh.scaling_factor);
+        }
+       coord = (c==0) ? pH->V[4]->Pos.X : ((c==1) ? pH->V[4]->Pos.Y : pH->V[4]->Pos.Z);
+       fprintf(P3DFILE, "%g\n", coord * CTX.mesh.scaling_factor);
+      } j--;
+      for (i = 0; i < (Nw - 1); i++) {
+        curHex = i + (j * (Nw - 1)) + (k * (Nw - 1) * (Nv - 1));
+        List_Read(ListHex, curHex, &pH);
+        coord = (c==0) ? pH->V[3]->Pos.X : ((c==1) ? pH->V[3]->Pos.Y : pH->V[3]->Pos.Z);
+        fprintf(P3DFILE, "%g ", coord * CTX.mesh.scaling_factor);
+      }
+      coord = (c==0) ? pH->V[7]->Pos.X : ((c==1) ? pH->V[7]->Pos.Y : pH->V[7]->Pos.Z);
+      fprintf(P3DFILE, "%g\n", coord * CTX.mesh.scaling_factor);
+    } k--;
+    for (j = 0; j < (Nv - 1); j++) {
+      for (i = 0; i < (Nw - 1); i++) {
+        curHex = i + (j * (Nw - 1)) + (k * (Nw - 1) * (Nv - 1));
+        List_Read(ListHex, curHex, &pH);
+        coord = (c==0) ? pH->V[1]->Pos.X : ((c==1) ? pH->V[1]->Pos.Y : pH->V[1]->Pos.Z);
+        fprintf(P3DFILE, "%g ", coord * CTX.mesh.scaling_factor);
+      }
+      coord = (c==0) ? pH->V[5]->Pos.X : ((c==1) ? pH->V[5]->Pos.Y : pH->V[5]->Pos.Z);
+      fprintf(P3DFILE, "%g\n", coord * CTX.mesh.scaling_factor);
+    } j--;
+    for (i = 0; i < (Nw - 1); i++) {
+      curHex = i + (j * (Nw - 1)) + (k * (Nw - 1) * (Nv - 1));
+      List_Read(ListHex, curHex, &pH);
+      coord = (c==0) ? pH->V[2]->Pos.X : ((c==1) ? pH->V[2]->Pos.Y : pH->V[2]->Pos.Z);
+      fprintf(P3DFILE, "%g ", coord * CTX.mesh.scaling_factor);
+    }
+    coord = (c==0) ? pH->V[6]->Pos.X : ((c==1) ? pH->V[6]->Pos.Y : pH->V[6]->Pos.Z);
+    fprintf(P3DFILE, "%g\n", coord * CTX.mesh.scaling_factor);
+  }
+}
+
+void _p3d_print_all_elements(Mesh *M)
+{
+  int i, Nv, Nu, ElemCnt = 0;
+  Volume *pV;
+  Surface *pS;
+  List_T *ListQuads;
+  List_T *ListHex;
+
+  List_T *ListSurfaces = Tree2List(M->Surfaces);
+  List_T *ListVolumes = Tree2List(M->Volumes);
+
+  List_T *ListStructSurf = List_Create(1,1,sizeof(Surface *));
+  List_T *ListStructVol = List_Create(1,1,sizeof(Volume *));
+
+  List_Sort(ListSurfaces, _p3d_cmp_surf_num);
+  List_Sort(ListVolumes, _p3d_cmp_vol_num);
+
+  for(i = 0; i < List_Nbr(ListSurfaces); i++) {
+    List_Read(ListSurfaces, i, &pS);
+    if (pS->Nu &&
+        pS->Nv &&
+        (Tree_Nbr(pS->Quadrangles) == (pS->Nu - 1) * (pS->Nv - 1))){
+        ElemCnt++;
+        List_Add(ListStructSurf, &pS);
+    }
+  }
+
+  for(i = 0; i < List_Nbr(ListVolumes); i++) {
+    List_Read(ListVolumes, i, &pV);
+    List_Read(pV->Surfaces, 0, &pS);
+    Nu = pS->Nu;
+    Nv = pS->Nv;
+    List_Read(pV->Surfaces, 1, &pS);
+    if (Nu &&
+        Nv &&
+        pS->Nv &&
+        (Tree_Nbr(pV->Hexahedra) == (Nu - 1) * (Nv - 1) * (pS->Nv - 1))){
+        ElemCnt++;
+        List_Add(ListStructVol, &pV);
+    }
+  }
+
+  fprintf(P3DFILE, "%d\n", ElemCnt);
+
+  for(i = 0; i < List_Nbr(ListStructSurf); i++) {
+    List_Read(ListStructSurf, i, &pS);
+    fprintf(P3DFILE, "%d %d 1\n", pS->Nv, pS->Nu);
+  }
+
+  for(i = 0; i < List_Nbr(ListStructVol); i++) {
+    List_Read(ListStructVol, i, &pV);
+    List_Read(pV->Surfaces, 0, &pS);
+    Nu = pS->Nu;
+    Nv = pS->Nv;
+    List_Read(pV->Surfaces, 1, &pS);
+    fprintf(P3DFILE, "%d %d %d\n", pS->Nv, Nv, Nu);
+  }
+
+  for(i = 0; i < List_Nbr(ListStructSurf); i++) {
+    List_Read(ListStructSurf, i, &pS);
+    ListQuads = Tree2List(pS->Quadrangles);
+    List_Sort(ListQuads, _p3d_cmp_entities);
+    _p3d_print_quads(ListQuads, pS->Nu, pS->Nv);
+    List_Delete(ListQuads);
+  }
+
+  for(i = 0; i < List_Nbr(ListStructVol); i++) {
+    List_Read(ListStructVol, i, &pV);
+    List_Read(pV->Surfaces, 0, &pS);
+    Nu = pS->Nu;
+    Nv = pS->Nv;
+    List_Read(pV->Surfaces, 1, &pS);
+    ListHex = Tree2List(pV->Hexahedra);
+    List_Sort(ListHex, _p3d_cmp_entities);
+    _p3d_print_hex(ListHex, Nu, Nv, pS->Nv);
+    List_Delete(ListHex);
+  }
+
+  List_Delete(ListSurfaces);
+  List_Delete(ListVolumes);
+  List_Delete(ListStructSurf);
+  List_Delete(ListStructVol);
+}
+
+void _p3d_print_elements(Mesh *M)
+{
+  // FIXME: to do
+}
+
+void Print_Mesh_P3D(Mesh *M, FILE *fp)
+{
+  P3DFILE = fp;
+
+  if(!List_Nbr(M->PhysicalGroups) || CTX.mesh.save_all) {
+    Msg(INFO, "Saving all elements (discarding physical groups)");
+    _p3d_print_all_elements(M);
+  }
+  else{
+    Msg(WARNING, "No Plot3d format of physicals yet, saving all elements!");
+    //_p3d_print_elements(M);
+    _p3d_print_all_elements(M);
+  }
+}
 
 // Public Print_Mesh routine
 
@@ -1523,6 +1836,8 @@ void Print_Mesh(Mesh *M, char *c, int Type)
   case FORMAT_UNV:  strcpy(ext, ".unv"); break;
   case FORMAT_GREF: strcpy(ext, ".Gref"); break;
   case FORMAT_DMG:  strcpy(ext, ".dmg"); break;
+  case FORMAT_STL:  strcpy(ext, ".stl"); break;
+  case FORMAT_P3D:  strcpy(ext, ".p3d"); break;
   default:
     Msg(GERROR, "Unknown mesh file format %d", Type);
     return;
@@ -1545,6 +1860,8 @@ void Print_Mesh(Mesh *M, char *c, int Type)
   case FORMAT_UNV:  Print_Mesh_UNV(M, fp); break;
   case FORMAT_GREF: Print_Mesh_GREF(M, fp); break;
   case FORMAT_DMG:  Print_Mesh_DMG(M, fp); break;
+  case FORMAT_STL:  Print_Mesh_STL(M, fp); break;
+  case FORMAT_P3D:  Print_Mesh_P3D(M, fp); break;
   default:
     break;
   }
