@@ -1,6 +1,6 @@
-// $Id: Read_Mesh.cpp,v 1.78 2004/10/08 02:41:20 geuzaine Exp $
+// $Id: Read_Mesh.cpp,v 1.86 2005/03/15 15:36:06 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -134,7 +134,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
   int NbTags, Tag;
   double x, y, z, lc1, lc2;
   Vertex *vert, verts[NB_NOD_MAX_ELM], *vertsp[NB_NOD_MAX_ELM], **vertspp;
-  Simplex *simp;
+  SimplexBase *simp;
   Quadrangle *quad;
   Hexahedron *hex;
   Prism *pri;
@@ -207,6 +207,9 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 
       if(CTX.mesh.check_duplicates)
         Duplicates = Tree_Create(sizeof(Vertex *), comparePosition);
+
+      int NN = (Nbr_Nodes > 100000) ? Nbr_Nodes/50 : Nbr_Nodes/10;
+
       for(i_Node = 0; i_Node < Nbr_Nodes; i_Node++) {
         fscanf(fp, "%d %lf %lf %lf", &Num, &x, &y, &z);
         vert = Create_Vertex(Num, x, y, z, 1.0, 0.0);
@@ -221,7 +224,11 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
           else
             Tree_Add(Duplicates, &vert);
         }
+	if(NN && (i_Node % NN == NN - 1))
+	  Msg(PROGRESS, "Read %d nodes", i_Node + 1);
       }
+      Msg(PROGRESS, "");
+
       if(CTX.mesh.check_duplicates)
         Tree_Delete(Duplicates);
     }
@@ -237,6 +244,8 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
       if(CTX.mesh.check_duplicates)
         Duplicates = Tree_Create(sizeof(Vertex *), comparePosition);
 
+      int NN = (Nbr_Elements > 100000) ? Nbr_Elements/50 : Nbr_Elements/10;
+
       for(i_Element = 0; i_Element < Nbr_Elements; i_Element++) {
 	
 	if(version <= 1.0){
@@ -306,7 +315,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case LGN2:
 	  c = addElementaryCurve(M, abs(Elementary));
 	  addPhysicalGroup(M, MSH_PHYSICAL_LINE, Physical, abs(Elementary));
-          simp = Create_Simplex(vertsp[0], vertsp[1], NULL, NULL);
+          simp = Create_SimplexBase(vertsp[0], vertsp[1], NULL, NULL);
           simp->Num = Num;
           simp->iEnt = Elementary;
           simp->iPart = Add_MeshPartition(Partition, M);
@@ -315,16 +324,20 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    simp->VSUP[0] = vertsp[2];
 	    simp->VSUP[0]->Degree = 2;
 	  }
-          if(!Tree_Insert(c->Simplexes, &simp)){
+          if(!Tree_Insert(c->SimplexesBase, &simp)){
 	    Msg(GERROR, "Line element %d already exists", simp->Num);
-	    Free_Simplex(&simp, 0);
+	    Free_SimplexBase(&simp, 0);
+	  }
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      List_Insert(c->Vertices, &vertsp[i], fcmp_int);
 	  }
           break;
         case TRI1:
         case TRI2:
 	  s = addElementarySurface(M, Elementary);
 	  addPhysicalGroup(M, MSH_PHYSICAL_SURFACE, Physical, Elementary);
-          simp = Create_Simplex(vertsp[0], vertsp[1], vertsp[2], NULL);
+          simp = Create_SimplexBase(vertsp[0], vertsp[1], vertsp[2], NULL);
           simp->Num = Num;
           simp->iEnt = Elementary;
           simp->iPart = Add_MeshPartition(Partition, M);
@@ -335,9 +348,13 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	      simp->VSUP[i]->Degree = 2;
 	    }
 	  }
-          if(!Tree_Insert(s->Simplexes, &simp)){
+          if(!Tree_Insert(s->SimplexesBase, &simp)){
 	    Msg(GERROR, "Triangle %d already exists", simp->Num);
-	    Free_Simplex(&simp, 0);
+	    Free_SimplexBase(&simp, 0);
+	  }
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      Tree_Insert(s->Vertices, &vertsp[i]);
 	  }
           break;
         case QUA1:
@@ -356,15 +373,19 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    }
 	  }
           if(!Tree_Insert(s->Quadrangles, &quad)){
-	    Msg(GERROR, "Quadrangle %d already exists", simp->Num);
-	    Free_Simplex(&simp, 0);
+	    Msg(GERROR, "Quadrangle %d already exists", quad->Num);
+	    Free_SimplexBase(&quad, 0);
+	  }
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      Tree_Insert(s->Vertices, &vertsp[i]);
 	  }
           break;
         case TET1:
         case TET2:
 	  v = addElementaryVolume(M, Elementary);
 	  addPhysicalGroup(M, MSH_PHYSICAL_VOLUME, Physical, Elementary);
-          simp = Create_Simplex(vertsp[0], vertsp[1], vertsp[2], vertsp[3]);
+          simp = Create_SimplexBase(vertsp[0], vertsp[1], vertsp[2], vertsp[3]);
           simp->Num = Num;
           simp->iEnt = Elementary;
           simp->iPart = Add_MeshPartition(Partition, M);
@@ -375,10 +396,16 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	      simp->VSUP[i]->Degree = 2;
 	    }
 	  }
-          if(!Tree_Insert(v->Simplexes, &simp)){
+          if(!Tree_Insert(v->SimplexesBase, &simp)){
 	    Msg(GERROR, "Tetrahedron %d already exists", simp->Num);
-	    Free_Simplex(&simp, 0);
+	    Free_SimplexBase(&simp, 0);
 	  }
+#if 0 // removed to speed things up (not used at the moment anyway)
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      Tree_Insert(v->Vertices, &vertsp[i]);
+	  }
+#endif
           break;
         case HEX1:
         case HEX2:
@@ -400,6 +427,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    Msg(GERROR, "Hexahedron %d already exists", hex->Num);
 	    Free_Hexahedron(&hex, 0);
 	  }
+#if 0 // removed to speed things up (not used at the moment anyway)
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      Tree_Insert(v->Vertices, &vertsp[i]);
+	  }
+#endif
           break;
         case PRI1:
         case PRI2:
@@ -421,6 +454,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    Msg(GERROR, "Prism %d already exists", pri->Num);
 	    Free_Prism(&pri, 0);
 	  }
+#if 0 // removed to speed things up (not used at the moment anyway)
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      Tree_Insert(v->Vertices, &vertsp[i]);
+	  }
+#endif
           break;
         case PYR1:
         case PYR2:
@@ -439,9 +478,15 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    }
 	  }
           if(!Tree_Insert(v->Pyramids, &pyr)){
-	    Msg(GERROR, "Pyramid %d already exists", pri->Num);
+	    Msg(GERROR, "Pyramid %d already exists", pyr->Num);
 	    Free_Pyramid(&pyr, 0);
 	  }
+#if 0 // removed to speed things up (not used at the moment anyway)
+	  else{
+	    for(i = 0; i < Nbr_Nodes; i++)
+	      Tree_Insert(v->Vertices, &vertsp[i]);
+	  }
+#endif
           break;
         case PNT:
 	  addPhysicalGroup(M, MSH_PHYSICAL_POINT, Physical, Elementary);
@@ -458,7 +503,11 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	  Msg(GERROR, "Unknown type for element %d", Num); 
           break;
         }
+
+	if(NN && (i_Element % NN == NN - 1))
+	  Msg(PROGRESS, "Read %d elements", i_Element + 1);
       }
+      Msg(PROGRESS, "");
 
       if(CTX.mesh.check_duplicates) {
         Tree_Action(Duplicates, Free_Vertex);
