@@ -1,6 +1,6 @@
-// $Id: SecondOrder.cpp,v 1.28 2004/10/28 18:50:27 geuzaine Exp $
+// $Id: SecondOrder.cpp,v 1.31 2005/02/20 16:41:24 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -60,38 +60,43 @@ Vertex *onCurve(Vertex * v1, Vertex * v2)
   int ok1 = 1, ok2 = 1;
   double u1 = 0., u2 = 0.;
 
-  if(List_Nbr(v1->ListCurves) == 1){
-    u1 = v1->u;
-  }
-  else if(v1->Num == THEC->beg->Num){
-    u1 = THEC->ubeg;
-  }
-  else if(v1->Num == THEC->end->Num){
-    u1 = THEC->uend;
-  }
-  else{
-    ok1 = 0;
-  }
-
-  if(List_Nbr(v2->ListCurves) == 1){
-    u2 = v2->u;
-  }
-  else if(v2->Num == THEC->beg->Num){
-    u2 = THEC->ubeg;
-  }
-  else if(v2->Num == THEC->end->Num){
-    u2 = THEC->uend;
-  }
-  else{
-    ok2 = 0;
-  }
-
-  // Ugly fix for closed curves
-  if((THEC->beg->Num == THEC->end->Num) &&
-     (u1 == THEC->ubeg || u1 == THEC->uend ||
-      u2 == THEC->ubeg || u2 == THEC->uend)){
+  if(!THEC->beg || !THEC->end){
     ok1 = ok2 = 0;
   }
+  else{
+    if(List_Nbr(v1->ListCurves) == 1){
+      u1 = v1->u;
+    }
+    else if(v1->Num == THEC->beg->Num){
+      u1 = THEC->ubeg;
+    }
+    else if(v1->Num == THEC->end->Num){
+      u1 = THEC->uend;
+    }
+    else{
+      ok1 = 0;
+    }
+    
+    if(List_Nbr(v2->ListCurves) == 1){
+      u2 = v2->u;
+    }
+    else if(v2->Num == THEC->beg->Num){
+      u2 = THEC->ubeg;
+    }
+    else if(v2->Num == THEC->end->Num){
+      u2 = THEC->uend;
+    }
+    else{
+      ok2 = 0;
+    }
+
+    // Ugly fix for closed curves
+    if((THEC->beg->Num == THEC->end->Num) &&
+       (u1 == THEC->ubeg || u1 == THEC->uend ||
+	u2 == THEC->ubeg || u2 == THEC->uend)){
+      ok1 = ok2 = 0;
+    }
+  }
 
   if(ok1 && ok2){
     v = InterpolateCurve(THEC, 0.5 * (u1 + u2), 0);
@@ -119,7 +124,7 @@ Vertex *onSurface(Vertex * v1, Vertex * v2)
 
   if(!THES)
     return NULL;
-  if(THES->Typ == MSH_SURF_PLAN)
+  if(THES->Typ == MSH_SURF_PLAN || THES->Typ == MSH_SURF_DISCRETE)
     return NULL;
 
   XYZtoUV(THES, v1->Pos.X, v1->Pos.Y, v1->Pos.Z, &U1, &V1, 1.0);
@@ -141,7 +146,7 @@ Vertex *onSurface(Vertex * v1, Vertex * v2, Vertex * v3, Vertex * v4)
 
   if(!THES)
     return NULL;
-  if(THES->Typ == MSH_SURF_PLAN)
+  if(THES->Typ == MSH_SURF_PLAN || THES->Typ == MSH_SURF_DISCRETE)
     return NULL;
 
   XYZtoUV(THES, v1->Pos.X, v1->Pos.Y, v1->Pos.Z, &U1, &V1, 1.0);
