@@ -1,7 +1,7 @@
 #ifndef _NUMERIC_H_
 #define _NUMERIC_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -61,13 +61,24 @@ double myasin(double a);
 double myacos(double a);
 void prodve(double a[3], double b[3], double c[3]);
 void prosca(double a[3], double b[3], double *c);
+void matvec(double mat[3][3], double vec[3], double res[3]);
+double norm3(double a[3]);
 double norme(double a[3]);
+void normal3points(double x0, double y0, double z0,
+		   double x1, double y1, double z1,
+		   double x2, double y2, double z2,
+		   double n[3]);
 int sys2x2(double mat[2][2], double b[2], double res[2]);
 int sys3x3(double mat[3][3], double b[3], double res[3], double *det);
 int sys3x3_with_tol(double mat[3][3], double b[3], double res[3], double *det);
 double det3x3(double mat[3][3]);
-int inv3x3(double mat[3][3], double inv[3][3], double *det);
+double trace3x3(double mat[3][3]);
+double trace2 (double mat[3][3]);
+double inv3x3(double mat[3][3], double inv[3][3]);
 double angle_02pi(double A3);
+void eigenvalue(double mat[3][3], double re[3]);
+void FindCubicRoots(const double coeff[4], double re[3], double im[3]);
+void eigsort(double d[3]);
 
 double InterpolateIso(double *X, double *Y, double *Z, 
 		      double *Val, double V, int I1, int I2, 
