@@ -1,6 +1,6 @@
-// $Id: OpenFile.cpp,v 1.65 2004/10/28 03:44:37 geuzaine Exp $
+// $Id: OpenFile.cpp,v 1.73 2005/03/13 17:58:38 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@
 #include "Draw.h"
 #include "GUI.h"
 extern GUI *WID;
+void UpdateViewsInGUI();
 #endif
 
 extern Mesh *THEM, M;
@@ -70,6 +71,19 @@ void FixWindowsPath(char *in, char *out){
 #endif
 }
 
+void SplitFileName(char *name, char *base, char *ext)
+{
+  strcpy(base, name);
+  strcpy(ext, "");
+  for(int i = strlen(name)-1; i >= 0; i--){
+    if(name[i] == '.'){
+      strcpy(ext, &name[i]);
+      base[i] = '\0';
+      break;
+    }
+  }
+}
+
 void SetBoundingBox(double xmin, double xmax,
 		    double ymin, double ymax, 
 		    double zmin, double zmax)
@@ -91,13 +105,78 @@ void SetBoundingBox(void)
     // else, if we have geometry points, use them
     CalculateMinMax(THEM->Points, NULL);
   }
+  else if(Tree_Nbr(THEM->Curves)) { 
+    // else, if we have (discrete) curves, use them
+    List_T *l = Tree2List(THEM->Curves);
+    int first = 1;
+    double bbox[6];
+    for(int i = 0; i < List_Nbr(l); i++){
+      Curve *c = *(Curve**)List_Pointer(l, i);
+      if(c->theSegmRep){
+	if(first){
+	  first = 0;
+	  for(int j = 0; j < 6; j++)
+	    bbox[j] = c->theSegmRep->bounding_box[j];
+	}
+	else{
+	  if(c->theSegmRep->bounding_box[0] < bbox[0])
+	    bbox[0] = c->theSegmRep->bounding_box[0];
+	  if(c->theSegmRep->bounding_box[1] > bbox[1])
+	    bbox[1] = c->theSegmRep->bounding_box[1];
+	  if(c->theSegmRep->bounding_box[2] < bbox[2])
+	    bbox[2] = c->theSegmRep->bounding_box[2];
+	  if(c->theSegmRep->bounding_box[3] > bbox[3])
+	    bbox[3] = c->theSegmRep->bounding_box[3];
+	  if(c->theSegmRep->bounding_box[4] < bbox[4])
+	    bbox[4] = c->theSegmRep->bounding_box[4];
+	  if(c->theSegmRep->bounding_box[5] > bbox[5])
+	    bbox[5] = c->theSegmRep->bounding_box[5];
+	}
+      }
+    }
+    CalculateMinMax(NULL, bbox);
+    List_Delete(l);
+  }
+  else if(Tree_Nbr(THEM->Surfaces)) { 
+    // else, if we have (discrete) surfaces, use them
+    List_T *l = Tree2List(THEM->Surfaces);
+    int first = 1;
+    double bbox[6];
+    for(int i = 0; i < List_Nbr(l); i++){
+      Surface *s = *(Surface**)List_Pointer(l, i);
+      if(s->thePolyRep){
+	if(first){
+	  first = 0;
+	  for(int j = 0; j < 6; j++)
+	    bbox[j] = s->thePolyRep->bounding_box[j];
+	}
+	else{
+	  if(s->thePolyRep->bounding_box[0] < bbox[0])
+	    bbox[0] = s->thePolyRep->bounding_box[0];
+	  if(s->thePolyRep->bounding_box[1] > bbox[1])
+	    bbox[1] = s->thePolyRep->bounding_box[1];
+	  if(s->thePolyRep->bounding_box[2] < bbox[2])
+	    bbox[2] = s->thePolyRep->bounding_box[2];
+	  if(s->thePolyRep->bounding_box[3] > bbox[3])
+	    bbox[3] = s->thePolyRep->bounding_box[3];
+	  if(s->thePolyRep->bounding_box[4] < bbox[4])
+	    bbox[4] = s->thePolyRep->bounding_box[4];
+	  if(s->thePolyRep->bounding_box[5] > bbox[5])
+	    bbox[5] = s->thePolyRep->bounding_box[5];
+	}
+      }
+    }
+    CalculateMinMax(NULL, bbox);
+    opt_geometry_surfaces(0, GMSH_SET | GMSH_GUI, 1);
+    List_Delete(l);
+  }
   else if(List_Nbr(CTX.post.list)) {
     // else, if we have views, use the max bb of all the views
     double bbox[6];
-    Post_View *v = (Post_View *)List_Pointer(CTX.post.list, 0);
+    Post_View *v = *(Post_View **)List_Pointer(CTX.post.list, 0);
     for(int i = 0; i < 6; i++) bbox[i] = v->BBox[i];
     for(int i = 1; i < List_Nbr(CTX.post.list); i++){
-      v = (Post_View *)List_Pointer(CTX.post.list, i);
+      v = *(Post_View **)List_Pointer(CTX.post.list, i);
       if(v->BBox[0] < bbox[0]) bbox[0] = v->BBox[0];
       if(v->BBox[1] > bbox[1]) bbox[1] = v->BBox[1];
       if(v->BBox[2] < bbox[2]) bbox[2] = v->BBox[2];
@@ -117,7 +196,7 @@ int ParseFile(char *f, int silent, int close, int warn_if_missing)
 {
   char yyname_old[256], tmp[256];
   FILE *yyin_old, *fp;
-  int yylineno_old, yyerrorstate_old, status;
+  int yylineno_old, yyerrorstate_old, numviews_old, status;
 
   if(!(fp = fopen(f, "r"))){
     if(warn_if_missing)
@@ -129,7 +208,8 @@ int ParseFile(char *f, int silent, int close, int warn_if_missing)
   yyin_old = yyin;
   yyerrorstate_old = yyerrorstate;
   yylineno_old = yylineno;
-  
+  numviews_old = List_Nbr(CTX.post.list);
+
   strncpy(yyname, f, 255);
   yyin = fp;
   yyerrorstate = 0;
@@ -171,6 +251,12 @@ int ParseFile(char *f, int silent, int close, int warn_if_missing)
   yyerrorstate = yyerrorstate_old;
   yylineno = yylineno_old;
 
+  if(List_Nbr(CTX.post.list) != numviews_old){
+#if defined(HAVE_FLTK)
+    UpdateViewsInGUI();
+#endif
+  }
+  
   GMSH_Solve_Plugin *sp = GMSH_PluginManager::instance()->findSolverPlugin();
   if(sp) {
     sp->readSolverFile(f);
@@ -191,9 +277,24 @@ void ParseString(char *str)
   }
 }
 
+void SetProjectName(char *name)
+{
+  char base[356], ext[256];
+  SplitFileName(name, base, ext);
+
+  strncpy(CTX.filename, name, 255);
+  strncpy(CTX.base_filename, base, 255);
+  strncpy(THEM->name, CTX.base_filename, 255);
+
+#if defined(HAVE_FLTK)
+  if(!CTX.batch)
+    WID->set_title(CTX.filename);
+#endif
+}
+
 int MergeProblem(char *name, int warn_if_missing)
 {
-  char ext[5], tmp[256];
+  char ext[256], base[256], tmp[256];
   int status;
   FILE *fp;
 
@@ -203,13 +304,25 @@ int MergeProblem(char *name, int warn_if_missing)
     return 0;
   }
 
-  if(strlen(name) > 4) {
-    strncpy(ext, &name[strlen(name) - 4], 5);
-  }
-  else {
-    strcpy(ext, "");
-  }
+  SplitFileName(name, base, ext);
 
+#if defined(HAVE_FLTK)
+  if(!CTX.batch) {
+    if(!strcmp(ext, ".gz")) {
+      // the real solution would be to rewrite all our I/O functions in
+      // terms of gzFile, but until then, this is better than nothing
+      if(fl_ask("File '%s' is in gzip format.\n\nDo you want to uncompress it?", name)){
+	fclose(fp);
+	sprintf(tmp, "gunzip -c %s > %s", name, base);
+	SystemCall(tmp);
+	if(!strcmp(CTX.filename, name)) // this is the project file
+	  SetProjectName(base);
+	return MergeProblem(base);
+      }
+    }
+  }
+#endif
+  
   if(!strcmp(ext, ".ppm") || !strcmp(ext, ".pnm")) {
     // An image file is used as an input, we transform it onto a post
     // pro file that could be used as a background mesh. We should
@@ -262,8 +375,6 @@ int MergeProblem(char *name, int warn_if_missing)
 
 void OpenProblem(char *name)
 {
-  char ext[6];
-
   if(CTX.threads_lock) {
     Msg(INFO, "I'm busy! Ask me that later...");
     return;
@@ -276,36 +387,9 @@ void OpenProblem(char *name)
   // Initialize pseudo random mesh generator to the same seed
   srand(1);
 
-  ParseString(TheOptString);
+  SetProjectName(name);
 
-  strncpy(CTX.filename, name, 255);
-  strncpy(CTX.base_filename, name, 255);
-
-  if(strlen(name) > 4) {
-    strncpy(ext, &name[strlen(name) - 4], 5);
-  }
-  else {
-    strcpy(ext, "");
-  }
-
-  if(!strcmp(ext, ".geo") || !strcmp(ext, ".GEO") ||
-     !strcmp(ext, ".msh") || !strcmp(ext, ".MSH") ||
-     !strcmp(ext, ".stl") || !strcmp(ext, ".STL") ||
-     !strcmp(ext, ".sms") || !strcmp(ext, ".SMS") ||
-     !strcmp(ext, ".ppm") || !strcmp(ext, ".PPM") ||
-     !strcmp(ext, ".pnm") || !strcmp(ext, ".PNM") ||
-     !strcmp(ext, ".pos") || !strcmp(ext, ".POS")) {
-    CTX.base_filename[strlen(name) - 4] = '\0';
-  }
-
-  strncpy(THEM->name, CTX.base_filename, 255);
-
-#if defined(HAVE_FLTK)
-  if(!CTX.batch)
-    WID->set_title(CTX.filename);
-#endif
-
-  int status = MergeProblem(CTX.filename);
+  int status = MergeProblem(name);
 
   ApplyLcFactor(THEM);
 
