@@ -1,6 +1,6 @@
-// $Id: CutMap.cpp,v 1.41 2004/11/09 16:27:53 remacle Exp $
+// $Id: CutMap.cpp,v 1.49 2005/03/09 08:07:04 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,8 +29,10 @@ StringXNumber CutMapOptions_Number[] = {
   {GMSH_FULLRC, "A", GMSH_CutMapPlugin::callbackA, 1.},
   {GMSH_FULLRC, "dTimeStep", NULL, -1.},
   {GMSH_FULLRC, "dView", NULL, -1.},
-  {GMSH_FULLRC, "iView", NULL, -1.},
-  {GMSH_FULLRC, "recurLevel", NULL, 4}
+  {GMSH_FULLRC, "ExtractVolume", GMSH_CutMapPlugin::callbackVol, 0.},
+  {GMSH_FULLRC, "RecurLevel", GMSH_CutMapPlugin::callbackRecur, 4},
+  {GMSH_FULLRC, "TargetError", GMSH_CutMapPlugin::callbackTarget, 0},
+  {GMSH_FULLRC, "iView", NULL, -1.}
 };
 
 extern "C"
@@ -50,12 +52,12 @@ double GMSH_CutMapPlugin::callbackA(int num, int action, double value)
 {
   double min = 0., max = 1.;
   if(action > 0){
-    int iview = (int)CutMapOptions_Number[3].def;
+    int iview = (int)CutMapOptions_Number[6].def;
     if(iview < 0) iview = num;
-    Post_View *v = (Post_View*)List_Pointer_Test(CTX.post.list, iview);
-    if(v){
-      min = v->Min;
-      max = v->Max;
+    Post_View **vv = (Post_View **)List_Pointer_Test(CTX.post.list, iview);
+    if(vv){
+      min = (*vv)->Min;
+      max = (*vv)->Max;
     }
   }
   switch(action){ // configure the input field
@@ -67,9 +69,42 @@ double GMSH_CutMapPlugin::callbackA(int num, int action, double value)
   return 0.;
 }
 
+double GMSH_CutMapPlugin::callbackVol(int num, int action, double value)
+{
+  switch(action){ // configure the input field
+  case 1: return 1.;
+  case 2: return -1.;
+  case 3: return 1.;
+  default: break;
+  }
+  return 0.;
+}
+
+double GMSH_CutMapPlugin::callbackRecur(int num, int action, double value)
+{
+  switch(action){ // configure the input field
+  case 1: return 1.;
+  case 2: return 0.;
+  case 3: return 10.;
+  default: break;
+  }
+  return 0.;
+}
+
+double GMSH_CutMapPlugin::callbackTarget(int num, int action, double value)
+{
+  switch(action){ // configure the input field
+  case 1: return 0.01;
+  case 2: return 0.;
+  case 3: return 1.;
+  default: break;
+  }
+  return 0.;
+}
+
 void GMSH_CutMapPlugin::getName(char *name) const
 {
-  strcpy(name, "Cut map");
+  strcpy(name, "Cut Map");
 }
 
 void GMSH_CutMapPlugin::getInfos(char *author, char *copyright,
@@ -78,17 +113,21 @@ void GMSH_CutMapPlugin::getInfos(char *author, char *copyright,
   strcpy(author, "J.-F. Remacle (remacle@scorec.rpi.edu)");
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
-         "Plugin(CutMap) extracts the isovalue surface of\n"
-         "value `A' from the view `iView' and draws the\n"
-	 "`dTimeStep'-th value of the view `dView' on this\n"
-	 "isovalue surface. If `iView' < 0, the plugin is\n"
-	 "run on the current view. If `dTimeStep' < 0, the\n"
+         "Plugin(CutMap) extracts the isosurface of value\n"
+         "`A' from the view `iView' and draws the\n"
+	 "`dTimeStep'-th value of the view `dView' on the\n"
+	 "isosurface. If `iView' < 0, the plugin is run\n"
+	 "on the current view. If `dTimeStep' < 0, the\n"
 	 "plugin uses, for each time step in `iView', the\n"
 	 "corresponding time step in `dView'. If `dView'\n"
 	 "< 0, the plugin uses `iView' as the value source.\n"
+	 "If `ExtractVolume' is nonzero, the plugin\n" 
+	 "extracts the isovolume with values greater (if\n"
+	 "`ExtractVolume' > 0) or smaller (if `ExtractVolume'\n"
+	 "< 0) than the isosurface `A'.\n"
 	 "\n"
-	 "Plugin(CutMap) creates as many views as there are\n"
-	 "time steps in `iView'.\n");
+	 "Plugin(CutMap) creates as many views as there\n"
+	 "are time steps in `iView'.\n");
 }
 
 int GMSH_CutMapPlugin::getNbOptions() const
@@ -108,19 +147,21 @@ void GMSH_CutMapPlugin::catchErrorMessage(char *errorMessage) const
 
 double GMSH_CutMapPlugin::levelset(double x, double y, double z, double val) const
 {
-  // we must look into the map for A - Map(x,y,z)
+  // we must look into the map for Map(x,y,z) - A
   // this is the case when the map is the same as the view,
   // the result is the extraction of isovalue A
-  return CutMapOptions_Number[0].def - val;
+  return val - CutMapOptions_Number[0].def;
 }
 
 Post_View *GMSH_CutMapPlugin::execute(Post_View * v)
 {
-  int iView = (int)CutMapOptions_Number[3].def;
+  int iView = (int)CutMapOptions_Number[6].def;
   _valueIndependent = 0;
   _valueView = (int)CutMapOptions_Number[2].def;
   _valueTimeStep = (int)CutMapOptions_Number[1].def;
+  _extractVolume = (int)CutMapOptions_Number[3].def;
   _recurLevel = (int)CutMapOptions_Number[4].def;
+  _targetError = CutMapOptions_Number[5].def;
   _orientation = GMSH_LevelsetPlugin::MAP;
   
   if(iView < 0)
@@ -131,7 +172,7 @@ Post_View *GMSH_CutMapPlugin::execute(Post_View * v)
     return v;
   }
   
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
   
   return GMSH_LevelsetPlugin::execute(v1);
 }
