@@ -1,6 +1,6 @@
-// $Id: CutPlane.cpp,v 1.37 2004/11/09 16:27:53 remacle Exp $
+// $Id: CutPlane.cpp,v 1.47 2005/01/09 02:18:59 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,8 +37,10 @@ StringXNumber CutPlaneOptions_Number[] = {
   {GMSH_FULLRC, "B", GMSH_CutPlanePlugin::callbackB, 0.},
   {GMSH_FULLRC, "C", GMSH_CutPlanePlugin::callbackC, 0.},
   {GMSH_FULLRC, "D", GMSH_CutPlanePlugin::callbackD, -0.01},
-  {GMSH_FULLRC, "iView", NULL, -1.},
-  {GMSH_FULLRC, "recurLevel", NULL, 4}
+  {GMSH_FULLRC, "ExtractVolume", GMSH_CutPlanePlugin::callbackVol, 0},
+  {GMSH_FULLRC, "RecurLevel", GMSH_CutPlanePlugin::callbackRecur, 4},
+  {GMSH_FULLRC, "TargetError", GMSH_CutPlanePlugin::callbackTarget, 0.},
+  {GMSH_FULLRC, "iView", NULL, -1.}
 };
 
 extern "C"
@@ -57,12 +59,14 @@ GMSH_CutPlanePlugin::GMSH_CutPlanePlugin()
 void GMSH_CutPlanePlugin::draw()
 {
 #if defined(HAVE_FLTK)
-  int num = (int)CutPlaneOptions_Number[4].def;
+  int num = (int)CutPlaneOptions_Number[7].def;
   if(num < 0) num = iview;
-  Post_View *v = (Post_View*)List_Pointer_Test(CTX.post.list, num);
-  if(!v) return;
-  Draw_PlaneInBoundingBox(v->BBox[0], v->BBox[2], v->BBox[4],
-			  v->BBox[1], v->BBox[3], v->BBox[5],
+  Post_View **vv = (Post_View **)List_Pointer_Test(CTX.post.list, num);
+  if(!vv) return;
+  glColor4ubv((GLubyte *) & CTX.color.fg);
+  glLineWidth(CTX.line_width);
+  Draw_PlaneInBoundingBox((*vv)->BBox[0], (*vv)->BBox[2], (*vv)->BBox[4],
+			  (*vv)->BBox[1], (*vv)->BBox[3], (*vv)->BBox[5],
 			  CutPlaneOptions_Number[0].def,
 			  CutPlaneOptions_Number[1].def,
 			  CutPlaneOptions_Number[2].def,
@@ -70,84 +74,68 @@ void GMSH_CutPlanePlugin::draw()
 #endif
 }
 
-void GMSH_CutPlanePlugin::callback()
+double GMSH_CutPlanePlugin::callback(int num, int action, double value, double *opt,
+				     double step, double min, double max)
 {
-#if defined(HAVE_FLTK)
-  CTX.post.plugin_draw_function = draw;
-  int old = CTX.draw_bbox;
-  CTX.draw_bbox = 1;
-  if(CTX.fast_redraw){
-    CTX.post.draw = 0;
-    CTX.mesh.draw = 0;
+  if(action > 0) iview = num;
+  switch(action){ // configure the input field
+  case 1: return step;
+  case 2: return min;
+  case 3: return max;
+  default: break;
   }
-  if(!CTX.batch) 
-    Draw();
-  CTX.post.plugin_draw_function = NULL;
-  CTX.draw_bbox = old;
-  CTX.post.draw = 1;
-  CTX.mesh.draw = 1;
+  *opt = value;
+#if defined(HAVE_FLTK)
+  DrawPlugin(draw);
 #endif
+  return 0.;
 }
 
 double GMSH_CutPlanePlugin::callbackA(int num, int action, double value)
 {
-  if(action > 0) iview = num;
-  switch(action){ // configure the input field
-  case 1: return 0.01;
-  case 2: return -1.;
-  case 3: return 1.;
-  default: break;
-  }
-  CutPlaneOptions_Number[0].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutPlaneOptions_Number[0].def,
+		  0.01, -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackB(int num, int action, double value)
 {
-  if(action > 0) iview = num;
-  switch(action){
-  case 1: return 0.01;
-  case 2: return -1.;
-  case 3: return 1.;
-  default: break;
-  }
-  CutPlaneOptions_Number[1].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutPlaneOptions_Number[1].def,
+		  0.01, -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackC(int num, int action, double value)
 {
-  if(action > 0) iview = num;
-  switch(action){
-  case 1: return 0.01;
-  case 2: return -1.;
-  case 3: return 1.;
-  default: break;
-  }
-  CutPlaneOptions_Number[2].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutPlaneOptions_Number[2].def,
+		  0.01, -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackD(int num, int action, double value)
 {
-  if(action > 0) iview = num;
-  switch(action){
-  case 1: return CTX.lc/200.;
-  case 2: return -CTX.lc;
-  case 3: return CTX.lc;
-  default: break;
-  }
-  CutPlaneOptions_Number[3].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutPlaneOptions_Number[3].def,
+		  CTX.lc/200., -CTX.lc, CTX.lc);
+}
+
+double GMSH_CutPlanePlugin::callbackVol(int num, int action, double value)
+{
+  return callback(num, action, value, &CutPlaneOptions_Number[4].def,
+		  1., -1, 1);
+}
+
+double GMSH_CutPlanePlugin::callbackRecur(int num, int action, double value)
+{
+  return callback(num, action, value, &CutPlaneOptions_Number[5].def,
+		  1, 0, 10);
+}
+
+double GMSH_CutPlanePlugin::callbackTarget(int num, int action, double value)
+{
+  return callback(num, action, value, &CutPlaneOptions_Number[6].def,
+		  0.01, 0., 1.);
 }
 
 void GMSH_CutPlanePlugin::getName(char *name) const
 {
-  strcpy(name, "Cut plane");
+  strcpy(name, "Cut Plane");
 }
 
 void GMSH_CutPlanePlugin::getInfos(char *author, char *copyright,
@@ -158,8 +146,10 @@ void GMSH_CutPlanePlugin::getInfos(char *author, char *copyright,
   strcpy(help_text,
          "Plugin(CutPlane) cuts the view `iView' with\n"
 	 "the plane `A'*X + `B'*Y + `C'*Z + `D' = 0. If\n"
-	 "`iView' < 0, the plugin is run on the current\n"
-	 "view.\n"
+	 "`ExtractVolume' is nonzero, the plugin extracts\n"
+	 "the elements on one side of the plane (depending\n"
+	 "on the sign of `ExtractVolume'). If `iView' < 0,\n"
+	 "the plugin is run on the current view.\n"
 	 "\n"
 	 "Plugin(CutPlane) creates one new view.\n");
 }
@@ -188,7 +178,7 @@ double GMSH_CutPlanePlugin::levelset(double x, double y, double z, double val) c
 
 Post_View *GMSH_CutPlanePlugin::execute(Post_View * v)
 {
-  int iView = (int)CutPlaneOptions_Number[4].def;
+  int iView = (int)CutPlaneOptions_Number[7].def;
   _ref[0] = CutPlaneOptions_Number[0].def;
   _ref[1] = CutPlaneOptions_Number[1].def;
   _ref[2] = CutPlaneOptions_Number[2].def;
@@ -196,17 +186,19 @@ Post_View *GMSH_CutPlanePlugin::execute(Post_View * v)
   _valueView = -1;
   _valueTimeStep = -1;
   _orientation = GMSH_LevelsetPlugin::PLANE;
+  _extractVolume = (int)CutPlaneOptions_Number[4].def;
   _recurLevel = (int)CutPlaneOptions_Number[5].def;
-
+  _targetError = CutPlaneOptions_Number[6].def;
+  
   if(iView < 0)
-    iView = v ? v->Index : 0;
-
+      iView = v ? v->Index : 0;
+  
   if(!List_Pointer_Test(CTX.post.list, iView)) {
-    Msg(GERROR, "View[%d] does not exist", iView);
-    return v;
+      Msg(GERROR, "View[%d] does not exist", iView);
+      return v;
   }
-
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
-
+  
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
+  
   return GMSH_LevelsetPlugin::execute(v1);
 }
