@@ -1,6 +1,6 @@
-// $Id: DecomposeInSimplex.cpp,v 1.13 2004/07/14 22:42:27 geuzaine Exp $
+// $Id: DecomposeInSimplex.cpp,v 1.19 2005/03/14 22:46:26 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "Views.h"
 #include "Context.h"
 #include "Malloc.h"
+#include "Numeric.h"
 
 extern Context_T CTX;
 
@@ -48,7 +49,7 @@ GMSH_DecomposeInSimplexPlugin::GMSH_DecomposeInSimplexPlugin()
 
 void GMSH_DecomposeInSimplexPlugin::getName(char *name) const
 {
-  strcpy(name, "Decompose in simplex");
+  strcpy(name, "Decompose in Simplex");
 }
 
 void GMSH_DecomposeInSimplexPlugin::getInfos(char *author, char *copyright,
@@ -132,11 +133,11 @@ Post_View *GMSH_DecomposeInSimplexPlugin::execute(Post_View * v)
     return v;
   }
 
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
 
-  // Bail out if the view is a duplicate or if other views duplicate it
-  if(v1->DuplicateOf || v1->Links) {
-    Msg(GERROR, "DecomposeInSimplex cannot be applied to a duplicated view");
+  // Bail out if the view is an alias or if other views duplicate it
+  if(v1->AliasOf || v1->Links) {
+    Msg(GERROR, "DecomposeInSimplex cannot be applied to an aliased view");
     return 0;
   }
 
@@ -200,11 +201,32 @@ void DecomposeInSimplex::reorder(int map[4], int n,
     zn[i] = z[map[i]];
   }
 
+  int map2[4] = {map[0], map[1], map[2], map[3]};
+#if 0
+  // make sure to create tets with positive volume?
+  if(n == 4){ // tets
+    double mat[3][3];
+    mat[0][0] = xn[1] - xn[0]; mat[0][1] = xn[2] - xn[0]; mat[0][2] = xn[3] - xn[0];
+    mat[1][0] = yn[1] - yn[0]; mat[1][1] = yn[2] - yn[0]; mat[1][2] = yn[3] - yn[0];
+    mat[2][0] = zn[1] - zn[0]; mat[2][1] = zn[2] - zn[0]; mat[2][2] = zn[3] - zn[0];
+    if(det3x3(mat) < 0.){
+      map2[0] = map[1];
+      map2[1] = map[0];
+      xn[0] = x[map2[0]];
+      yn[0] = y[map2[0]];
+      zn[0] = z[map2[0]];
+      xn[1] = x[map2[1]];
+      yn[1] = y[map2[1]];
+      zn[1] = z[map2[1]];
+    }
+  }
+#endif
+
   for(int ts = 0; ts < _numTimeSteps; ts++)
     for(int i = 0; i < n; i++) {
       for(int j = 0; j < _numComponents; j++)
 	valn[ts*n*_numComponents + i*_numComponents + j] = 
-	  val[ts*_numNodes*_numComponents + map[i]*_numComponents + j];
+	  val[ts*_numNodes*_numComponents + map2[i]*_numComponents + j];
   }
 }
 
@@ -213,7 +235,7 @@ void DecomposeInSimplex::decompose(int num,
 				   double *xn, double *yn, double *zn, double *valn)
 {
   int quadTri[2][4] = {{0,1,2,-1}, {0,2,3,-1}};
-  int hexaTet[6][4] = {{0,1,2,5}, {0,2,5,6}, {0,4,5,6}, {0,2,3,6}, {0,4,6,7}, {0,3,6,7}};
+  int hexaTet[6][4] = {{0,1,3,7}, {0,4,1,7}, {1,4,5,7}, {1,2,3,7}, {1,6,2,7}, {1,5,6,7}};
   int prisTet[3][4] = {{0,1,2,4}, {0,2,4,5}, {0,3,4,5}};
   int pyraTet[2][4] = {{0,1,3,4}, {1,2,3,4}};
 
