@@ -1,7 +1,7 @@
 #ifndef _EVALUATE_H_
-#define _EVALUATE_H
+#define _EVALUATE_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "Plugin.h"
+#include "OctreePost.h"
 
 extern "C"
 {
@@ -29,6 +30,13 @@ extern "C"
 
 class GMSH_EvaluatePlugin : public GMSH_Post_Plugin
 {
+private:
+  OctreePost *_octree;
+  void evaluate(Post_View *v, List_T *list, int nbElm,
+		Post_View *v2, List_T *list2, int nbElm2,
+		int nbNod, int nbComp, int comp, 
+		int timeStep, int timeStep2,
+		char *expression);
 public:
   GMSH_EvaluatePlugin();
   void getName(char *name) const;
