@@ -1,6 +1,6 @@
-// $Id: HarmonicToTime.cpp,v 1.2 2004/10/31 01:50:27 geuzaine Exp $
+// $Id: HarmonicToTime.cpp,v 1.6 2005/01/08 20:15:19 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,8 +29,8 @@
 extern Context_T CTX;
 
 StringXNumber HarmonicToTimeOptions_Number[] = {
-  {GMSH_FULLRC, "realPart", NULL, 0.},
-  {GMSH_FULLRC, "imaginaryPart", NULL, 1.},
+  {GMSH_FULLRC, "RealPart", NULL, 0.},
+  {GMSH_FULLRC, "ImaginaryPart", NULL, 1.},
   {GMSH_FULLRC, "nSteps", NULL, 20.},
   {GMSH_FULLRC, "iView", NULL, -1.}
 };
@@ -51,7 +51,7 @@ GMSH_HarmonicToTimePlugin::GMSH_HarmonicToTimePlugin()
 
 void GMSH_HarmonicToTimePlugin::getName(char *name) const
 {
-  strcpy(name, "Harmonic to time");
+  strcpy(name, "Harmonic to Time");
 }
 
 void GMSH_HarmonicToTimePlugin::getInfos(char *author, char *copyright,
@@ -61,10 +61,10 @@ void GMSH_HarmonicToTimePlugin::getInfos(char *author, char *copyright,
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
          "Plugin(HarmonicToTime) takes the values in the\n"
-	 "time steps `realPart' and `imaginaryPart' of\n"
+	 "time steps `RealPart' and `ImaginaryPart' of\n"
          "the view `iView', and creates a new view\n"
-         "containing (`iView'[`realPart'] * cos(p) -\n"
-         "`iView'[`imaginaryPart'] * sin(p)), with\n"
+         "containing (`iView'[`RealPart'] * cos(p) -\n"
+         "`iView'[`ImaginaryPart'] * sin(p)), with\n"
          "p = 2*Pi*k/`nSteps', k = 0, ..., `nSteps'-1.\n"
 	 "If `iView' < 0, the plugin is run on the\n"
 	 "current view.\n"
@@ -136,7 +136,7 @@ Post_View *GMSH_HarmonicToTimePlugin::execute(Post_View * v)
     return v;
   }
 
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
   if(rIndex < 0 || rIndex >= v1->NbTimeStep ||
      iIndex < 0 || iIndex >= v1->NbTimeStep){
     Msg(GERROR, "Wrong real or imaginary part index");
@@ -144,8 +144,6 @@ Post_View *GMSH_HarmonicToTimePlugin::execute(Post_View * v)
   }
 
   Post_View *v2 = BeginView(1);
-  // get v1 again, in case BeginView induced a realloc of the view list
-  v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
 
   h2t(v1->NbSP, v1->SP, &v2->NbSP, v2->SP, 1, 1, rIndex, iIndex, nSteps);
   h2t(v1->NbVP, v1->VP, &v2->NbVP, v2->VP, 1, 3, rIndex, iIndex, nSteps);
