@@ -1,6 +1,6 @@
-// $Id: Plugin.cpp,v 1.63 2004/11/09 16:27:53 remacle Exp $
+// $Id: Plugin.cpp,v 1.73 2005/01/12 19:10:41 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -41,6 +41,12 @@
 #include "Skin.h"
 #include "Extract.h"
 #include "HarmonicToTime.h"
+#include "Integrate.h"
+#include "Gradient.h"
+#include "Curl.h"
+#include "Divergence.h"
+#include "Annotate.h"
+#include "Remove.h"
 #include "DecomposeInSimplex.h"
 #include "Smooth.h"
 #include "Transform.h"
@@ -48,7 +54,11 @@
 #include "SphericalRaise.h"
 #include "DisplacementRaise.h"
 #include "StructuralSolver.h"
+#include "Eigenvectors.h"
+#include "Eigenvalues.h"
+#include "Lambda2.h"
 #include "Evaluate.h"
+#include "Probe.h"
 #include "Context.h"
 
 using namespace std;
@@ -155,8 +165,9 @@ GMSH_PluginManager *GMSH_PluginManager::instance()
 
 void GMSH_PluginManager::registerDefaultPlugins()
 {
-  // SOLVE PLUGINS
   char *homeplugins = getenv("GMSHPLUGINSHOME");
+
+  // SOLVE PLUGINS
   if(CTX.solver.plugins){
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("StructuralSolver", GMSH_RegisterStructuralSolverPlugin()));
@@ -190,6 +201,26 @@ void GMSH_PluginManager::registerDefaultPlugins()
 		      ("DisplacementRaise", GMSH_RegisterDisplacementRaisePlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("HarmonicToTime", GMSH_RegisterHarmonicToTimePlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Integrate", GMSH_RegisterIntegratePlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Gradient", GMSH_RegisterGradientPlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Curl", GMSH_RegisterCurlPlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Divergence", GMSH_RegisterDivergencePlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Annotate", GMSH_RegisterAnnotatePlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Remove", GMSH_RegisterRemovePlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Eigenvectors", GMSH_RegisterEigenvectorsPlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Eigenvalues", GMSH_RegisterEigenvaluesPlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Lambda2", GMSH_RegisterLambda2Plugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("Probe", GMSH_RegisterProbePlugin()));
 #if defined(HAVE_TRIANGLE)
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("Triangulate", GMSH_RegisterTriangulatePlugin()));
