@@ -1,6 +1,6 @@
-// $Id: Skin.cpp,v 1.28 2004/09/16 19:15:27 geuzaine Exp $
+// $Id: Skin.cpp,v 1.30 2005/01/01 19:35:39 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -181,8 +181,8 @@ Post_View *GMSH_SkinPlugin::execute(Post_View * v)
     return v;
   }
 
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
   Post_View *v2 = BeginView(1);
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
 
   _nbTimeStep = v1->NbTimeStep;
 
