@@ -1,6 +1,6 @@
-// $Id: Smooth.cpp,v 1.19 2004/05/16 20:04:43 geuzaine Exp $
+// $Id: Smooth.cpp,v 1.21 2005/01/01 19:35:39 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -90,7 +90,7 @@ Post_View *GMSH_SmoothPlugin::execute(Post_View * v)
     return v;
   }
 
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
 
   v1->smooth();
   return v1;
