@@ -1,6 +1,6 @@
-// $Id: SphericalRaise.cpp,v 1.19 2004/07/21 18:18:23 geuzaine Exp $
+// $Id: SphericalRaise.cpp,v 1.22 2005/01/01 19:35:39 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ GMSH_SphericalRaisePlugin::GMSH_SphericalRaisePlugin()
 
 void GMSH_SphericalRaisePlugin::getName(char *name) const
 {
-  strcpy(name, "Spherical raise");
+  strcpy(name, "Spherical Raise");
 }
 
 void GMSH_SphericalRaisePlugin::getInfos(char *author, char *copyright,
@@ -179,7 +179,7 @@ Post_View *GMSH_SphericalRaisePlugin::execute(Post_View * v)
     return v;
   }
 
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
 
   sphericalRaise(v1, timeStep, center, raise);
   return v1;
