@@ -1,6 +1,6 @@
-// $Id: StreamLines.cpp,v 1.12 2004/06/24 16:02:21 geuzaine Exp $
+// $Id: StreamLines.cpp,v 1.22 2005/03/04 19:08:38 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,23 +24,26 @@
 #include "StreamLines.h"
 #include "List.h"
 #include "Context.h"
-#include "Views.h"
-#include "Message.h"
+
+#if defined(HAVE_FLTK)
+#include "GmshUI.h"
+#include "Draw.h"
+#endif
 
 extern Context_T CTX;
 
 StringXNumber StreamLinesOptions_Number[] = {
-  {GMSH_FULLRC, "X0", NULL, 2.39},
-  {GMSH_FULLRC, "Y0", NULL, .445},
-  {GMSH_FULLRC, "Z0", NULL, 0.},
-  {GMSH_FULLRC, "X1", NULL, 2.39},
-  {GMSH_FULLRC, "Y1", NULL, 0.94},
-  {GMSH_FULLRC, "Z1", NULL, 0.},
-  {GMSH_FULLRC, "X2", NULL, 2.39},
-  {GMSH_FULLRC, "Y2", NULL, .445},
-  {GMSH_FULLRC, "Z2", NULL, 1.},
-  {GMSH_FULLRC, "nPointsU", NULL, 20},
-  {GMSH_FULLRC, "nPointsV", NULL, 1},
+  {GMSH_FULLRC, "X0", GMSH_StreamLinesPlugin::callbackX0, 0.},
+  {GMSH_FULLRC, "Y0", GMSH_StreamLinesPlugin::callbackY0, 0.},
+  {GMSH_FULLRC, "Z0", GMSH_StreamLinesPlugin::callbackZ0, 0.},
+  {GMSH_FULLRC, "X1", GMSH_StreamLinesPlugin::callbackX1, 1.},
+  {GMSH_FULLRC, "Y1", GMSH_StreamLinesPlugin::callbackY1, 0.},
+  {GMSH_FULLRC, "Z1", GMSH_StreamLinesPlugin::callbackZ1, 0.},
+  {GMSH_FULLRC, "X2", GMSH_StreamLinesPlugin::callbackX2, 0.},
+  {GMSH_FULLRC, "Y2", GMSH_StreamLinesPlugin::callbackY2, 1.},
+  {GMSH_FULLRC, "Z2", GMSH_StreamLinesPlugin::callbackZ2, 0.},
+  {GMSH_FULLRC, "nPointsU", GMSH_StreamLinesPlugin::callbackU, 10},
+  {GMSH_FULLRC, "nPointsV", GMSH_StreamLinesPlugin::callbackV, 1},
   {GMSH_FULLRC, "MaxIter", NULL, 100},
   {GMSH_FULLRC, "DT", NULL, .1},
   {GMSH_FULLRC, "dView", NULL, -1.},
@@ -55,15 +58,110 @@ extern "C"
   }
 }
 
-
 GMSH_StreamLinesPlugin::GMSH_StreamLinesPlugin()
 {
   ;
 }
 
+void GMSH_StreamLinesPlugin::draw()
+{
+#if defined(HAVE_FLTK)
+  glColor4ubv((GLubyte *) & CTX.color.fg);
+  double p[3];
+  for(int i = 0; i < getNbU(); ++i){
+    for(int j = 0; j < getNbV(); ++j){
+      getPoint(i, j, p);
+      Draw_Point(1, CTX.point_size, &p[0], &p[1], &p[2], 1);
+    }
+  }
+#endif
+}
+
+double GMSH_StreamLinesPlugin::callback(int num, int action, double value, double *opt,
+					double step, double min, double max)
+{
+  switch(action){ // configure the input field
+  case 1: return step;
+  case 2: return min;
+  case 3: return max;
+  default: break;
+  }
+  *opt = value;
+#if defined(HAVE_FLTK)
+  DrawPlugin(draw);
+#endif
+  return 0.;
+}
+
+double GMSH_StreamLinesPlugin::callbackX0(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[0].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackY0(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[1].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackZ0(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[2].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackX1(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[3].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackY1(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[4].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackZ1(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[5].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackX2(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[6].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackY2(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[7].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackZ2(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[8].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+}
+
+double GMSH_StreamLinesPlugin::callbackU(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[9].def,
+		  1, 1, 100);
+}
+
+double GMSH_StreamLinesPlugin::callbackV(int num, int action, double value)
+{
+  return callback(num, action, value, &StreamLinesOptions_Number[10].def,
+		  1, 1, 100);
+}
+
 void GMSH_StreamLinesPlugin::getName(char *name) const
 {
-  strcpy(name, "Stream lines");
+  strcpy(name, "Stream Lines");
 }
 
 void GMSH_StreamLinesPlugin::getInfos(char *author, char *copyright,
@@ -73,9 +171,9 @@ void GMSH_StreamLinesPlugin::getInfos(char *author, char *copyright,
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
 	 "Plugin(StreamLines) computes stream lines\n"
-	 "from a triangle/tetrahedron vector view `iView'\n"
-	 "and optionally interpolates the scalar view `dView'\n"
-	 "on the resulting stream lines. It takes as input a\n"
+	 "from a vector view `iView' and optionally\n"
+	 "interpolates the scalar view `dView' on the\n"
+	 "resulting stream lines. It takes as input a\n"
 	 "grid defined by the 3 points (`X0',`Y0',`Z0')\n"
 	 "(origin), (`X1',`Y1',`Z1') (axis of U) and\n"
 	 "(`X2',`Y2',`Z2') (axis of V). The number of points\n"
@@ -109,20 +207,20 @@ void GMSH_StreamLinesPlugin::catchErrorMessage(char *errorMessage) const
   strcpy(errorMessage, "StreamLines failed...");
 }
 
-int GMSH_StreamLinesPlugin::getNbU()const 
+int GMSH_StreamLinesPlugin::getNbU()
 {
   return (int)StreamLinesOptions_Number[9].def;
 }
 
-int GMSH_StreamLinesPlugin::getNbV()const 
+int GMSH_StreamLinesPlugin::getNbV()
 {
   return (int)StreamLinesOptions_Number[10].def;
 }
 
-void GMSH_StreamLinesPlugin::getPoint(int iU, int iV, double *X) const 
+void GMSH_StreamLinesPlugin::getPoint(int iU, int iV, double *X)
 {
-  double u = (double)iU / (double)(getNbU());
-  double v = (double)iV / (double)(getNbV());
+  double u = getNbU() > 1 ? (double)iU / (double)(getNbU() - 1.) : 0.;
+  double v = getNbV() > 1 ? (double)iV / (double)(getNbV() - 1.) : 0.;
   X[0] = StreamLinesOptions_Number[0].def + 
     u  * (StreamLinesOptions_Number[3].def-StreamLinesOptions_Number[0].def) +
     v  * (StreamLinesOptions_Number[6].def-StreamLinesOptions_Number[0].def) ;
@@ -140,11 +238,21 @@ Post_View * GMSH_StreamLinesPlugin::GenerateView(int iView, int dView) const
   const double a1=0.5, a2=0.5, a3=1.0, a4=1.0;
   const double DT = StreamLinesOptions_Number[12].def;
   double XINIT[3], X[3], DX[3], X1[3], X2[3], X3[3], X4[3];
-  double sizeElem = 0.033, val[3], *val2 = NULL;
+  double val[3], *val2 = NULL;
 
   Post_View *View = BeginView(1);
-  Post_View *v1 = (Post_View*)List_Pointer_Test(CTX.post.list, iView);
-  Post_View *v2 = (Post_View*)List_Pointer_Test(CTX.post.list, dView);
+
+  Post_View *v1;
+  if(List_Pointer_Test(CTX.post.list, iView))
+    List_Read(CTX.post.list, iView, &v1);
+  else
+    v1 = NULL;
+
+  Post_View *v2;
+  if(List_Pointer_Test(CTX.post.list, dView))
+    List_Read(CTX.post.list, dView, &v2);
+  else
+    v2 = NULL;
 
   if(!v1) {
     Msg(GERROR, "View[%d] does not exist", iView);
@@ -185,21 +293,22 @@ Post_View * GMSH_StreamLinesPlugin::GenerateView(int iView, int dView) const
 	// X4 = X + a4 * DT * V(X3)
 	// X = X + b1 X1 + b2 X2 + b3 X3 + b4 x4
 
-	o.searchVector(X[0], X[1], X[2], val, &sizeElem, 0);
-	// double normV = sqrt(val[0]*val[0]+
-	//	 	       val[1]*val[1]+
-	//		       val[2]*val[2]);	     
+	// o.searchVector(X[0], X[1], X[2], val, 0, &sizeElem);
+	// double normV = sqrt(val[0]*val[0]+val[1]*val[1]+val[2]*val[2]);	     
 	// if (normV==0.0) normV = 1.0;
-	// double DT = sizeElem / normV ; /// CFL = 1 ==> secure 
+	// double DT = sizeElem / normV ; // CFL = 1 ==> secure 
+	o.searchVector(X[0], X[1], X[2], val, 0);
 	for(int k = 0; k < 3; k++) X1[k] = X[k] + DT * val[k] * a1;
-	o.searchVector(X1[0], X1[1], X1[2], val, &sizeElem, 0);
+	o.searchVector(X1[0], X1[1], X1[2], val, 0);
 	for(int k = 0; k < 3; k++) X2[k] = X[k] + DT * val[k] * a2;
-	o.searchVector(X2[0], X2[1], X2[2], val, &sizeElem, 0);
+	o.searchVector(X2[0], X2[1], X2[2], val, 0);
 	for(int k = 0; k < 3; k++) X3[k] = X[k] + DT * val[k] * a3;
-	o.searchVector(X3[0], X3[1], X3[2], val, &sizeElem, 0);
+	o.searchVector(X3[0], X3[1], X3[2], val, 0);
 	for(int k = 0; k < 3; k++) X4[k] = X[k] + DT * val[k] * a4;
-	for(int k = 0; k < 3; k++) X[k] += (b1*(X1[k]-X[k]) + b2*(X2[k]-X[k]) + 
-					    b3*(X3[k]-X[k]) + b4*(X4[k]-X[k])) ;
+
+	for(int k = 0; k < 3; k++) 
+	  X[k] += (b1*(X1[k]-X[k]) + b2*(X2[k]-X[k]) + 
+		   b3*(X3[k]-X[k]) + b4*(X4[k]-X[k])) ;
 	for(int k = 0; k < 3; k++) DX[k] = X[k] - XINIT[k];
 
 	if(v2){
