@@ -1,7 +1,7 @@
 #ifndef _STREAM_LINES_H_
 #define _STREAM_LINES_H
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@ extern "C"
 
 class GMSH_StreamLinesPlugin : public GMSH_Post_Plugin
 {
+  static double callback(int num, int action, double value, double *opt,
+			 double step, double min, double max);
 public:
   GMSH_StreamLinesPlugin();
   void getName(char *name) const;
@@ -39,10 +41,24 @@ public:
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   Post_View *execute(Post_View *);
-  virtual int getNbU() const ;
-  virtual int getNbV() const ;
-  virtual void getPoint(int iU, int iV, double *X ) const  ;
   virtual Post_View * GenerateView(int iView, int dView) const ;
+
+  static int getNbU();
+  static int getNbV();
+  static void getPoint(int iU, int iV, double *X );
+
+  static double callbackX0(int, int, double);
+  static double callbackY0(int, int, double);
+  static double callbackZ0(int, int, double);
+  static double callbackX1(int, int, double);
+  static double callbackY1(int, int, double);
+  static double callbackZ1(int, int, double);
+  static double callbackX2(int, int, double);
+  static double callbackY2(int, int, double);
+  static double callbackZ2(int, int, double);
+  static double callbackU(int, int, double);
+  static double callbackV(int, int, double);
+  static void draw();
 };
 
 #endif
