@@ -5,6 +5,10 @@
 #include "Utils.h"
 #include "Numeric.h"
 
+#ifdef HAVE_FLTK 
+#include "Shortcut_Window.h"
+#endif
+
 extern Mesh *THEM;
 extern Context_T CTX;
 
@@ -41,9 +45,11 @@ Structural_BeamSection:: Structural_BeamSection( const char *direct, std::string
   m.BGM.bgm = NULL;
   m.Grid.init = 0;
   Init_Mesh(&m);
+  //  THEM=&m;
 
   char temp[256];
   sprintf(temp, "%s/%s", direct,name.c_str());
+  //  printf("%s/%s\n", direct,name.c_str());
   // read the section (msh format)
   FILE *f = fopen (temp,"r");  
   Read_Mesh (&m, f, temp,FORMAT_MSH);
@@ -66,11 +72,13 @@ void Structural_BeamSection :: computeGeometricalProperties ()
 		    {.5,1,.5},
 		    {.5,.5,1}};
 
+  //  printf("%d surfaces\n",List_Nbr(surfaces));
   for (int i=0;i<List_Nbr(surfaces);++i)
     {
       Surface *s;
       List_Read(surfaces,i,&s);
-      List_T *triangles = Tree2List(s->Simplexes);
+      List_T *triangles = Tree2List(s->SimplexesBase);
+      //      printf("surface %d %d triangles\n",s->Num,List_Nbr(triangles));
       for(int j=0;j<List_Nbr(triangles);++j)
 	{
 	  Simplex *simp;
@@ -78,6 +86,7 @@ void Structural_BeamSection :: computeGeometricalProperties ()
 	  Vertex v = (*simp->V[0]+*simp->V[1]+*simp->V[2])*0.333333333333333333;
 	  double A = simp->surfsimpl();
 	  area+=A;
+	  //	  printf("triangle %d aire %g\n",i,A);
 	  xc += v.Pos.X*A;
 	  yc += v.Pos.Y*A;
 	}
@@ -106,7 +115,7 @@ void Structural_BeamSection :: computeGeometricalProperties ()
       List_Delete(triangles);
     }
   List_Delete(surfaces);  
-  printf("%s %g %g %g %g %g\n",name.c_str(),area,xc,yc,Iy,Iz);
+  //  printf("%s %g %g %g %g %g\n",name.c_str(),area,xc,yc,Iy,Iz);
 }
 
 void Structural_Texture::setup ()
@@ -156,7 +165,9 @@ void Structural_BeamSection ::  GL_DrawBeam (double pinit[3], double dir[3], con
       texture.setup();
     }
 
-  double X[3] = {dir[0],dir[1],dir[2]};
+  const double LL = sqrt(dir[0]*dir[0]+dir[1]*dir[1]+dir[2]*dir[2]);
+
+  double X[3] = {dir[0]/LL,dir[1]/LL,dir[2]/LL};
   double Z[3] = {dirz[0],dirz[1],dirz[2]};
   double Y[3];
   prodve(X,Z,Y);
@@ -168,6 +179,9 @@ void Structural_BeamSection ::  GL_DrawBeam (double pinit[3], double dir[3], con
   double invrot[3][3] = {{Z[0],Z[1],Z[2]},
 			 {Y[0],Y[1],Y[2]},
 			 {X[0],X[1],X[2]}};
+  //  printf("%g %g %g\n",invrot[0][0], invrot[0][1], invrot[0][2]);
+  //  printf("%g %g %g\n",invrot[1][0], invrot[1][1], invrot[1][2]);
+  //  printf("%g %g %g\n",invrot[2][0], invrot[2][1], invrot[2][2]);
   
   List_T *vertices = Tree2List (m.Vertices);
   Vertex *vert;
@@ -183,8 +197,8 @@ void Structural_BeamSection ::  GL_DrawBeam (double pinit[3], double dir[3], con
   List_T *surfaces = Tree2List (m.Surfaces);
   List_T *curves = Tree2List (m.Curves);
   List_T *points = Tree2List (m.Points);
-  glEnable(GL_POLYGON_OFFSET_FILL);
   if(CTX.geom.light) glEnable(GL_LIGHTING);
+  if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
   glEnable( GL_TEXTURE_2D );  
@@ -194,8 +208,8 @@ void Structural_BeamSection ::  GL_DrawBeam (double pinit[3], double dir[3], con
     {
       Surface *s;
       List_Read(surfaces,i,&s);
-      List_T *triangles = Tree2List(s->Simplexes);
-      //      printf("%g %g %d %d\n",xc,yc,List_Nbr(vertices),List_Nbr(triangles));
+      List_T *triangles = Tree2List(s->SimplexesBase);
+      //printf("%g %g %d %d\n",xc,yc,List_Nbr(vertices),List_Nbr(triangles));
       for(int j=0;j<List_Nbr(triangles);++j)
 	{
 	  Simplex *simp;
@@ -219,7 +233,7 @@ void Structural_BeamSection ::  GL_DrawBeam (double pinit[3], double dir[3], con
     {
       Curve *c;
       List_Read(curves,i,&c);
-      List_T *lines = Tree2List(c->Simplexes);
+      List_T *lines = Tree2List(c->SimplexesBase);
       //      printf("%g %g %d %d\n",xc,yc,List_Nbr(vertices),List_Nbr(triangles));
       for(int j=0;j<List_Nbr(lines);++j)
 	{
@@ -409,6 +423,13 @@ Structural_BeamSection * StructuralSolver :: GetBeamSection (const std::string &
     }
   return 0;
 }
+int  GetModel (const std::string & name)
+{
+  if (!strcmp(name.c_str(),"Beam"))return 1;
+  if (!strcmp(name.c_str(),"Bar"))return 2;
+  return 3;
+}
+
 Structural_Material StructuralSolver :: GetMaterial (const std::string & name) const 
 {
   std::list<struct Structural_Material > :: const_iterator it  = materials.begin();
@@ -425,15 +446,29 @@ Structural_Material StructuralSolver :: GetMaterial (const std::string & name) c
 }
 
 
-#define BEAM_SECTION_ 3
-#define BEAM_MATERIAL_ 4
-#define POINT_ALONG_ 0
-#define POINT_ACROSS_ 1
-#define POINT_AROUND_ 2
-#define ANGLE_ 3
-#define ALONG_ 0
-#define ACROSS_ 1
-#define AROUND_ 2
+#define BEAM_SECTION_ 6
+#define BEAM_MATERIAL_ 7
+#define BEAM_MODEL_ 8
+
+#define DIR1X_ 20
+#define DIR1Y_ 21
+#define DIR1Z_ 22
+#define DIR2X_ 23
+#define DIR2Y_ 24
+#define DIR2Z_ 25
+#define DIR1F_ 26
+#define DIR2F_ 27
+#define DIR3F_ 28
+#define DIR1M_ 29
+#define DIR2M_ 30
+#define DIR3M_ 31
+
+#define DIR1F__ 0
+#define DIR2F__ 1
+#define DIR3F__ 2
+#define DIR1M__ 3
+#define DIR2M__ 4
+#define DIR3M__ 5
 
 #ifdef HAVE_FLTK 
 void close_cb(Fl_Widget* w, void* data)
@@ -464,55 +499,102 @@ void StructuralSolver ::popupPropertiesForPhysicalEntity (int dim)
   }
   
   int width = 31 * fontsize;
-  int height = 5 * WB + 9 * BH;
+  int height = 5 * WB + 11 * BH;
 
-  _window = new Fl_Window(width, height, "Structural Solver");
+  _window = new Dialog_Window(width, height, "Structural Solver");
   _window->box(WINDOW_BOX);
   {
     Fl_Tabs *o = new Fl_Tabs(WB, WB, width - 2 * WB, height - 3 * WB - BH);
     // 0: 
     {
-      g[0] = new Fl_Group(WB, WB + BH, width - 2 * WB, height - 3 * WB - 2 * BH, "Physical Point");
+      g[0] = new Fl_Group(WB, WB + BH, width - 2 * WB, height - 3 * WB - 2 * BH, "Nodal Constraint");
       
-      static Fl_Menu_Item _type[] = {
+      static Fl_Menu_Item _typeF[] = {
 	{"Displacement fixed (mm)", 0, 0, 0},
 	{"Load fixed (kN)", 0, 0, 0},
 	{0}
       };
+      static Fl_Menu_Item _typeM[] = {
+	{"Rotation fixed (rad)", 0, 0, 0},
+	{"Moment fixed (kNmm)", 0, 0, 0},
+	{0}
+      };
 
-      _value[ANGLE_] = new Fl_Value_Input(2 * WB, 2 * WB + 1 * BH, IW, BH, "Angle of Rotation");
-      _value[ANGLE_]->value(90);
-      _value[ANGLE_]->align(FL_ALIGN_RIGHT);
 
-      _choice[POINT_ALONG_] = new Fl_Choice(2 * WB, 2 * WB + 2 * BH, IW, BH, "Along the Local Axis");
-      _choice[POINT_ALONG_]->menu(_type);
-      _choice[POINT_ALONG_]->align(FL_ALIGN_RIGHT);
-      
-      _value[ALONG_] = new Fl_Value_Input(2 * WB, 2 * WB + 3 * BH, IW, BH,"Value");
-      _value[ALONG_]->value(0.0);
-      _value[ALONG_]->align(FL_ALIGN_RIGHT);
-      
-      _choice[POINT_ACROSS_] = new Fl_Choice(2 * WB, 2 * WB + 4 * BH, IW, BH, "Across the Local Axis");
-      _choice[POINT_ACROSS_]->menu(_type);
-      _choice[POINT_ACROSS_]->align(FL_ALIGN_RIGHT);
-      
-      _value[ACROSS_] = new Fl_Value_Input(2 * WB, 2 * WB + 5 * BH, IW, BH,"Value");
-      _value[ACROSS_]->value(0.0);
-      _value[ACROSS_]->align(FL_ALIGN_RIGHT);
-      
-      
-      _choice[POINT_AROUND_] = new Fl_Choice(2 * WB, 2 * WB + 6 * BH, IW, BH, "Around Z axis");
-      _choice[POINT_AROUND_]->menu(_type);
-      _choice[POINT_AROUND_]->align(FL_ALIGN_RIGHT);
-      
-      _value[AROUND_] = new Fl_Value_Input(2 * WB, 2 * WB + 7 * BH, IW, BH,       "");
-      _value[AROUND_]->value(0.0);
-      _value[AROUND_]->align(FL_ALIGN_RIGHT);
+      _value[DIR1X_] = new Fl_Value_Input(2 * WB, 2 * WB + 1 * BH, IW/3, BH,       "");
+      _value[DIR1X_]->value(1.0);
+      _value[DIR1X_]->align(FL_ALIGN_RIGHT);
+      _value[DIR1Y_] = new Fl_Value_Input(2 * WB+IW/3, 2 * WB + 1* BH, IW/3, BH,       "");
+      _value[DIR1Y_]->value(0.0);
+      _value[DIR1Y_]->align(FL_ALIGN_RIGHT);
+      _value[DIR1Z_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 1 * BH, IW/3, BH, "Local x axe");
+      _value[DIR1Z_]->value(0.0);
+      _value[DIR1Z_]->align(FL_ALIGN_RIGHT);
+
+      _value[DIR2X_] = new Fl_Value_Input(2 * WB, 2 * WB + 2 * BH, IW/3, BH,       "");
+      _value[DIR2X_]->value(0.0);
+      _value[DIR2X_]->align(FL_ALIGN_RIGHT);
+      _value[DIR2Y_] = new Fl_Value_Input(2 * WB+IW/3, 2 * WB + 2* BH, IW/3, BH,       "");
+      _value[DIR2Y_]->value(1.0);
+      _value[DIR2Y_]->align(FL_ALIGN_RIGHT);
+      _value[DIR2Z_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 2 * BH, IW/3, BH, "Local y axe");
+      _value[DIR2Z_]->value(0.0);
+      _value[DIR2Z_]->align(FL_ALIGN_RIGHT);
+
+      _choice[DIR1F__] = new Fl_Choice(2 * WB, 2 * WB + 3 * BH, 2*IW/3, BH, "");
+      _choice[DIR1F__]->menu(_typeF);
+      _choice[DIR1F__]->align(FL_ALIGN_RIGHT);
+      _value[DIR1F_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 3 * BH, IW/3, BH,"Value along x");
+      _value[DIR1F_]->value(0.0);
+      _value[DIR1F_]->align(FL_ALIGN_RIGHT);
+
+      _choice[DIR1M__] = new Fl_Choice(2 * WB, 2 * WB + 4 * BH, 2*IW/3, BH, "");
+      _choice[DIR1M__]->menu(_typeM);
+      _choice[DIR1M__]->align(FL_ALIGN_RIGHT);
+      _value[DIR1M_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 4 * BH, IW/3, BH,"Value around x");
+      _value[DIR1M_]->value(0.0);
+      _value[DIR1M_]->align(FL_ALIGN_RIGHT);
+
+
+      _choice[DIR2F__] = new Fl_Choice(2 * WB, 2 * WB + 5 * BH, 2*IW/3, BH, "");
+      _choice[DIR2F__]->menu(_typeF);
+      _choice[DIR2F__]->align(FL_ALIGN_RIGHT);
+      _value[DIR2F_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 5 * BH, IW/3, BH,"Value along y");
+      _value[DIR2F_]->value(0.0);
+      _value[DIR2F_]->align(FL_ALIGN_RIGHT);
+
+      _choice[DIR2M__] = new Fl_Choice(2 * WB, 2 * WB + 6 * BH, 2*IW/3, BH, "");
+      _choice[DIR2M__]->menu(_typeM);
+      _choice[DIR2M__]->align(FL_ALIGN_RIGHT);
+      _value[DIR2M_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 6 * BH, IW/3, BH,"Value around y");
+      _value[DIR2M_]->value(0.0);
+      _value[DIR2M_]->align(FL_ALIGN_RIGHT);
+
+      _choice[DIR3F__] = new Fl_Choice(2 * WB, 2 * WB + 7 * BH, 2*IW/3, BH, "");
+      _choice[DIR3F__]->menu(_typeF);
+      _choice[DIR3F__]->align(FL_ALIGN_RIGHT);
+      _value[DIR3F_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 7 * BH, IW/3, BH,"Value along z");
+      _value[DIR3F_]->value(0.0);
+      _value[DIR3F_]->align(FL_ALIGN_RIGHT);
+
+      _choice[DIR3M__] = new Fl_Choice(2 * WB, 2 * WB + 8 * BH, 2*IW/3, BH, "");
+      _choice[DIR3M__]->menu(_typeM);
+      _choice[DIR3M__]->align(FL_ALIGN_RIGHT);
+      _value[DIR3M_] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 8 * BH, IW/3, BH,"Value around z");
+      _value[DIR3M_]->value(0.0);
+      _value[DIR3M_]->align(FL_ALIGN_RIGHT);
       g[0]->end();
     }
     // 2: Physical Line
     {
-      g[1] = new Fl_Group(WB, WB + BH, width - 2 * WB, height - 3 * WB - 2 * BH, "Physical Line");
+      g[1] = new Fl_Group(WB, WB + BH, width - 2 * WB, height - 3 * WB - 2 * BH, "Beam");
+
+      static Fl_Menu_Item _model[] = {
+	{"Beam", 0, 0, 0},
+	{"Bar", 0, 0, 0},
+	{0}
+      };
+
       {
 	_choice[BEAM_SECTION_] = new Fl_Choice(2 * WB, 2 * WB + 1 * BH, IW, BH, "Beam Section");      
 	std::list<struct Structural_BeamSection* > :: iterator it  = beam_sections.begin();
@@ -534,37 +616,45 @@ void StructuralSolver ::popupPropertiesForPhysicalEntity (int dim)
 	_choice[BEAM_MATERIAL_]->align(FL_ALIGN_RIGHT);
       }
 
-      _value[6] = new Fl_Value_Input(2 * WB, 2 * WB + 3 * BH, IW/2, BH,       "");
+      {
+	_choice[BEAM_MODEL_] = new Fl_Choice(2 * WB, 2 * WB + 3 * BH, IW, BH, "Kinematic Model");      
+	_choice[BEAM_MODEL_]->menu(_model);
+	_choice[BEAM_MODEL_]->align(FL_ALIGN_RIGHT);
+      }
+
+      _value[6] = new Fl_Value_Input(2 * WB, 2 * WB + 4 * BH, IW/2, BH,       "");
       _value[6]->value(0.0);
       _value[6]->align(FL_ALIGN_RIGHT);
-      _value[7] = new Fl_Value_Input(2 * WB+IW/2, 2 * WB + 3 * BH, IW/2, BH, "X-Load");
+      _value[7] = new Fl_Value_Input(2 * WB+IW/2, 2 * WB + 4 * BH, IW/2, BH, "X-Load");
       _value[7]->value(0.0);
       _value[7]->align(FL_ALIGN_RIGHT);
-      _value[8] = new Fl_Value_Input(2 * WB, 2 * WB + 4 * BH, IW/2, BH,       "");
+      _value[8] = new Fl_Value_Input(2 * WB, 2 * WB + 5 * BH, IW/2, BH,       "");
       _value[8]->value(0.0);
       _value[8]->align(FL_ALIGN_RIGHT);
-      _value[9] = new Fl_Value_Input(2 * WB+IW/2, 2 * WB + 4 * BH, IW/2, BH, "Y-Load");
+      _value[9] = new Fl_Value_Input(2 * WB+IW/2, 2 * WB + 5 * BH, IW/2, BH, "Y-Load");
       _value[9]->value(0.0);
       _value[9]->align(FL_ALIGN_RIGHT);
-      _value[10] = new Fl_Value_Input(2 * WB, 2 * WB + 5 * BH, IW/2, BH,       "");
+      _value[10] = new Fl_Value_Input(2 * WB, 2 * WB + 6 * BH, IW/2, BH,       "");
       _value[10]->value(0.0);
       _value[10]->align(FL_ALIGN_RIGHT);
-      _value[11] = new Fl_Value_Input(2 * WB+IW/2, 2 * WB + 5 * BH, IW/2, BH, "Z-Load");
+      _value[11] = new Fl_Value_Input(2 * WB+IW/2, 2 * WB + 6 * BH, IW/2, BH, "Z-Load");
       _value[11]->value(0.0);
       _value[11]->align(FL_ALIGN_RIGHT);
       
-      _value[12] = new Fl_Value_Input(2 * WB, 2 * WB + 6 * BH, IW/3, BH,       "");
+      _value[12] = new Fl_Value_Input(2 * WB, 2 * WB + 7 * BH, IW/3, BH,       "");
       _value[12]->value(0.0);
       _value[12]->align(FL_ALIGN_RIGHT);
-      _value[13] = new Fl_Value_Input(2 * WB+IW/3, 2 * WB + 6 * BH, IW/3, BH,       "");
+      _value[13] = new Fl_Value_Input(2 * WB+IW/3, 2 * WB + 7* BH, IW/3, BH,       "");
       _value[13]->value(0.0);
       _value[13]->align(FL_ALIGN_RIGHT);
-      _value[14] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 6 * BH, IW/3, BH, "Y-Direction");
+      _value[14] = new Fl_Value_Input(2 * WB+2*IW/3, 2 * WB + 7 * BH, IW/3, BH, "Y-Direction");
       _value[14]->value(1.0);
       _value[14]->align(FL_ALIGN_RIGHT);
       
       g[1]->end();
     }
+
+
     o->end();
 
     {
@@ -584,10 +674,24 @@ void StructuralSolver :: addPhysicalLine (int id)
   PhysicalLineInfo info;  
   info.section = std::string(_choice[BEAM_SECTION_] ->mvalue()->text);  
   info.material= std::string(_choice[BEAM_MATERIAL_]->mvalue()->text);
+  info.model= std::string(_choice[BEAM_MODEL_]->mvalue()->text);
   info.dirz[0]  = _value[12]->value();
   info.dirz[1]  = _value[13]->value();
   info.dirz[2]  = _value[14]->value();
+  info.fx1      = _value[6]->value();
+  info.fx2      = _value[7]->value();
+  info.fy1      = _value[8]->value();
+  info.fy2      = _value[8]->value();
+  info.fz1      = _value[10]->value();
+  info.fz2      = _value[11]->value();
   lines[id] = info;
+
+  double f1 = sqrt (info.fx1*info.fx1+info.fy1*info.fy1+info.fz1*info.fz1);
+  double f2 = sqrt (info.fx2*info.fx2+info.fy2*info.fy2+info.fz2*info.fz2);
+
+  MAX_FORCE = (MAX_FORCE>f1)?MAX_FORCE:f1;
+  MAX_FORCE = (MAX_FORCE>f2)?MAX_FORCE:f2;
+
 #endif  
 }
 
@@ -596,21 +700,38 @@ void StructuralSolver :: addPhysicalPoint (int id)
 #ifdef HAVE_FLTK 
   PhysicalPointInfo info;
 
-  info.angle = _value[ANGLE_]->value();
+  info.essential_W [0] = _choice[DIR1F__] ->value();
+  info.essential_W [1] = _choice[DIR2F__] ->value();
+  info.essential_W [2] = _choice[DIR3F__] ->value();
+  info.essential_Theta [0] = _choice[DIR1M__] ->value();
+  info.essential_Theta [1] = _choice[DIR2M__] ->value();
+  info.essential_Theta [2] = _choice[DIR3M__] ->value();
 
-  info.disp[0] = _choice[POINT_ALONG_] ->value();
-  info.disp[1] = _choice[POINT_ACROSS_] ->value();
-  info.disp[2] = _choice[POINT_AROUND_] ->value();
-  info.val[0] = _value[ALONG_]->value();
-  info.val[1] = _value[ACROSS_]->value();
-  info.val[2] = _value[AROUND_]->value();
+  info.dirx[0] = _value[DIR1X_] -> value ();
+  info.dirx[1] = _value[DIR1Y_] -> value ();
+  info.dirx[2] = _value[DIR1Z_] -> value ();
 
-  if (info.disp[0] == 0)
-    MAX_FORCE = (MAX_FORCE>info.val[0])?MAX_FORCE:info.val[0];
-  if (info.disp[1] == 0)
-    MAX_FORCE = (MAX_FORCE>info.val[1])?MAX_FORCE:info.val[1];
-  if (info.disp[2] == 0)
-    MAX_FORCE = (MAX_FORCE>info.val[2])?MAX_FORCE:info.val[2];
+  info.diry[0] = _value[DIR2X_] -> value ();
+  info.diry[1] = _value[DIR2Y_] -> value ();
+  info.diry[2] = _value[DIR2Z_] -> value ();
+
+  prodve (info.dirx,info.diry,info.dirz);
+
+
+  info.values_W[0] = _value[DIR1F_] -> value ();
+  info.values_W[1] = _value[DIR2F_] -> value ();
+  info.values_W[2] = _value[DIR3F_] -> value ();
+
+  info.values_Theta[0] = _value[DIR1M_] -> value ();
+  info.values_Theta[1] = _value[DIR2M_] -> value ();
+  info.values_Theta[2] = _value[DIR3M_] -> value ();
+
+  if (info.essential_W[0] == 1)
+    MAX_FORCE = (MAX_FORCE>fabs(info.values_W[0]))?MAX_FORCE:fabs(info.values_W[0]);
+  if (info.essential_W[1] == 1)
+    MAX_FORCE = (MAX_FORCE>fabs(info.values_W[1]))?MAX_FORCE:fabs(info.values_W[1]);
+  if (info.essential_W[2] == 1)
+    MAX_FORCE = (MAX_FORCE>fabs(info.values_W[2]))?MAX_FORCE:fabs(info.values_W[2]);
 
   points[id] = info;
 #endif
@@ -644,8 +765,8 @@ void StructuralSolver :: writeSolverFile ( const char *geom_file ) const
 	int id = (*it).first;
 	if (getPhysical ( id , 1 ))
 	  {
-	    fprintf(f,"BEAM PHYSICAL %d SECTION %s MATERIAL %s LOADS %g %g %g %g %g %g %g %g %g \n",
-		    id,i.section.c_str(),i.material.c_str(),i.fx1,i.fy1,i.fx2,i.fy2,i.fz1,i.fz2,i.dirz[0],i.dirz[1],i.dirz[2]);
+	    fprintf(f,"BEAM PHYSICAL %d SECTION %s MATERIAL %s MODEL %s LOADS %g %g %g %g %g %g %g %g %g \n",
+		    id,i.section.c_str(),i.material.c_str(),i.model.c_str(),i.fx1,i.fy1,i.fx2,i.fy2,i.fz1,i.fz2,i.dirz[0],i.dirz[1],i.dirz[2]);
 	  }
       }
   }
@@ -657,8 +778,16 @@ void StructuralSolver :: writeSolverFile ( const char *geom_file ) const
 	const PhysicalPointInfo &i = (*it).second;
 	int id = (*it).first;
 	if (getPhysical ( id , 0 ))
-	  {
-	    fprintf(f,"NODE %d %g %d %g %d %g %d %g \n",id,i.angle,i.disp[0],i.val[0],i.disp[1],i.val[1],i.disp[2],i.val[2]);
+	{
+	    fprintf(f,"NODE %d %g %g %g %g %g %g %g %g %g %d %d %d %d %d %d %g %g %g %g %g %g\n",
+		    id,
+		    i.dirx[0],i.dirx[1],i.dirx[2],
+		    i.diry[0],i.diry[1],i.diry[2],
+		    i.dirz[0],i.dirz[1],i.dirz[2],
+		    i.essential_W[0],i.essential_W[1],i.essential_W[2],
+		    i.essential_Theta[0],i.essential_Theta[1],i.essential_Theta[2],
+		    i.values_W[0],i.values_W[1],i.values_W[2],
+		    i.values_Theta[0],i.values_Theta[1],i.values_Theta[2]);
 	  }
       }
   }
@@ -676,10 +805,12 @@ void StructuralSolver :: writeSolverFile ( const char *geom_file ) const
 	  {
 	    Structural_BeamSection* bs = GetBeamSection (i.section);
 	    Structural_Material    mt = GetMaterial    (i.material);
-	    fprintf(f,"111 %d %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g\n",
+	    int model = GetModel (i.model);
+	    fprintf(f,"111 %d %d %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g %g\n",
 		    id,                             // physical id
+		    model,                             // model (1=beam, 2=bar)
 		    bs->area, bs->Iy, bs->Iz, bs->J,// section info
-		    mt.par[0],mt.par[1],mt.par[2],// material info
+		    mt.par[0],mt.par[1],mt.par[2],mt.par[3],// material info E, sigma_e, rho, nu
 		    i.fx1,i.fy1,i.fx2,i.fy2,i.fz1,i.fz2,// lineic loads
 		    i.dirz[0],i.dirz[1],i.dirz[2]); // direction
 	  }
@@ -694,7 +825,15 @@ void StructuralSolver :: writeSolverFile ( const char *geom_file ) const
 	int id = (*it).first;
 	if (getPhysical ( id , 0 ))
 	  {
-	    fprintf(f,"222 %d %g %d %g %d %g %d %g \n",id,i.angle,i.disp[0],i.val[0],i.disp[1],i.val[1],i.disp[2],i.val[2]);
+	    fprintf(f,"222 %d %g %g %g %g %g %g %g %g %g %d %d %d %d %d %d %g %g %g %g %g %g\n",
+		    id,
+		    i.dirx[0],i.dirx[1],i.dirx[2],
+		    i.diry[0],i.diry[1],i.diry[2],
+		    i.dirz[0],i.dirz[1],i.dirz[2],
+		    i.essential_W[0],i.essential_W[1],i.essential_W[2],
+		    i.essential_Theta[0],i.essential_Theta[1],i.essential_Theta[2],
+		    i.values_W[0],i.values_W[1],i.values_W[2],
+		    i.values_Theta[0],i.values_Theta[1],i.values_Theta[2]);
 	  }
       }
   }
@@ -703,7 +842,7 @@ void StructuralSolver :: writeSolverFile ( const char *geom_file ) const
 
 void StructuralSolver :: readSolverFile ( const char *geom_file ) 
 {
-  char name[256],line[256],a1[24],a2[24],a3[24],a4[240],a5[24],a6[240],a7[24];
+  char name[256],line[256],a1[24],a2[24],a3[24],a4[240],a5[24],a6[240],a7[24],a8[24],a9[24];
   sprintf(name,"%s.str",geom_file);
   FILE *f = fopen(name,"r");  
   if (!f)return;
@@ -716,26 +855,43 @@ void StructuralSolver :: readSolverFile ( const char *geom_file )
 	{
 	  int id;
 	  PhysicalLineInfo info;
-	  sscanf(line,"%s %s %d %s %s %s %s %s %lf %lf %lf %lf %lf %lf %lf %lf %lf \n",
-		 a1,a2,&id,a3,a4,a5,a6,a7,&info.fx1,&info.fy1,&info.fx2,&info.fy2,&info.fz1,&info.fz2,&info.dirz[0],&info.dirz[1],&info.dirz[2]);
+	  sscanf(line,"%s %s %d %s %s %s %s %s %s %s %lf %lf %lf %lf %lf %lf %lf %lf %lf \n",
+		 a1,a2,&id,a3,a4,a5,a6,a7,a8,a9,&info.fx1,&info.fy1,&info.fx2,&info.fy2,&info.fz1,&info.fz2,&info.dirz[0],&info.dirz[1],&info.dirz[2]);
+	  info.model = std::string(a8);
 	  info.material = std::string(a6);
 	  info.section  = std::string(a4);
-	  lines[id] = info;
+	  lines[id] = info;       
+
+	  double f1 = sqrt (info.fx1*info.fx1+info.fy1*info.fy1+info.fz1*info.fz1);
+	  double f2 = sqrt (info.fx2*info.fx2+info.fy2*info.fy2+info.fz2*info.fz2);
+
+	  MAX_FORCE = (MAX_FORCE>f1)?MAX_FORCE:f1;
+	  MAX_FORCE = (MAX_FORCE>f2)?MAX_FORCE:f2;
+
 	}
       if (!strcmp(name,"NODE"))
 	{
 	  int id;
-	  PhysicalPointInfo info;
-	  sscanf(line,"%s %d %lf %d %lf %d %lf %d %lf \n",a1,&id,
-		 &info.angle,
-		 &info.disp[0],&info.val[0],
-		 &info.disp[1],&info.val[1],
-		 &info.disp[2],&info.val[2]);
-	  points[id] = info;
-	  if (info.disp[0] == 1)
-	    MAX_FORCE = (MAX_FORCE>fabs(info.val[0]))?MAX_FORCE:fabs(info.val[0]);
-	  if (info.disp[1] == 1)
-	    MAX_FORCE = (MAX_FORCE>fabs(info.val[1]))?MAX_FORCE:fabs(info.val[1]);
+	  PhysicalPointInfo i;
+	  sscanf(line,"%s %d %lf %lf %lf %lf %lf %lf %lf %lf %lf %d %d %d %d %d %d %lf %lf %lf %lf %lf %lf\n",
+		 a1, 
+		 &id,
+		 &i.dirx[0],&i.dirx[1],&i.dirx[2],
+		 &i.diry[0],&i.diry[1],&i.diry[2],
+		 &i.dirz[0],&i.dirz[1],&i.dirz[2],
+		 &i.essential_W[0],&i.essential_W[1],&i.essential_W[2],
+		 &i.essential_Theta[0],&i.essential_Theta[1],&i.essential_Theta[2],
+		 &i.values_W[0],&i.values_W[1],&i.values_W[2],
+		 &i.values_Theta[0],&i.values_Theta[1],&i.values_Theta[2]);
+	  points[id] = i;
+	  if (i.essential_W[0] == 1)
+	      MAX_FORCE = (MAX_FORCE>fabs(i.values_W[0]))?MAX_FORCE:fabs(i.values_W[0]);
+	  if (i.essential_W[1] == 1)
+	      MAX_FORCE = (MAX_FORCE>fabs(i.values_W[1]))?MAX_FORCE:fabs(i.values_W[1]);
+	  if (i.essential_W[2] == 1)
+	      MAX_FORCE = (MAX_FORCE>fabs(i.values_W[2]))?MAX_FORCE:fabs(i.values_W[2]);
+
+	  printf("typ %d val = %g\n",i.essential_W[0],i.values_W[0]);
 	}
       if (feof(f) )break;
     }
@@ -744,39 +900,35 @@ void StructuralSolver :: readSolverFile ( const char *geom_file )
 }
 
 
-void Draw_Kinematic_Constraint ( const int type [3], 
+void Draw_Kinematic_Constraint ( const int type_W     [3], 
+				 const int type_Theta [3], 
 				 const double size, 
 				 const double pos[3], 
-				 double dir[3])
+				 double dir[3],
+				 double dir2[3])
 {
 #ifdef HAVE_FLTK
 
-  // presently, it's only 2D , 1st and second component are for dir and dir2 
-  // and third one is for rotation around z
-
-  double dir2[3];
-  double ez[3]={0,0,1};
-  prodve(dir,ez,dir2);
+// 3D VERSION 
 
   double size_feature = size*0.5;
   double width_ground = size*0.25;
 
-  if(!type[0] && !type[1] && !type[2]) // clamped
+  if(!type_W[0] && !type_W[1] && !type_W[2] && !type_Theta[0] && !type_Theta[1] && !type_Theta[2] ) // clamped
     {
       size_feature = 0.0;
       width_ground = 0.5*size;
     }
 
-  if(type[0] && type[1] && type[2]) // free
+  if(type_W[0] && type_W[1] && type_W[2] && type_Theta[0] && type_Theta[1] && type_Theta[2]) // free
     {
       return;
     }
 
   if(CTX.geom.light) glEnable(GL_LIGHTING);
+  if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
 
-  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
-  glEnable(GL_POLYGON_OFFSET_FILL);
-  glColor3f    (0.8,0.8,0.8);
+  glColor3f    (0.9,0.9,0.9);
   
   glBegin(GL_QUADS);
   glVertex3d ( pos[0] - size_feature * dir [0] - size * 0.5 * dir2 [0],
@@ -798,7 +950,7 @@ void Draw_Kinematic_Constraint ( const int type [3],
 
   glLineWidth(2);
 
-  if (!type[0])
+  if (!type_W[0])
     {
       glBegin(GL_LINES);
       glVertex3d ( pos[0] - size_feature * dir [0] - size * 0.5 * dir2 [0],
@@ -810,15 +962,17 @@ void Draw_Kinematic_Constraint ( const int type [3],
       glEnd();
     }
   
-  if(type[0] || type[1] || type[2]) // everything except clamped
+  if(type_Theta[0] || type_Theta[1] || type_Theta[2]) // everything except clamped
     {
       double radius_circle = size_feature * 0.5;
       double radius_circle2 = size_feature * 0.5;
 
-      if (!type[1]) radius_circle = 0;
-      if (!type[2]) radius_circle2 = 0;
+      // NO FREE SLIP ON THE WALL
+      if (!type_W[1] && !type_W[2]) radius_circle = 0;
+      // NO FREE ROTATION ON THE WALL
+      if (!type_Theta[0] && !type_Theta[1] && !type_Theta[2]) radius_circle2 = 0;
 
-      if (!type[0])
+      if (!type_W[0])
 	{
 	  glBegin(GL_LINES);
 	  
@@ -836,14 +990,14 @@ void Draw_Kinematic_Constraint ( const int type [3],
 	  glEnd();
 	}
 
-      if (!type[0]){
+      if (!type_W[0]){
 	glColor4f(1,1,1,1);
 	Draw_Disk (radius_circle2*.5*.8, 0.00, pos[0], pos[1], pos[2]+0.01*radius_circle2,CTX.geom.light); 
 	glColor4f(0,0,0,1);
 	Draw_Disk (radius_circle2*.5, 0.8, pos[0], pos[1], pos[2],CTX.geom.light); 
       }
 
-      if (!type[1] || !type[0]){
+      if (!type_W[1] || !type_W[0]){
 	glBegin(GL_LINES);
 	glVertex3d ( pos[0] - (size_feature-radius_circle) * dir [0] - size * 0.5 * dir2 [0],
 		     pos[1] - (size_feature-radius_circle) * dir [1] - size * 0.5 * dir2 [1],
@@ -912,40 +1066,47 @@ bool StructuralSolver :: GL_enhancePoint ( Vertex *v)
 	  if (it !=points.end())
 	    {	
     
-	      double angle = 3.1415926*it->second.angle/180;
-	      
-	      double size = 0.05*(CTX.max[0]-CTX.min[0]);
-	      double dir[3] = {cos(angle),sin(angle),0};
-	      double dir2[3];
-	      double Z[3]={0,0,1};
-	      prodve(dir,Z,dir2);
+	      double size = 0.1*(CTX.max[0]-CTX.min[0]);
+	      double dir[3] = {it->second.dirx[0],it->second.dirx[1],it->second.dirx[2]};
+	      double dir2[3] = {it->second.diry[0],it->second.diry[1],it->second.diry[2]};
 	      norme(dir);
+	      norme(dir2);
+	      double dir3[3];
+	      prodve(dir,dir2,dir3);
 	      double pos[3] = {v->Pos.X,v->Pos.Y,v->Pos.Z};
-	      Draw_Kinematic_Constraint (it->second.disp,size,pos,dir);
+	      Draw_Kinematic_Constraint (it->second.essential_W,it->second.essential_Theta,size,pos,dir,dir2);
 
 	      double dv[3] = {0,0,0};		  
 	      
-	      if (it->second.disp[0] == 1)
+	      if (it->second.essential_W[0] == 1)
 		{
-		  dv[0] += dir[0] * it->second.val[0];
-		  dv[1] += dir[1] * it->second.val[0];
+		  dv[0] += dir[0] * it->second.values_W[0];
+		  dv[1] += dir[1] * it->second.values_W[0];
+		  dv[2] += dir[2] * it->second.values_W[0];
 		}
-	      if (it->second.disp[1] == 1)
+	      if (it->second.essential_W[1] == 1)
 		{
-		  dv[0] += dir2[0] * it->second.val[1];
-		  dv[1] += dir2[1] * it->second.val[1];
+		  dv[0] += dir2[0] * it->second.values_W[1];
+		  dv[1] += dir2[1] * it->second.values_W[1];
+		  dv[2] += dir2[2] * it->second.values_W[1];
+		}
+	      if (it->second.essential_W[2] == 1)
+		{
+		  dv[0] += dir3[0] * it->second.values_W[2];
+		  dv[1] += dir3[1] * it->second.values_W[2];
+		  dv[2] += dir3[2] * it->second.values_W[2];
 		}
 
 	      const double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
 	      const double l = sqrt (dv[0]*dv[0]+dv[1]*dv[1]);
-	      const double kk = (CTX.max[0]-CTX.min[0])*.1 / (MAX_FORCE);
+	      const double kk = (CTX.max[0]-CTX.min[0])*.2 / (MAX_FORCE);
 	      if (l != 0.0)
 		{
 		  glColor4ubv((GLubyte *) & CTX.color.text);
 		  Draw_Vector (CTX.vector_type,  0, CTX.arrow_rel_head_radius, 
 			       CTX.arrow_rel_stem_length, 0.5*CTX.arrow_rel_stem_radius,
 			       v->Pos.X-dv[0]*kk, v->Pos.Y-dv[1]*kk, v->Pos.Z-dv[2]*kk,
-			       dv[0]*kk, dv[1]*kk, dv[2]*kk, NULL, CTX.geom.light);
+			       dv[0]*kk, dv[1]*kk, dv[2]*kk, CTX.geom.light);
 		  sprintf(Num, "%g kN", l);
 		  glRasterPos3d((v->Pos.X + dv[0])+ offset / CTX.s[0],
 				(v->Pos.Y + dv[1])+ offset / CTX.s[1],
@@ -965,24 +1126,67 @@ bool StructuralSolver :: GL_enhancePoint ( Vertex *v)
 bool StructuralSolver :: GL_enhanceLine ( int CurveId, Vertex *v1, Vertex *v2) 
 {
 #ifdef HAVE_FLTK  
-  //  return true;
   PhysicalGroup *p;
+  //  printf("%d physical groups \n",List_Nbr(THEM->PhysicalGroups));
   for(int i = 0; i < List_Nbr(THEM->PhysicalGroups); i++) 
     { 
       List_Read(THEM->PhysicalGroups, i, &p);
       if(p->Typ == MSH_PHYSICAL_LINE) {
+	//	printf("physical line found ... curve Id is %d\n",CurveId);
 	if(List_Search(p->Entities, &CurveId, fcmp_absint)) { 
 	  std::map<int,struct PhysicalLineInfo>::const_iterator it = lines.find(p->Num);
+	  //	  printf("curve found in physical line %d \n",p->Num);
 	  if (it !=lines.end())
 	    {	      
 	      double pinit [3] = {v1->Pos.X,v1->Pos.Y,v1->Pos.Z};
 	      double dir [3] = {v2->Pos.X-v1->Pos.X,v2->Pos.Y-v1->Pos.Y,v2->Pos.Z-v1->Pos.Z};
 	      Structural_BeamSection *section =  GetBeamSection (it->second.section);
-
 	      //	      printf ("%g,%g,%g\n",it->second.dirz[0],it->second.dirz[1],it->second.dirz[2]);
 
+	      const int nbArrow = 10;
+	      const double kk = (CTX.max[0]-CTX.min[0])*.1 / (MAX_FORCE);
+	      glColor4ubv((GLubyte *) & CTX.color.text);
+	      double X1=0.,Y1=0.,Z1=0.,X2=0.,Y2=0.,Z2=0.;
+	      for (int iArrow = 0 ; iArrow < nbArrow ; iArrow++)
+		{
+		  
+		  const double xi = (double)iArrow/(double)(nbArrow-1);
+		  const double X = v1->Pos.X + xi * ( v2->Pos.X - v1->Pos.X );
+		  const double Y = v1->Pos.Y + xi * ( v2->Pos.Y - v1->Pos.Y );
+		  const double Z = v1->Pos.Z + xi * ( v2->Pos.Z - v1->Pos.Z );
+		  double dv[3] = 
+		    {
+		      it->second.fx1 + xi* (it->second.fx2 - it->second.fx1), 
+		      it->second.fy1 + xi* (it->second.fy2 - it->second.fy1), 
+		      it->second.fz1 + xi* (it->second.fz2 - it->second.fz1) 
+		    };		  
+		  Draw_Vector (CTX.vector_type,  0, CTX.arrow_rel_head_radius, 
+			       CTX.arrow_rel_stem_length, 0.5*CTX.arrow_rel_stem_radius,
+			       X-dv[0]*kk, Y-dv[1]*kk, Z-dv[2]*kk,
+			       dv[0]*kk, dv[1]*kk, dv[2]*kk, CTX.geom.light);		  
+		  
+		  if (iArrow==0)
+		    {
+		      X1 =  X-dv[0]*kk;
+		      Y1 =  Y-dv[1]*kk;
+		      Z1 =  Z-dv[2]*kk;
+		    }
+		  if (iArrow==nbArrow-1)
+		    {
+		      X2 =  X-dv[0]*kk;
+		      Y2 =  Y-dv[1]*kk;
+		      Z2 =  Z-dv[2]*kk;
+		    }
+		}
+
+	      glBegin(GL_LINES);
+	      glVertex3d ( X1,Y1,Z1);
+	      glVertex3d ( X2,Y2,Z2);
+	      glEnd  ();
+	      
 	      if (section)
 		{
+		  //		  printf("section found\n");
 		  section -> GL_DrawBeam (pinit,dir,it->second.dirz,textures[it->second.material]);
 		  return true;
 		}
