@@ -1,7 +1,7 @@
 #ifndef _TRANSFORM_H_
-#define _TRANSFORM_H
+#define _TRANSFORM_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
