@@ -1,6 +1,6 @@
-// $Id: Triangulate.cpp,v 1.24 2004/07/05 15:20:06 geuzaine Exp $
+// $Id: Triangulate.cpp,v 1.28 2005/03/02 08:14:29 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -62,11 +62,11 @@ void GMSH_TriangulatePlugin::getInfos(char *author, char *copyright,
   strcpy(author, "C. Geuzaine (geuz@geuz.org)");
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
-         "Plugin(Triangulate) triangulates the points\n"
-	 "in the view `iView', assuming that all the\n"
-         "points belong to a surface that can be\n"
-	 "univoquely projected into a plane. If `iView'\n"
-	 "< 0, the plugin is run on the current view.\n"
+         "Plugin(Triangulate) triangulates the points in the\n"
+	 "view `iView', assuming that all the points belong\n"
+         "to a surface that can be projected one-to-one\n"
+	 "onto a plane. If `iView' < 0, the plugin is run on\n"
+	 "the current view.\n"
 	 "\n"
 	 "Plugin(Triangulate) creates one new view.\n");
 }
@@ -219,8 +219,8 @@ Post_View *GMSH_TriangulatePlugin::execute(Post_View * v)
     return v;
   }
 
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
   Post_View *v2 = BeginView(1);
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
 
   Triangulate(v1->NbSP, v1->SP, &v2->NbST, v2->ST, v1->NbTimeStep, 1);
   Triangulate(v1->NbVP, v1->VP, &v2->NbVT, v2->VT, v1->NbTimeStep, 3);
